#ifndef TEMPLAND_TOOLKIT_BUILD
#define TEMPLAND_TOOLKIT_BUILD 1
#endif

#include "TableauLayout.h"
#include "TableauDialog.h"

#if TEMPLAND_TOOLKIT_BUILD
#include "../../Utilities/Logger.h"
#include "../../Utilities/Writer.h"
#endif

#include "ViewTree/SurfaceItem.h"
#include "ViewTree/CurveItem.h"
#include "ViewTree/SliceItem.h"
#include "ViewTree/VolumeItem.h"
#include "ViewTree/GridItem.h"
#include "ViewTree/RefPlanesItem.h"
#include "ViewTree/SemiLandmarksTopItem.h"
#include "ViewTree/SemiLandmarkItem.h"

#include <QMessageBox>
#include <QColorDialog>
#include <QFileDialog>
#include <QInputDialog>
#include <QSettings>

#include "ew/view3_surface.h"
#include "ew/surface3.h"
#include "ew/form3_ioerror.h"
#include "ew/surface3_ioerror.h"
#include "ew/dataflow_surface3_explicit.h"


unsigned char* chooseColor(const unsigned char* oldCol)
{
    unsigned char* col = 0;
    QColor result;
    result.setRgb(oldCol[0], oldCol[1], oldCol[2]);
    result = QColorDialog::getColor(result);
    if(result.isValid())
    {
        int ir, ig, ib;
        result.getRgb(&ir, &ig, &ib);
        col = new unsigned char[3];
        col[0] = (unsigned char)ir;
        col[1] = (unsigned char)ig;
        col[2] = (unsigned char)ib;
    }
    return col;
}

#if TEMPLAND_TOOLKIT_BUILD
TableauLayout::TableauLayout(QWidget *parent, QGLWidget* share2) : QWidget(parent), m_tps(0), m_dig3(&m_ewNetwork), m_newTableau(false),
                                                m_savedAtleastOnce(false), m_treesFull(false), m_tableauSaved(true), m_templateSaved(true),
                                                m_targetSaved(true), m_templateFile(""), m_targetFile(""), m_tableauFile(""), m_regular( false ), m_selectPointsInViews( true ), m_templateTopSurface(0), m_targetTopSurface(0)
#else
TableauLayout::TableauLayout(QWidget *parent, QGLWidget* share2) : QWidget(parent), m_dig3(&m_ewNetwork), m_newTableau(false),
                                                m_savedAtleastOnce(false), m_treesFull(false), m_tableauSaved(true), m_templateSaved(true),
                                                m_targetSaved(true), m_templateFile(""), m_targetFile(""), m_tableauFile(""), m_regular( false ), m_selectPointsInViews( true ), m_templateTopSurface(0), m_targetTopSurface(0)
#endif
{
	setupUi(this, &m_ewNetwork, share2);

    m_templateView->setDig3View(new EW_Dig3_View(&m_dig3, 0, 0, m_templateView));
    m_templateSliceView->setDig3View(new EW_Dig3_View(&m_dig3, 0, 1, m_templateSliceView));
	m_targetView->setDig3View(new EW_Dig3_View(&m_dig3, 1, 0, m_targetView));
    m_targetSliceView->setDig3View(new EW_Dig3_View(&m_dig3, 1, 1, m_targetSliceView));

    EW_Transform3 trm;
    trm.set_to_identity();

    for(int i=0; i<m_dig3.get_n_views(); ++i)
        m_dig3.get_views()[i]->set_view_mapping(&trm);

    unsigned char col[3] = {0, 0, 127};
    m_templateView->getDig3View()->get_slice_items()[1]->set_color(col);
    m_targetView->getDig3View()->get_slice_items()[3]->set_color(col);
    m_templateSliceView->getDig3View()->get_slice_items()[1]->set_color(col);
    m_targetSliceView->getDig3View()->get_slice_items()[3]->set_color(col);

    m_templateFormItem = new FormItem(m_templateItemTree, false );
    m_templateFormItem->setText(0, "Form ()");
    m_targetFormItem = new FormItem(m_targetItemTree, true);
    m_targetFormItem->setText(0, "Form ()");

    connect(m_frameSBox, SIGNAL(valueChanged(int)), this, SLOT(moveToFrame(int)));
    connect(m_frameSlider, SIGNAL(valueChanged(int)), this, SLOT(moveToFrame(int)));

    connect(m_targetItemTree, SIGNAL(viewItemChanged(ViewTree*, QTreeWidgetItem*, int)),
            this, SLOT(applyItemChange(ViewTree*, QTreeWidgetItem*, int)));
    connect(m_templateItemTree, SIGNAL(viewItemChanged(ViewTree*, QTreeWidgetItem*, int)),
            this, SLOT(applyItemChange(ViewTree*, QTreeWidgetItem*, int)));
    connect(m_targetItemTree, SIGNAL(viewItemSelected(ViewTree*, QTreeWidgetItem*)),
            this, SLOT(highlightItem(ViewTree*, QTreeWidgetItem*)));
    connect(m_templateItemTree, SIGNAL(viewItemSelected(ViewTree*, QTreeWidgetItem*)),
            this, SLOT(highlightItem(ViewTree*, QTreeWidgetItem*)));

    m_filmGrpBox->setVisible(false);
    m_srcGrpBox->setVisible(false);
    m_templateItemTree->setVisible(false);

    m_templateTopLandmarks = 0;
    m_targetTopLandmarks = 0;

    fillViewTrees(m_templateFormItem);
    fillViewTrees(m_targetFormItem);

    connect( m_targetView, SIGNAL(landmarkSelected(const QString&, int, int, int)), this, SLOT(landmarkSelected(const QString&, int, int, int)) );
    connect( m_targetSliceView, SIGNAL(landmarkSelected(const QString&, int, int, int)), this, SLOT(landmarkSelected(const QString&, int, int, int)) );
    connect( m_templateView, SIGNAL(landmarkSelected(const QString&, int, int, int)), this, SLOT(landmarkSelected(const QString&, int, int, int)) );
    connect( m_templateSliceView, SIGNAL(landmarkSelected(const QString&, int, int, int)), this, SLOT(landmarkSelected(const QString&, int, int, int)) );

    connect( m_targetView, SIGNAL(adjustLandmarkState()), this, SLOT(adjustLandmarkState()) );
    connect( m_targetSliceView, SIGNAL(adjustLandmarkState()), this, SLOT(adjustLandmarkState()) );

    connect( m_targetView, SIGNAL(landmarkMoved(const QString&, int)), this, SLOT(landmarkMoved(const QString&, int)) );
    connect( m_targetSliceView, SIGNAL(landmarkMoved(const QString&, int)), this, SLOT(landmarkMoved(const QString&, int)) );

    m_targetView->templateLandmarkTopItem( m_templateTopLandmarks );
    m_targetSliceView->templateLandmarkTopItem( m_templateTopLandmarks );
    m_targetView->targetLandmarkTopItem( m_targetTopLandmarks );
    m_targetSliceView->targetLandmarkTopItem( m_targetTopLandmarks );

    m_templateView->IsTemplate( true );
    m_templateSliceView->IsTemplate( true );

    //m_filmstripAdd->setDisabled( true );
    //m_filmstripReplace->setDisabled( true );
    //m_filmstripDelete->setDisabled( true );
    emit appendForm( false );
    emit replaceForm( false );
    emit deleteForm( false );
    emit swapPrevForm( false );
    emit swapNextForm( false );
    m_tblStart->setDisabled( true );
    m_tblEnd->setDisabled( true );
    m_tblLeftE->setDisabled( true );
    m_tblLeft->setDisabled( true );
    m_tblRight->setDisabled( true );
    m_tblListRightE->setDisabled( true );

    m_filmFirstFrame->setDisabled( true );
    m_filmPlayBackward->setDisabled( true );
    m_filmPlay->setDisabled( true );
    m_filmPause->setDisabled( true );
    m_filmStop->setDisabled( true );
    m_filmLastFrame->setDisabled( true );
    m_filmStepPrev->setDisabled( true );
    m_filmStepNext->setDisabled( true );
    QString v;
    v = v.setNum( m_filmstrip.secPerFrame() );
    m_secFrameTXT->setText( v );
    m_secFrameTXT->setDisabled( true );
    v = v.setNum( (int)m_filmstrip.stepPerFrame() );
    m_stepsFrameTXT->setText( v );
    m_stepsFrameTXT->setDisabled( true );

#if TEMPLAND_TOOLKIT_BUILD
m_splineTemplateLmks = new MatrixD;
m_splineTargetLmks = new MatrixD;
#endif

}

TableauLayout::~TableauLayout()
{
    for(int i = m_dig3.get_n_views() - 1; i >= 0; --i)
        delete m_dig3.get_views()[i];
    if(m_ewWarnings.str().length() > 0)
        QMessageBox::information(this, "Session Warnings", QString("EW library Warnings:\n") + m_ewWarnings.str().c_str());
}

void TableauLayout::fillViewTrees(FormItem* formItem)
{
    m_treesFull = false;

    connect( formItem==m_targetFormItem ? m_targetView : m_templateView, SIGNAL(viewChanged()),this, SLOT(tableauUpdated()));
    connect( formItem==m_targetFormItem ? m_targetSliceView : m_templateSliceView, SIGNAL(viewChanged()),this, SLOT(tableauUpdated()));

    if( formItem==m_targetFormItem )
    {
        connect( m_targetView, SIGNAL(viewChanged()),this, SLOT(targetFormUpdated()));
        connect( m_targetSliceView, SIGNAL(viewChanged()),this, SLOT(targetFormUpdated()));
    }

    connect(formItem, SIGNAL(formFocus(FormItem*, ViewTreeItem*)), this, SLOT(focusView(FormItem*, ViewTreeItem*)));
    if( formItem == m_targetFormItem )
    {
    connect(formItem, SIGNAL(formSaved(FormItem*)), this, SLOT(saveForm(FormItem*)));
    connect(formItem, SIGNAL(formSavedAs(FormItem*)), this, SLOT(saveFormAs(FormItem*)));
    connect(formItem, SIGNAL(formReseted(FormItem*)), this, SLOT(resetForm(FormItem*)));

    //connect(formItem, SIGNAL(formNew(FormItem*)), this, SLOT(tblNew(FormItem*)));
    connect(formItem, SIGNAL(formAppend(FormItem*)), this, SLOT(tblAppend(FormItem*)));
    connect(formItem, SIGNAL(formInsert(FormItem*)), this, SLOT(tblInsert(FormItem*)));
    connect(formItem, SIGNAL(formReplace(FormItem*)), this, SLOT(tblReplace(FormItem*)));
    connect(formItem, SIGNAL(formDelete(FormItem*)), this, SLOT(tblDelete(FormItem*)));

    connect( this, SIGNAL(newForm(bool)), formItem, SLOT(newForm(bool)) );
    connect( this, SIGNAL(appendForm(bool)), formItem, SLOT(appendForm(bool)) );
    connect( this, SIGNAL(insertForm(bool)), formItem, SLOT(insertForm(bool)) );
    connect( this, SIGNAL(replaceForm(bool)), formItem, SLOT(replaceForm(bool)) );
    connect( this, SIGNAL(deleteForm(bool)), formItem, SLOT(deleteForm(bool)) );
    }

    connect(formItem, SIGNAL(formLoaded(FormItem*)), this, SLOT(loadForm(FormItem*)));

    connect( formItem, SIGNAL(formSwapPrev(FormItem*)), this, SLOT(tblSwapPrev()));
    connect( formItem, SIGNAL(formSwapNext(FormItem*)), this, SLOT(tblSwapNext()));

//    VolumeItem* targetVolume = new VolumeItem(formItem);
//    targetVolume->setText(0, "Volume");
//    targetVolume->setCheckState(1, Qt::Checked);
//    targetVolume->setCheckState(2, Qt::Checked);

    SurfaceTopItem* targetTopSurfaces = 0;
    if( formItem == m_targetFormItem )
    {
        targetTopSurfaces = m_targetTopSurface = new SurfaceTopItem(formItem, (formItem==m_targetFormItem) );
    }
    else
    {
        targetTopSurfaces = m_templateTopSurface = new SurfaceTopItem(formItem, (formItem==m_targetFormItem) );
    }
    targetTopSurfaces->setText(0, "Surfaces");
    targetTopSurfaces->setCheckState(1, Qt::Checked);
    targetTopSurfaces->setCheckState(2, Qt::Checked);
    connect(targetTopSurfaces, SIGNAL(surfaceFocus(FormItem*, ViewTreeItem*)), this, SLOT(focusView(FormItem*, ViewTreeItem*)));
    if( formItem == m_targetFormItem )
    {
        connect(targetTopSurfaces, SIGNAL(surfaceItemAdded(SurfaceItem*, FormItem*)),
                this, SLOT(addSurface(SurfaceItem*, FormItem*)));
    }
    connect(targetTopSurfaces, SIGNAL(surfaceColorChange(SurfaceItem*, FormItem*, bool)),
            this, SLOT(changeSurfaceColor(SurfaceItem*, FormItem*, bool)));
    connect(targetTopSurfaces, SIGNAL(surfaceDeleted(FormItem*, int)), this, SLOT(deleteSurface(FormItem*, int)));

//    CurvesTopItem* targetTopCurves = new CurvesTopItem(formItem);
//    targetTopCurves->setText(0, "Curves");
//    targetTopCurves->setCheckState(1, Qt::Checked);
//    targetTopCurves->setCheckState(2, Qt::Checked);

    LandmarksTopItem* lmkTopItem = 0;
    if( formItem == m_targetFormItem )
    {
        lmkTopItem = m_targetTopLandmarks = new LandmarksTopItem(formItem);
        m_targetTopLandmarks->templateTreeView( false );
    }
    else
    {
        lmkTopItem = m_templateTopLandmarks = new LandmarksTopItem(formItem);
        m_templateTopLandmarks->templateTreeView( true );
    }
    lmkTopItem->setText(0, "Landmarks");
    lmkTopItem->setCheckState(1, Qt::Checked);
    lmkTopItem->setCheckState(2, Qt::Checked);
    connect( lmkTopItem, SIGNAL(lmkFocus(FormItem*, ViewTreeItem*)), this, SLOT(focusView(FormItem*, ViewTreeItem*)));
    connect( lmkTopItem, SIGNAL(lmkColorChanged(FormItem*)), this, SLOT(changeLmkColor(FormItem*)));
    connect( lmkTopItem, SIGNAL(lmkAdded(FormItem*)), this, SLOT(formUpdated(FormItem*)));
    connect( formItem==m_targetFormItem ? m_targetView : m_templateView, SIGNAL(landmarkAdded(const QString&)),
            lmkTopItem, SLOT(addLandmarkItem(const QString&)));
    connect( formItem==m_targetFormItem ? m_targetSliceView : m_templateSliceView, SIGNAL(landmarkAdded(const QString&)),
            lmkTopItem, SLOT(addLandmarkItem(const QString&)));
    connect( lmkTopItem, SIGNAL(lmkDeleted(FormItem*, int)), this, SLOT(deleteLmk(FormItem*, int)));
    connect( lmkTopItem, SIGNAL(lmkDeletedChild(FormItem*, ViewTreeItem*, int)), this, SLOT(deleteLmk(FormItem*, ViewTreeItem*, int)));
    connect( lmkTopItem, SIGNAL(lmkSymbolChanged(FormItem*, EW_View3_Landmarks::SymbolType)),
                            this, SLOT(changeLmksSymbol(FormItem*, EW_View3_Landmarks::SymbolType)));
    if( formItem == m_templateFormItem )
    {
        connect( lmkTopItem, SIGNAL(lmkMapped(FormItem*, int)), this, SLOT(mapLmk(FormItem*, int)) );
        connect( lmkTopItem, SIGNAL(lmkMappedAll(FormItem*, int)), this, SLOT(mapAllLmk(FormItem*, int)) );
        //connect( lmkTopItem, SIGNAL(spline()), this, SLOT(computeSpline()) );

        //connect( lmkTopItem, SIGNAL(match()), this, SLOT(matchAllLmk()) );
        connect( lmkTopItem, SIGNAL(matchSpline()), this, SLOT(matchSpline()) );
        connect( lmkTopItem, SIGNAL(matchSplineWarp()), this, SLOT(matchSplineWarp()) );
    }
    else
    {
        connect( lmkTopItem, SIGNAL(lmkRegister(FormItem*, int)), this, SLOT(matchLmk(FormItem*, int)) );
        connect( lmkTopItem, SIGNAL(addLandmarkHere(const QString&, int)), m_targetView, SLOT(addLandmarkHere(const QString&, int)) );
        connect( lmkTopItem, SIGNAL(addLandmarkHere(const QString&, int)), m_targetSliceView, SLOT(addLandmarkHere(const QString&, int)) );
        connect( lmkTopItem, SIGNAL(lmkProject(FormItem*, ViewTreeItem*, int)), this, SLOT(lmkProject(FormItem*, ViewTreeItem*, int)) );
        connect( lmkTopItem, SIGNAL(lmkSlide(FormItem*, ViewTreeItem*, int, bool)), this, SLOT(lmkSlide(FormItem*, ViewTreeItem*, int, bool)) );
        connect( lmkTopItem, SIGNAL(lmkRename(FormItem*, ViewTreeItem*, int, const QString&)), this, SLOT(renameLmk(FormItem*, ViewTreeItem*, int, const QString&)) );
        connect( lmkTopItem, SIGNAL(lmkState(FormItem*, ViewTreeItem*, int, int)), this, SLOT(stateLmk(FormItem*, ViewTreeItem*, int, int)) );
    }

    SliceItem* targetSlice = new SliceItem(formItem==m_targetFormItem? m_targetItemTree: m_templateItemTree);
    targetSlice->setText(0, "Slice");
    targetSlice->setCheckState(1, Qt::Checked);
    targetSlice->setCheckState(2, Qt::Checked);
    connect(targetSlice, SIGNAL(sliceColorAction(ViewTree*)), this, SLOT(changeSliceColor(ViewTree*)));
    connect(targetSlice, SIGNAL(sliceFocus(FormItem*, ViewTreeItem*)), this, SLOT(focusView(FormItem*, ViewTreeItem*)));
    connect(targetSlice, SIGNAL(sliceClipAction(ViewTree*)), this, SLOT(changeSliceClipR(ViewTree*)));

//    GridItem* targetGrid = new GridItem(formItem==m_targetFormItem? m_targetItemTree: m_templateItemTree);
//    targetGrid->setText(0, "Grid");
//    targetGrid->setCheckState(1, Qt::Unchecked);
//    targetGrid->setCheckState(2, Qt::Unchecked);

//    RefPlanesItem* targetRefPlanes = new RefPlanesItem(formItem==m_targetFormItem? m_targetItemTree: m_templateItemTree);
//    targetRefPlanes->setText(0, "Reference Planes");
//    targetRefPlanes->setCheckState(1, Qt::Unchecked);
//    targetRefPlanes->setCheckState(2, Qt::Unchecked);

    formItem->setExpanded(true);
    m_treesFull = true;

    m_templateItemTree->resizeColumnToContents( 0 );
    m_templateItemTree->resizeColumnToContents( 1 );
    m_templateItemTree->resizeColumnToContents( 2 );

    m_targetItemTree->resizeColumnToContents( 0 );
    m_targetItemTree->resizeColumnToContents( 1 );
    m_targetItemTree->resizeColumnToContents( 2 );
}

void TableauLayout::lmkProject(FormItem* form, ViewTreeItem* item, int index)
{
    QString num;
    num.setNum( index );

    if( form == m_templateFormItem )
        return;

    if( m_dig3.get_spaces()[1]->get_form_data()->surfaces.size() == 0 )
    {
        QMessageBox::critical( this, "Error", "There is no surface to project the landmarks on. Please import a surface." );

        return;
    }

    if( item->getType() == ViewTreeItem::LANDMARKS_ITEM )
    {
        projectAll( false );
    }
    else if( item->getType() == ViewTreeItem::LANDMARK_ITEM )
    {
        projectLmk( item, index, false );
    }
    else if( item->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
    {
        projectSemiLmk( item, false );
    }

    // add the landmark states
    updateTreeViewStates( 1 );
}

void TableauLayout::projectAll( bool checksurface )
{
    if( checksurface )
    {
        if( m_dig3.get_spaces()[1]->get_form_data()->surfaces.size() == 0 )
        {
            QMessageBox::critical( this, "Error", "There is no surface to project the landmarks on. Please import a surface." );

            return;
        }
    }

    int notprojected = 0;
    int count = m_targetTopLandmarks->childCount();
    for( int i = 0; i < count; ++i )
    {
        ViewTreeItem* vti = (ViewTreeItem*)m_targetTopLandmarks->child( i );

        if( vti->getType() == ViewTreeItem::LANDMARK_ITEM )
        {
            LandmarkItem *li = (LandmarkItem*)vti;
            if( projectLmk( vti, li->getLmkIndex(), checksurface, false ) == false )
            {
                ++notprojected;
            }
        }
        else if( vti->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
        {
            if( projectSemiLmk( vti, checksurface, false ) == false )
            {
                ++notprojected;
            }
        }
    }

#if TEMPLAND_TOOLKIT_BUILD
    QString msg; msg.setNum( count - notprojected );
    msg += " Landmarks have been projected.";
    if( notprojected > 0 )
    {
        QString ss; ss.setNum( notprojected );
        msg += " ";
        msg += ss;
        msg += " Landmarks were already projected.";
    }
    Logger::getInstance()->log( msg, Logger::INFO );
#endif
}

bool TableauLayout::projectLmk( ViewTreeItem* item, int index, bool checksurface, bool showstatus )
{
    if( checksurface )
    {
        if( m_dig3.get_spaces()[1]->get_form_data()->surfaces.size() == 0 )
        {
            QMessageBox::critical( this, "Error", "There is no surface to project the landmarks on. Please import a surface." );

            return false;
        }
    }

    std::string s = ((LandmarkItem*)item)->getLmkID().toStdString();
    const EW_Form3* form = m_dig3.get_spaces()[1]->get_form_data();
    if( form != 0 )
        for(unsigned int j=0; j<form->pointsets.size(); ++j)
            if( form->pointsets[j].type == EW_Form3::TYPE_LANDMARK )
            {
                if( form->pointsets[j].id == s )
                {
                    if( form->pointsets[j].state == EW_Form3::STATE_WARPED ||
                        form->pointsets[j].state == EW_Form3::STATE_SET )
                    {
                        double p[3];
                        p[0] = form->pointsets[j].locations[0];
                        p[1] = form->pointsets[j].locations[1];
                        p[2] = form->pointsets[j].locations[2];

                        const EW_Dataflow_Surface3_Explicit * const * sur = m_dig3.get_spaces()[1]->get_surface_nodes();

                        int face[3];
                        double coeff[3];
                        double normal[3];
                        double proj[3];

                        (*sur)->project( face, coeff, normal, proj, p );

                        EW_Form3_PointSet ps = form->pointsets[j];

                        ps.locations[0] = proj[0];
                        ps.locations[1] = proj[1];
                        ps.locations[2] = proj[2];

                        ps.state = EW_Form3::STATE_PROJECTED;

                        EW_Dig3_Space *sp = m_dig3.get_spaces()[1];
                        bool b = false;
                        sp->set_form_pointset(&b, &ps);

                        // make sure to note form has been changed
                        m_targetSaved = false;

#if TEMPLAND_TOOLKIT_BUILD
                        if( showstatus )
                        {
                            QString msg = s.c_str();
                            msg += " Landmark has been projected.";
                            Logger::getInstance()->log( msg, Logger::INFO );
                        }
#endif

                        return true;
                    }
                    else if( form->pointsets[j].state == EW_Form3::STATE_PROJECTED )
                    {
                        if( checksurface )
                            QMessageBox::critical( this, "Error", "Landmark already projected." );

#if TEMPLAND_TOOLKIT_BUILD
    QString msg; msg = s.c_str(); msg += " has already been projected.";
    Logger::getInstance()->log( msg, Logger::WARNING );
#endif
                        return false;
                    }
//                    else if( form->pointsets[j].state == EW_Form3::STATE_SET )
//                    {
//                        QMessageBox::critical( this, "Error", "Landmark needs to be warped before projection." );
//                        return false;
//                    }
                }
            }
    return false;
}

bool TableauLayout::projectSemiLmk( ViewTreeItem* item, bool checksurface, bool showstatus )
{
    if( checksurface )
    {
        if( m_dig3.get_spaces()[1]->get_form_data()->surfaces.size() == 0 )
        {
            QMessageBox::critical( this, "Error", "There is no surface to project the landmarks on. Please import a surface." );

            return false;
        }
    }

    std::string s = ((SemiLandmarksTopItem*)item)->getLmkID().toStdString();
    const EW_Form3* form = m_dig3.get_spaces()[1]->get_form_data();
    if( form != 0 )
        for(unsigned int j=0; j<form->pointsets.size(); ++j)
            if( form->pointsets[j].type == EW_Form3::TYPE_SEMI_LANDMARK  )
            {
                if( form->pointsets[j].id == s )
                {
                    if( form->pointsets[j].state == EW_Form3::STATE_WARPED ||
                        form->pointsets[j].state == EW_Form3::STATE_OPTIMIZED ||
                        form->pointsets[j].state == EW_Form3::STATE_SET )
                    {
                        EW_Form3_PointSet ps = form->pointsets[j];
                        ps.locations.clear();
                        ps.relax_params.clear();
                        int n = form->pointsets[j].n;

                        const EW_Dataflow_Surface3_Explicit * const * sur = m_dig3.get_spaces()[1]->get_surface_nodes();

                        for( int i = 0; i < n; ++i )
                        {
                            double p[3];
                            int face[3];
                            double coeff[3];
                            double normal[3];
                            double proj[3];

                            p[0] = form->pointsets[j].locations[i * 3];
                            p[1] = form->pointsets[j].locations[i * 3 + 1];
                            p[2] = form->pointsets[j].locations[i * 3 + 2];

                            (*sur)->project( face, coeff, normal, proj, p );

                            ps.locations.push_back( proj[0] );
                            ps.locations.push_back( proj[1] );
                            ps.locations.push_back( proj[2] );

                            ps.relax_params.push_back( normal[0] );
                            ps.relax_params.push_back( normal[1] );
                            ps.relax_params.push_back( normal[2] );

                        }

                        ps.state = EW_Form3::STATE_PROJECTED;

                        EW_Dig3_Space *sp = m_dig3.get_spaces()[1];
                        bool b = false;
                        sp->set_form_pointset(&b, &ps);

                        // make sure to note form has been changed
                        m_targetSaved = false;

#if TEMPLAND_TOOLKIT_BUILD
                        if( showstatus )
                        {
                            QString num; num.setNum( n );
                            QString msg;
                            msg = num;
                            msg += " SemiLandmarks in ";
                            msg += s.c_str();
                            msg += " patch have been projected.";
                            Logger::getInstance()->log( msg, Logger::INFO );
                        }
#endif
                        return true;
                    }
                    else if( form->pointsets[j].state == EW_Form3::STATE_PROJECTED )
                    {
                        if( checksurface )
                            QMessageBox::critical( this, "Error", "Landmark already projected." );
#if TEMPLAND_TOOLKIT_BUILD
    QString msg; msg = s.c_str(); msg += " has already been projected.";
    Logger::getInstance()->log( msg, Logger::WARNING );
#endif
                        return false;
                    }
//                    else if( form->pointsets[j].state == EW_Form3::STATE_SET )
//                    {
//                        QMessageBox::critical( this, "Error", "Landmarks needs to be warped before projection." );
//                        return false;
//                    }
                }
            }
    return false;
}

void TableauLayout::lmkSlide( FormItem* form, ViewTreeItem* item, int index, bool slide)
{
    if( m_dig3.get_spaces()[1]->get_form_data()->surfaces.size() == 0 )
    {
        QMessageBox::critical( this, "Error", "There is no surface to slide the landmarks on. Please import a surface." );
        return;
    }

    const EW_Dataflow_Surface3_Explicit * const * sur = m_dig3.get_spaces()[1]->get_surface_nodes();
    std::string str = (*sur)->get_filename();

    /*if( item->getType() == ViewTreeItem::LANDMARK_ITEM )
    {
        std::string s = ((LandmarkItem*)item)->getLmkID().toStdString();

        const EW_Form3* form = m_dig3.get_spaces()[1]->get_form_data();
        if( form != 0 )
        for(unsigned int j=0; j<form->pointsets.size(); ++j)
            if( form->pointsets[j].type == EW_Form3::TYPE_SEMI_LANDMARK  )
            {
                if( form->pointsets[j].id == s )
                {
                    if( form->pointsets[j].state == EW_Form3::STATE_WARPED )
                    {
                        EW_Form3_Embedding fe;
                        fe.subset_id = str;
                        fe.superset_id = s;

                        EW_Form3_PointSet ps = form->pointsets[j];
                        ps.state = EW_Form3::STATE_PROJECTED;
                        for( unsigned int i = 0; i < ps.relax_dims.size(); ++i )
                            ps.relax_dims[i] = slide ?  1 : 0;
                        EW_Dig3_Space *sp = m_dig3.get_spaces()[1];
                        // set new pointset and form embedding
                        bool b = false;
                        sp->set_form_embedding( &b, &fe );
                        b = false;
                        sp->set_form_pointset(&b, &ps);
                    }
                }
            }

    }
    else*/ if( item->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
    {
        std::string s = ((SemiLandmarksTopItem*)item)->getLmkID().toStdString();

        const EW_Form3* form = m_dig3.get_spaces()[1]->get_form_data();
        if( form != 0 )
        for(unsigned int j=0; j<form->pointsets.size(); ++j)
            if( form->pointsets[j].type == EW_Form3::TYPE_SEMI_LANDMARK  )
            {
                if( form->pointsets[j].id == s )
                {
                    if( form->pointsets[j].state == EW_Form3::STATE_PROJECTED )
                    {
                        EW_Form3_Embedding fe;
                        fe.subset_id = str;
                        fe.superset_id = s;

                        EW_Form3_PointSet ps = form->pointsets[j];
                        //ps.state = EW_Form3::STATE_PROJECTED;
                        for( unsigned int i = 0; i < ps.relax_dims.size(); ++i )
                            ps.relax_dims[i] = slide ?  2 : 0;
                        if( (int)ps.relax_dims.size() < ps.n )
                            for( int i = (int)ps.relax_dims.size(); i < ps.n; ++i )
                                ps.relax_dims.push_back( slide ?  2 : 0 );
                        EW_Dig3_Space *sp = m_dig3.get_spaces()[1];
                        // set new pointset and form embedding
                        bool b = false;
                        sp->set_form_embedding( &b, &fe );
                        b = false;
                        sp->set_form_pointset(&b, &ps);

                        int n = form->pointsets[j].n;

                        const EW_Dataflow_Spline3* ds = m_dig3.get_spline_node();
                        try
                        {
                            const double* points = ds->get_optimized_lmk_images();
                            if( points )
                            {
                                for( int i = 0; i < n; ++i )
                                {
                                    int index = ds->lmk_index( 1, j, i );
                                    ps.locations[i * 3    ] = points[index * 3    ];
                                    ps.locations[i * 3 + 1] = points[index * 3 + 1];
                                    ps.locations[i * 3 + 2] = points[index * 3 + 2];
                                }
                                ps.state = EW_Form3::STATE_OPTIMIZED;
                                b = false;
                                sp->set_form_pointset(&b, &ps);

                                // now update their state
                                updateTreeViewStates( 1 );
                                return;
                            }
                        }
                        catch( std::exception& ex )
                        {
                            if( ds->get_nonsingular() == false )
                                QMessageBox::critical( this, "Error", "Template should be loaded in order to slide Landmarks." );
                            else
                                QMessageBox::critical( this, "Error", ex.what() );
                        }

                    } // end projected check
                    else
                    {
                        QMessageBox::critical( this, "Error", "Landmarks should be projected before slid." );
                        return;
                    }
                }
            }
    }
}

void TableauLayout::mapLmk(FormItem* item, int index)
{
    if( m_regular )
        do_regular_map(item, index);
    else
        do_ew_map(item, index);
}

void TableauLayout::do_regular_map(FormItem* item, int index)
{
#if TEMPLAND_TOOLKIT_BUILD
    if( !m_tps )
    {
        QMessageBox::critical(this, "Error", "Compute spline before mapping Template landmarks to Specimen.");
        return;
    }


    // get the template landmark
    int size = m_templateTopLandmarks->childCount();
    for( int i = 0; i < size; ++i )
    {
        ViewTreeItem* vti = (ViewTreeItem*)m_templateTopLandmarks->child( i );
        if( vti->getType() != ViewTreeItem::LANDMARK_ITEM )
            continue;
        LandmarkItem* lmi = (LandmarkItem*)m_templateTopLandmarks->child( i );
        if( lmi->getLmkIndex() == index )
        {
            std::string tt = lmi->getLmkID().toStdString();

            // has the template landmark been mapped before?
            bool mapped = false;
            for( unsigned int i = 0; i < m_regLmks.size() && !mapped; ++i )
                mapped = m_regLmks[i].templateID == tt;

            if( !mapped )
            {
                for( unsigned int i = 0; i < m_mappedLmks.size() && !mapped; ++i )
                    mapped = m_mappedLmks[i] == tt;
            }

            if( mapped )
            {
                QString msg = tt.c_str();
                msg += " has been mapped before.";
                QMessageBox::critical( this, "Error", msg );
                return;
            }

            double x = 0, y = 0, z = 0;
            int index;
            bool r1 = getPointsetLocation( tt, 0, x, y, z, index );

            if( !r1 )
                continue;

            Matrix<double> points;
            points.ReSize( 1, 3 );
            points.set( 0, 0, x );
            points.set( 0, 1, y );
            points.set( 0, 2, z );
            Matrix<double> wpoints( points.GetRows(), points.GetCols());

            m_tps->WarpPoints( points.GetMatrix(), &wpoints, points.GetRows());

            m_targetView->placePoint( wpoints.get( 0, 0 ), wpoints.get( 0, 1 ), wpoints.get( 0, 2 ), false, false, lmi->getLmkID(), EW_Form3::STATE_WARPED );

            m_mappedLmks.push_back( lmi->getLmkID().toStdString() );
        }
    }
#endif
}

bool TableauLayout::isInTargetTreeView( const QString& id )
{
    int size = m_targetTopLandmarks->childCount();
    for( int i = 0; i < size; ++i )
    {
        ViewTreeItem* vti = (ViewTreeItem*)m_targetTopLandmarks->child( i );
        if( vti->getType() == ViewTreeItem::LANDMARK_ITEM )
        {
            if( ((LandmarkItem*)vti)->getLmkID() == id )
            {
                return true;
            }
        }
        else if( vti->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
        {
            if( ((SemiLandmarksTopItem*)vti)->getLmkID() == id )
            {
                return true;
            }
        }
    }
    return false;
}

void TableauLayout::do_ew_map(FormItem* item, int index)
{
    const EW_Dataflow_Spline3* ds = m_dig3.get_spline_node();
    //if( !m_tps )
    if( ds->get_n_lmks() < 4 || ds->get_energy() == -1)
    {
        QMessageBox::critical(this, "Error", "Compute spline before warping Template landmarks to Specimen.");
        return;
    }

    if( ds->get_nonsingular() == false )
    {
        QMessageBox::critical(this, "Error", "Spline is singular. Template landmarks can not be warped.");
        return;
    }

    try
    {

        // get the template landmark
        int size = m_templateTopLandmarks->childCount();
        for( int i = 0; i < size; ++i )
        {
            ViewTreeItem* vti = (ViewTreeItem*)m_templateTopLandmarks->child( i );
            if( vti->getType() == ViewTreeItem::LANDMARK_ITEM )
            {
                LandmarkItem* lmi = (LandmarkItem*)m_templateTopLandmarks->child( i );
                if( lmi->getLmkIndex() == index )
                {
                    std::string tt = lmi->getLmkID().toStdString();

                    double x = 0, y = 0, z = 0;
                    int index;
                    bool r1 = getPointsetLocation( tt, 0, x, y, z, index );

                    if( !r1 )
                        continue;

                    double pin[3];
                    double pout[3];
                    pin[0] = x;
                    pin[1] = y;
                    pin[2] = z;

                    ds->warp_points( pout, pin, 1 );

                    m_targetTopLandmarks->assignLandmarksHere( 0 );
                    // is it already available in target
                    if( isInTargetTreeView( lmi->getLmkID() ) )
                    {
                        // update the existing point location
                        std::string str = lmi->getLmkID().toStdString();
                        const EW_Form3 * form = m_dig3.get_spaces()[1]->get_form_data();
                        for( unsigned int i = 0; i < form->pointsets.size(); ++i )
                        {
                            if( form->pointsets[i].id == str )
                            {
                                EW_Form3_PointSet ps = form->pointsets[i];
                                ps.state = EW_Form3::STATE_WARPED;
                                bool r = false;
                                ps.locations[0] = pout[0];
                                ps.locations[1] = pout[1];
                                ps.locations[2] = pout[2];
                                m_dig3.get_spaces()[1]->set_form_pointset(&r, &ps);
                                //m_dig3.get_spaces()[1]->set_form_pointset_location( i, 0, pout );
                                break;
                            }
                        }
                    }
                    else
                    {
                        m_targetView->placePoint( pout[0], pout[1], pout[2], false, false, lmi->getLmkID(), EW_Form3::STATE_WARPED );
                    }

                    // add the landmark states
                    updateTreeViewStates( 1 );

                    return;
                }
            }
            else if( vti->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
            {
                SemiLandmarksTopItem* lmi = (SemiLandmarksTopItem*)m_templateTopLandmarks->child( i );
                if( lmi->getLmkIndex() != index )
                    continue;

                std::string tt = lmi->getLmkID().toStdString();
                double x, y, z;
                int index2 = 0;
                bool r = getPointsetLocation( tt, 0, x, y, z, index2 );
                if( r == false )
                    continue;
                const EW_Form3 * form = m_dig3.get_spaces()[0]->get_form_data();
                int numSemi = form->pointsets[ index2 ].n;
                double* pin = new double[ 3 * numSemi ];
                double* pout = new double[ 3 * numSemi ];
                int count = 0;
                for( int c = 0; c < numSemi * 3; c += 3 )
                {
                    pin[count] = form->pointsets[ index2 ].locations[ c ];
                    pin[count+1] = form->pointsets[ index2 ].locations[ c + 1 ];
                    pin[count+2] = form->pointsets[ index2 ].locations[ c + 2 ];
                    pout[count] = 0.0;
                    pout[count+1] = 0.0;
                    pout[count+2] = 0.0;
                    count+=3;
                }

                const EW_Dataflow_Spline3* ds = m_dig3.get_spline_node();
        #if TEMPLAND_TOOLKIT_BUILD
                Writer::Instance().writeline( "warp_points BEGIN" );
        #endif
                ds->warp_points( pout, pin, numSemi );
        #if TEMPLAND_TOOLKIT_BUILD
                Writer::Instance().writeline( "warp_points END" );
        #endif

                count = 0;
                // is it already available in target
                if( isInTargetTreeView( lmi->getLmkID() ) )
                {
                    // update the existing point location
                    std::string str = lmi->getLmkID().toStdString();
                    const EW_Form3 * form = m_dig3.get_spaces()[1]->get_form_data();
                    for( unsigned int i = 0; i < form->pointsets.size(); ++i )
                    {
                        if( form->pointsets[i].id == str )
                        {
                            EW_Form3_PointSet ps = form->pointsets[i];
                            ps.state = EW_Form3::STATE_WARPED;
                            bool r = false;
                            m_dig3.get_spaces()[1]->set_form_pointset(&r, &ps);
                            for( int c = 0; c < ps.n; ++c )
                            {
                                //ps.locations[c * 3    ] = pout[count];
                                //ps.locations[c * 3 + 1] = pout[count+1];
                                //ps.locations[c * 3 + 2] = pout[count+2];
                                double ptemp[3];
                                ptemp[0] = pout[count];
                                ptemp[1] = pout[count+1];
                                ptemp[2] = pout[count+2];
                                m_dig3.get_spaces()[1]->set_form_pointset_location( i, c * 3, ptemp );

                                count+=3;
                            }


                            if( ps.n <= numSemi )
                            {
                                // activate the semilandmark patch
                                for( int c = 0; c < m_targetTopLandmarks->childCount(); ++c )
                                {
                                    ViewTreeItem* vti = (ViewTreeItem*)m_targetTopLandmarks->child( c );
                                    if( vti->getType() == ViewTreeItem::LANDMARK_ITEM )
                                    {
                                        if( ((LandmarkItem*)vti)->getLmkID() == lmi->getLmkID() )
                                        {
                                            m_targetTopLandmarks->assignLandmarksHere( vti );
                                            break;
                                        }
                                    }
                                    else if( vti->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
                                    {
                                        if( ((SemiLandmarksTopItem*)vti)->getLmkID() == lmi->getLmkID() )
                                        {
                                            m_targetTopLandmarks->assignLandmarksHere( vti );
                                            break;
                                        }
                                    }
                                }
                                for( int i = ps.n; i < numSemi; ++i )
                                {
                                    m_targetView->placePoint( pout[count], pout[count+1], pout[count+2], false, false, "", EW_Form3::STATE_WARPED );
                                    count+=3;
                                }
                                m_targetTopLandmarks->assignLandmarksHere( 0 );
                            }
                        } // end if check id
                    } // for loop
                }// end if existing semilandmark patch
                else
                {
                    m_targetTopLandmarks->addSurfaceSemiLandmark( tt.c_str() );
                    m_targetTopLandmarks->assignLandmarksHere( (ViewTreeItem*)m_targetTopLandmarks->child( m_targetTopLandmarks->childCount() - 1 ) );
                    for( int c = 0; c < numSemi; ++c )
                    {
                        m_targetView->placePoint( pout[count], pout[count+1], pout[count+2], false, false, "", EW_Form3::STATE_WARPED );
                        count+=3;
                    }
                    m_targetTopLandmarks->assignLandmarksHere( 0 );
                }

                delete [] pout; pout = 0;
                delete [] pin; pin = 0;


                // add the landmark states
                updateTreeViewStates( 1 );

                return;
            }
        } // end for loop


    }
    catch( ... )
    {
    }

}

void TableauLayout::mapAllLmk(FormItem* item, int index)
{
    if( m_regular )
        do_regular_mapall(item, index);
    else
        do_ew_mapall(item, index);
}

void TableauLayout::do_regular_mapall(FormItem* item, int index)
{
#if TEMPLAND_TOOLKIT_BUILD
    if( !m_tps )
    {
        QMessageBox::critical(this, "Error", "Compute spline before warping Template landmarks to Specimen.");
        return;
    }

    // get the template landmark
    int size = m_templateTopLandmarks->childCount();
    std::vector<int> indices;
    std::vector<int> semiindices;
    for( int i = 0; i < size; ++i )
    {
        ViewTreeItem* vti = (ViewTreeItem*)m_templateTopLandmarks->child( i );
        if( vti->getType() != ViewTreeItem::LANDMARK_ITEM && vti->getType() != ViewTreeItem::SEMILANDMARKS_ITEM )
            continue;

        std::string tt;
        if( vti->getType() == ViewTreeItem::LANDMARK_ITEM )
            tt = ((LandmarkItem*)vti)->getLmkID().toStdString();
        else if( vti->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
            tt = ((SemiLandmarksTopItem*)vti)->getLmkID().toStdString();

        // has the template landmark been mapped before?
        bool mapped = false;
        for( unsigned int ii = 0; ii < m_regLmks.size() && !mapped; ++ii )
            mapped = m_regLmks[ii].templateID == tt;
        if( !mapped )
            for( unsigned int iii = 0; iii < m_mappedLmks.size() && !mapped; ++iii )
                mapped = m_mappedLmks[iii] == tt;
        if( !mapped )
        {
            if( vti->getType() == ViewTreeItem::LANDMARK_ITEM )
                indices.push_back( i );
            else if( vti->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
            {
                // now make sure that the semi landmark we want to warp does not already exist in target "specimen"
                int size = m_targetTopLandmarks->childCount();
                bool found = false;
                for( int f = 0; f < size && !found; ++f )
                {
                    ViewTreeItem* vti = (ViewTreeItem*)m_targetTopLandmarks->child( f );
                    if( vti->getType() != ViewTreeItem::SEMILANDMARKS_ITEM )
                        continue;
                    std::string tar = ((SemiLandmarksTopItem*)vti)->getLmkID().toStdString();
                    if( tar == tt )
                        found = true;
                }
                if( !found )
                    semiindices.push_back( i );
            } // end else
        } // end if
    } // end for

    if( indices.size() )
    {
        Matrix<double> points;
        points.ReSize( indices.size(), 3 );
        int count = 0;

        vector<QString> sl;
        for( unsigned int i = 0; i < indices.size(); ++i )
        {
            ViewTreeItem* vti = (ViewTreeItem*)m_templateTopLandmarks->child( indices[i] );
            if( vti->getType() != ViewTreeItem::LANDMARK_ITEM )
                continue;
            LandmarkItem* lmi = (LandmarkItem*)vti;
            std::string tt = lmi->getLmkID().toStdString();

            sl.push_back( lmi->getLmkID() );

            double x = 0, y = 0, z = 0;
            int index;
            bool r1 = getPointsetLocation( tt, 0, x, y, z, index );

            if( !r1 )
                continue;

            points.set( count, 0, x );
            points.set( count, 1, y );
            points.set( count, 2, z );
            count++;

            m_mappedLmks.push_back( lmi->getLmkID().toStdString() );
        }


        Matrix<double> wpoints( points.GetRows(), points.GetCols());

        m_tps->WarpPoints( points.GetMatrix(), &wpoints, points.GetRows());

        for( unsigned int i = 0; i < wpoints.GetRows(); ++i )
            m_targetView->placePoint( wpoints.get( i, 0 ), wpoints.get( i, 1 ), wpoints.get( i, 2 ), false, false, sl[i], EW_Form3::STATE_WARPED );

        // mapped successfully
        QString msg, num;
        num.setNum( count );
        msg = "Warped ";
        msg += num;
        msg += " landmarks. ";
        emit status( msg );
    } // end if landmark indices

    for( unsigned int i = 0; i < semiindices.size(); ++i )
    {
        ViewTreeItem* vti = (ViewTreeItem*)m_templateTopLandmarks->child( semiindices[i] );
        if( vti->getType() != ViewTreeItem::SEMILANDMARKS_ITEM )
            continue;
        std::string tt = ((SemiLandmarksTopItem*)vti)->getLmkID().toStdString();
        Matrix<double> points;
        int count = 0;

        // now find the semi landmark pointset in the list
        const EW_Form3* form = m_dig3.get_spaces()[0]->get_form_data();
        if( form != 0 )
            for(unsigned int j=0; j<form->pointsets.size(); ++j)
                if( form->pointsets[j].type == EW_Form3::TYPE_SEMI_LANDMARK )
                {
                    if( form->pointsets[j].id == tt )
                    {
                        points.ReSize( form->pointsets[j].n, 3 );

                        // fill in the semi landmarks location in array
                        for( int l = 0; l < form->pointsets[j].n; ++l )
                        {
                            points.set( count, 0, form->pointsets[j].locations[l] );
                            points.set( count, 1, form->pointsets[j].locations[l+1] );
                            points.set( count, 2,  form->pointsets[j].locations[l+2] );
                            count++;
                        }

                        // warp and add
                        if( points.GetRows() > 0 )
                        {
                            m_targetTopLandmarks->addSurfaceSemiLandmark( tt.c_str() );
                            m_targetTopLandmarks->assignLandmarksHere( (ViewTreeItem*)m_targetTopLandmarks->child( m_targetTopLandmarks->childCount() - 1 ) );

                            Matrix<double> wpoints( points.GetRows(), points.GetCols());

                            m_tps->WarpPoints( points.GetMatrix(), &wpoints, points.GetRows());

                            for( unsigned int i = 0; i < wpoints.GetRows(); ++i )
                                m_targetView->placePoint( wpoints.get( i, 0 ), wpoints.get( i, 1 ), wpoints.get( i, 2 ), false, false, "", EW_Form3::STATE_WARPED );

                            m_targetTopLandmarks->assignLandmarksHere( 0 );

                            m_mappedLmks.push_back( tt );
                        }

                        break;
                    } // end if id is identical
                } // end if pointset semi landmark
    } // end semilandmark loop
#endif
}

void TableauLayout::do_ew_mapall(FormItem* item, int index)
{
    const EW_Dataflow_Spline3* ds = m_dig3.get_spline_node();

    if( ds->get_nonsingular() == false )
        return;

    if( ds->get_n_lmks() < 4 || ds->get_energy() == -1)
    {
        QMessageBox::critical(this, "Error", "Compute spline before warping Template landmarks to Specimen.");
        return;
    }

    if( ds->get_nonsingular() == false )
    {
        QMessageBox::critical(this, "Error", "Spline is singular. Template landmarks can not be warped.");
        return;
    }

    try
    {
        // get the template landmark
        int size = m_templateTopLandmarks->childCount();
        std::vector<int> indices;
        std::vector<int> indiceslbl;
        std::vector<int> semiindices;
        for( int i = 0; i < size; ++i )
        {
            ViewTreeItem* vti = (ViewTreeItem*)m_templateTopLandmarks->child( i );
            if( vti->getType() != ViewTreeItem::LANDMARK_ITEM && vti->getType() != ViewTreeItem::SEMILANDMARKS_ITEM )
                continue;

            std::string tt;
            if( vti->getType() == ViewTreeItem::LANDMARK_ITEM )
                tt = ((LandmarkItem*)vti)->getLmkID().toStdString();
            else if( vti->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
                tt = ((SemiLandmarksTopItem*)vti)->getLmkID().toStdString();

            double x, y, z;
            int index = 0;
            bool r = getPointsetLocation( tt, 0, x, y, z, index );
            if( !r )
                continue;

            int r1 = ds->lmk_index( 0, index, 0 );
            if( r1 == -1 )
            {
                if( vti->getType() == ViewTreeItem::LANDMARK_ITEM )
                {
                    indices.push_back( index );
                    indiceslbl.push_back( i );
                }
                else if( vti->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
                    semiindices.push_back( i );//index );
            }
        }

        if( indices.size() )
        {
            double* pin = new double[ 3 * indices.size() ];
            double* pout = new double[ 3 * indices.size() ];
            int count = 0;

            std::vector<QString> sl;
            for( unsigned int i = 0; i < indices.size(); ++i )
            {
                const EW_Form3 * form = m_dig3.get_spaces()[0]->get_form_data();

                sl.push_back( form->pointsets[ indices[i] ].id.c_str() );

                pin[count] = form->pointsets[ indices[i] ].locations[0];
                pin[count+1] = form->pointsets[ indices[i] ].locations[1];
                pin[count+2] = form->pointsets[ indices[i] ].locations[2];
                pout[count] = 0.0;
                pout[count+1] = 0.0;
                pout[count+2] = 0.0;
                count+=3;
            }

            const EW_Dataflow_Spline3* ds = m_dig3.get_spline_node();
    #if TEMPLAND_TOOLKIT_BUILD
            Writer::Instance().writeline( "warp_points BEGIN" );
    #endif
            ds->warp_points( pout, pin, indices.size() );
    #if TEMPLAND_TOOLKIT_BUILD
            Writer::Instance().writeline( "warp_points END" );
    #endif


            count = 0;
            for( unsigned int i = 0; i < indices.size(); ++i )
            {
                ViewTreeItem* vti = (ViewTreeItem*)m_templateTopLandmarks->child( indiceslbl[i] );

                m_targetTopLandmarks->assignLandmarksHere( 0 );
                // is it already available in target
                if( isInTargetTreeView( ((LandmarkItem*)vti)->getLmkID() ) )
                {
                    // update the existing point location
                    std::string str = ((LandmarkItem*)vti)->getLmkID().toStdString();
                    const EW_Form3 * form = m_dig3.get_spaces()[1]->get_form_data();
                    for( unsigned int i = 0; i < form->pointsets.size(); ++i )
                    {
                        if( form->pointsets[i].id == str )
                        {
                            EW_Form3_PointSet ps = form->pointsets[i];
                            ps.state = EW_Form3::STATE_WARPED;
                            bool r = false;
                            m_dig3.get_spaces()[1]->set_form_pointset(&r, &ps);
                            //ps.locations[0] = pout[count];
                            //ps.locations[1] = pout[count+1];
                            //ps.locations[2] = pout[count+2];
                            double ptemp[3];
                            ptemp[0] = pout[count];
                            ptemp[1] = pout[count+1];
                            ptemp[2] = pout[count+2];
                            m_dig3.get_spaces()[1]->set_form_pointset_location( i, 0, ptemp );
                            break;
                        }
                    }
                }
                else
                {
                    m_targetView->placePoint( pout[count], pout[count+1], pout[count+2], false, false, sl[i], EW_Form3::STATE_WARPED );
                }
                count+=3;
            }


            delete [] pout; pout = 0;
            delete [] pin; pin = 0;

            // mapped successfully
            QString msg, num;
            num.setNum( indices.size() );
            msg = "Warped ";
            msg += num;
            msg += " landmarks. ";
            emit status( msg );
        }

        for( unsigned int i = 0; i < semiindices.size(); ++i )
        {
            ViewTreeItem* vti = (ViewTreeItem*)m_templateTopLandmarks->child( semiindices[i] );
            if( vti->getType() != ViewTreeItem::SEMILANDMARKS_ITEM )
                continue;
            std::string tt = ((SemiLandmarksTopItem*)vti)->getLmkID().toStdString();
            double x, y, z;
            int index = 0;
            bool r = getPointsetLocation( tt, 0, x, y, z, index );
            if( r == false )
                continue;
            const EW_Form3 * form = m_dig3.get_spaces()[0]->get_form_data();
            int numSemi = form->pointsets[ index ].n;
            double* pin = new double[ 3 * numSemi ];
            double* pout = new double[ 3 *numSemi ];
            int count = 0;
            int semiInd = 0;
            for( int c = 0; c < numSemi * 3; c += 3 )
            {
                pin[count] = form->pointsets[ index ].locations[ c ];
                pin[count+1] = form->pointsets[ index ].locations[ c + 1 ];
                pin[count+2] = form->pointsets[ index ].locations[ c + 2 ];
                pout[count] = 0.0;
                pout[count+1] = 0.0;
                pout[count+2] = 0.0;
                count+=3;
                ++semiInd;
            }

            const EW_Dataflow_Spline3* ds = m_dig3.get_spline_node();
    #if TEMPLAND_TOOLKIT_BUILD
            Writer::Instance().writeline( "warp_points BEGIN" );
    #endif
            ds->warp_points( pout, pin, numSemi );
    #if TEMPLAND_TOOLKIT_BUILD
            Writer::Instance().writeline( "warp_points END" );
    #endif
            count = 0;
            // is it already available in target
            if( isInTargetTreeView( ((SemiLandmarksTopItem*)vti)->getLmkID() ) )
            {
                // update the existing point location
                const EW_Form3 * form = m_dig3.get_spaces()[1]->get_form_data();
                for( unsigned int i = 0; i < form->pointsets.size(); ++i )
                {
                    if( form->pointsets[i].id == tt )
                    {
                        EW_Form3_PointSet ps = form->pointsets[i];
                        ps.state = EW_Form3::STATE_WARPED;
                        bool r = false;
                        m_dig3.get_spaces()[1]->set_form_pointset(&r, &ps);
                        for( int c = 0; c < ps.n; ++c )
                        {
                            //ps.locations[c * 3    ] = pout[count];
                            //ps.locations[c * 3 + 1] = pout[count+1];
                            //ps.locations[c * 3 + 2] = pout[count+2];
                            double ptemp[3];
                            ptemp[0] = pout[count];
                            ptemp[1] = pout[count+1];
                            ptemp[2] = pout[count+2];
                            m_dig3.get_spaces()[1]->set_form_pointset_location( i, c * 3, ptemp );
                            count+=3;
                        }


                        if( ps.n <= numSemi )
                        {
                            // activate the semilandmark patch
                            for( int c = 0; c < m_targetTopLandmarks->childCount(); ++c )
                            {
                                ViewTreeItem* vti2 = (ViewTreeItem*)m_targetTopLandmarks->child( c );
                                if( vti2->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
                                {
                                    if( ((SemiLandmarksTopItem*)vti2)->getLmkID() == ((SemiLandmarksTopItem*)vti)->getLmkID() )
                                    {
                                        m_targetTopLandmarks->assignLandmarksHere( vti2 );
                                        break;
                                    }
                                }
                            }
                            for( int i = ps.n; i < numSemi; ++i )
                            {
                                m_targetView->placePoint( pout[count], pout[count+1], pout[count+2], false, false, "", EW_Form3::STATE_WARPED );
                                count+=3;
                            }
                            m_targetTopLandmarks->assignLandmarksHere( 0 );
                        }
                    } // end if check id
                } // for loop
            }// end if existing semilandmark patch
            else
            {
                m_targetTopLandmarks->addSurfaceSemiLandmark( tt.c_str() );
                m_targetTopLandmarks->assignLandmarksHere( (ViewTreeItem*)m_targetTopLandmarks->child( m_targetTopLandmarks->childCount() - 1 ) );
                for( int c = 0; c < numSemi; ++c )
                {
                    m_targetView->placePoint( pout[count], pout[count+1], pout[count+2], false, false, "", EW_Form3::STATE_WARPED );
                    count+=3;
                }
                m_targetTopLandmarks->assignLandmarksHere( 0 );
            }

            delete [] pout; pout = 0;
            delete [] pin; pin = 0;

            // mapped successfully
            QString msg, num;
            num.setNum( semiInd );
            msg = "Warped ";
            msg += num;
            msg += " semilandmarks. ";
            emit status( msg );
        } //end semilandmarks


        // add the landmark states
        updateTreeViewStates( 1 );
    }
    catch( ... )
    {
    }
}

void TableauLayout::adjustLandmarkState()
{
    // add the landmark states
    updateTreeViewStates( 1 );
}

void TableauLayout::matchLmk(FormItem* item, int index)
{
    if( m_regular )
        do_regular_match( item, index );
    else
        do_ew_match( item, index );
}

void TableauLayout::do_regular_match(FormItem* item, int index)
{
#if TEMPLAND_TOOLKIT_BUILD
    int size = m_templateTopLandmarks->childCount();
    QStringList sl;
    for( int i = 0; i < size; ++i )
    {
        ViewTreeItem* vti = (ViewTreeItem*)m_templateTopLandmarks->child( i );
        if( vti->getType() != ViewTreeItem::LANDMARK_ITEM )
            continue;
        sl.push_back( ((LandmarkItem*)m_templateTopLandmarks->child( i ))->getLmkID() );
    }

    if( size == 0 )
        return;

    if( sl.size() )
    {
//        MapListDialog* mapDialog = new MapListDialog(this, sl);
//        if( mapDialog->exec() )
//        {
            // get the template landmark
            int size = m_targetTopLandmarks->childCount();
            for( int i = 0; i < size; ++i )
            {
                ViewTreeItem* vti = (ViewTreeItem*)m_targetTopLandmarks->child( i );
                if( vti->getType() != ViewTreeItem::LANDMARK_ITEM )
                    continue;
                LandmarkItem* lmi = (LandmarkItem*)m_targetTopLandmarks->child( i );
                if( lmi->getLmkIndex() == index )
                {
                    std::string tt = lmi->getLmkID().toStdString();

                    if( !sl.contains( lmi->getLmkID() ) )
                    {
                        QMessageBox::critical( this, "Error", "Specimen Landmark name does not match any Templand Landmarks." );
                        return;
                    }

                    double x1 = 0, y1 = 0, z1 = 0;
                    int index;
                    bool r1 = getPointsetLocation( tt, 1, x1, y1, z1, index );
                    double x2 = 0, y2 = 0, z2 = 0;
                    bool r2 = getPointsetLocation( tt, 0, x2, y2, z2, index );
                    if( r1 && r2 )
                    {
                        MappedLmk ml;
                        ml.templateID = tt;
                        ml.targetID = tt;
                        ml.tempX = x2; ml.tempY = y2; ml.tempZ = z2;
                        ml.tarX = x1; ml.tarY = y1; ml.tarZ = z1;
                        m_regLmks.push_back( ml );

                        // mapped successfully
                        QString msg;
                        msg = "Template: ";
                        msg += lmi->getLmkID();
                        msg += " and Specimen: ";
                        msg += lmi->getLmkID();
                        msg += " Landmarks have been matched.";
                        emit status( msg );

                        m_targetTopLandmarks->enableRegChild( index, false );
                    }
                    break;
                }
            } // end for loop
//        } // end dialog
//        delete mapDialog;
    }
    else
    {
        QMessageBox::critical(this, "Error", "Add some Specimen landmarks before registered.");
    }
#endif
}

void TableauLayout::do_ew_match(FormItem* item, int index)
{
    int size = m_templateTopLandmarks->childCount();
    QStringList sl;
    for( int i = 0; i < size; ++i )
    {
        ViewTreeItem* vti = (ViewTreeItem*)m_templateTopLandmarks->child( i );
        if( vti->getType() != ViewTreeItem::LANDMARK_ITEM )
            continue;
        sl.push_back( ((LandmarkItem*)m_templateTopLandmarks->child( i ))->getLmkID() );
    }

    if( size == 0 )
        return;

    if( sl.size() )
    {
//        MapListDialog* mapDialog = new MapListDialog(this, sl);
//        if( mapDialog->exec() )
//        {
            // get the template landmark
            int size = m_targetTopLandmarks->childCount();
            for( int i = 0; i < size; ++i )
            {
                ViewTreeItem* vti = (ViewTreeItem*)m_targetTopLandmarks->child( i );
                if( vti->getType() != ViewTreeItem::LANDMARK_ITEM )
                    continue;
                LandmarkItem* lmi = (LandmarkItem*)m_targetTopLandmarks->child( i );
                if( lmi->getLmkIndex() == index )
                {
                    std::string tt = lmi->getLmkID().toStdString();

                    if( !sl.contains( lmi->getLmkID() ) )
                    {
                        QMessageBox::critical( this, "Error", "Specimen Landmark name does not match any Templand Landmarks." );
                        return;
                    }

                    double x1 = 0, y1 = 0, z1 = 0;
                    int targetindex = 0, templateindex = 0;
                    bool r1 = getPointsetLocation( tt, 1, x1, y1, z1, targetindex );
                    double x2 = 0, y2 = 0, z2 = 0;
                    bool r2 = getPointsetLocation( tt, 0, x2, y2, z2, templateindex );
                    if( r1 && r2 )
                    {
//                        MappedLmk ml;
//                        ml.templateID = tt;
//                        ml.targetID = tt;
//                        ml.tempX = x2; ml.tempY = y2; ml.tempZ = z2;
//                        ml.tarX = x1; ml.tarY = y1; ml.tarZ = z1;

                        const EW_Dataflow_Spline3* ds = m_dig3.get_spline_node();
                        int result1 = ds->lmk_index( 1, targetindex, 0 );
                        int result0 = ds->lmk_index( 0, templateindex, 0 );
                        if( result1 != -1 && result0 != -1 )
                        {
                        //m_regLmks.push_back( ml );

                        // mapped successfully
                        QString msg;
                        msg = "Template: ";
                        msg += lmi->getLmkID();
                        msg += " and Specimen: ";
                        msg += lmi->getLmkID();
                        msg += " Landmarks have been matched.";
                        emit status( msg );

                        m_targetTopLandmarks->enableRegChild( index, false );
                        }
                        else
                        {
                        // not mapped successfully
                        QString msg;
                        msg = "Template: ";
                        msg += lmi->getLmkID();
                        msg += " and Specimen: ";
                        msg += lmi->getLmkID();
                        msg += " Landmarks has not been matched.";
                        emit status( msg );
                        }
                    }
                    break;
                }
            } // end for loop
//        } // end dialog
//        delete mapDialog;
    }
    else
    {
        QMessageBox::critical(this, "Error", "Add some Specimen landmarks before registered.");
    }
}

void TableauLayout::matchAllLmk()
{
    if( m_regular )
        do_regular_matchall();
    else
        do_ew_matchall();
}

void TableauLayout::do_regular_matchall()
{
#if TEMPLAND_TOOLKIT_BUILD
    m_regLmks.clear();

    int size = m_templateTopLandmarks->childCount();
    QStringList sl;
    for( int i = 0; i < size; ++i )
    {
        ViewTreeItem* vti = (ViewTreeItem*)m_templateTopLandmarks->child( i );
        if( vti->getType() != ViewTreeItem::LANDMARK_ITEM )
            continue;
        sl.push_back( ((LandmarkItem*)m_templateTopLandmarks->child( i ))->getLmkID() );
    }

    if( size == 0 )
        return;

    for( int i = 0; i < m_targetTopLandmarks->childCount(); ++i )
    {
        ViewTreeItem* vti = (ViewTreeItem*)m_targetTopLandmarks->child( i );
        if( vti->getType() != ViewTreeItem::LANDMARK_ITEM )
            continue;
        QString lbl = ((LandmarkItem*)vti)->getLmkID();

        if( sl.contains( lbl ) )
        {
            double x1, y1, z1;
            double x2, y2, z2;
            int index;
            bool r1 = getPointsetLocation( lbl.toStdString(), 0, x1, y1, z1, index );
            bool r2 = getPointsetLocation( lbl.toStdString(), 1, x2, y2, z2, index );

            if( r1 && r2 )
            {
                MappedLmk ml;
                ml.templateID = lbl.toStdString();
                ml.targetID = ml.templateID;
                ml.tempX = x1; ml.tempY = y1; ml.tempZ = z1;
                ml.tarX = x2; ml.tarY = y2; ml.tarZ = z2;
                m_regLmks.push_back( ml );
            }
        }
    }

    if( m_regLmks.size() >= 4 )
    {
        QString num; num = num.setNum( m_regLmks.size() );
        emit status( num + " landmarks have been matched." );
    }
#endif
}

void TableauLayout::do_ew_matchall()
{
    //m_regLmks.clear();
    std::vector<double> template_x;
    std::vector<double> template_y;
    std::vector<double> template_z;
    std::vector<double> target_x;
    std::vector<double> target_y;
    std::vector<double> target_z;

    int size = m_templateTopLandmarks->childCount();
    QStringList sl;
    for( int i = 0; i < size; ++i )
    {
        ViewTreeItem* vti = (ViewTreeItem*)m_templateTopLandmarks->child( i );
        if( vti->getType() != ViewTreeItem::LANDMARK_ITEM )
            continue;
        sl.push_back( ((LandmarkItem*)m_templateTopLandmarks->child( i ))->getLmkID() );
    }

    if( size == 0 )
        return;

    int counter = 0;
    for( int i = 0; i < m_targetTopLandmarks->childCount(); ++i )
    {
        LandmarkItem* lti = (LandmarkItem*)m_targetTopLandmarks->child( i );
        if( lti->getType() != ViewTreeItem::LANDMARK_ITEM )
            continue;
        QString lbl = lti->getLmkID();

        if( sl.contains( lbl ) )
        {
            double x1, y1, z1;
            double x2, y2, z2;
            int targetindex = 0, templateindex = 0;
            bool r1 = getPointsetLocation( lbl.toStdString(), 0, x1, y1, z1, templateindex );
            bool r2 = getPointsetLocation( lbl.toStdString(), 1, x2, y2, z2, targetindex );


            if( r1 && r2 )
            {
                // now make sure the target landmark is FIXED for it to be matched
                if( m_dig3.get_spaces()[1]->get_form_data()->pointsets[targetindex].state != EW_Form3::STATE_SET )
                    continue;

                const EW_Dataflow_Spline3* ds = m_dig3.get_spline_node();
                int result0 = ds->lmk_index( 0, templateindex, 0 );
//#if TEMPLAND_TOOLKIT_BUILD
//                QString nr; nr.setNum( result0 );
//                QString nr2; nr2.setNum( templateindex );
//                QString nr3 = "lmk_index( 0, " + nr2 + ", 0 ); RETURNS " + nr;
//                Writer::Instance().writeline( nr3.toStdString().c_str() );
//#endif
                int result1 = ds->lmk_index( 1, targetindex, 0 );
//#if TEMPLAND_TOOLKIT_BUILD
//                QString nr4; nr4.setNum( result1 );
//                QString nr5; nr5.setNum( targetindex );
//                 QString nr6 = "lmk_index( 1, " + nr5 + ", 0 ); RETURNS " + nr4;
//                Writer::Instance().writeline( nr6.toStdString().c_str() );
//
//#endif
                if( result1 >= 0 && result0 >= 0 )
                {
                    template_x.push_back(x1);
                    template_y.push_back(y1);
                    template_z.push_back(z1);
                    target_x.push_back(x2);
                    target_y.push_back(y2);
                    target_z.push_back(z2);

                    ++counter;
                }
                else
                {
                }
            }
        } // end if contains
    }

#if TEMPLAND_TOOLKIT_BUILD
    if (counter > 0)
    {
        m_splineTemplateLmks->ReSize(counter,3);
        m_splineTargetLmks->ReSize(counter,3);

        for (int i=0;i<counter;i++)
        {
            (*m_splineTemplateLmks)[i][0] = template_x[i];
            (*m_splineTemplateLmks)[i][1] = template_y[i];
            (*m_splineTemplateLmks)[i][2] = template_z[i];

            (*m_splineTargetLmks)[i][0] = target_x[i];
            (*m_splineTargetLmks)[i][1] = target_y[i];
            (*m_splineTargetLmks)[i][2] = target_z[i];
        }
    }
#endif
    if( counter >= 4 )
    {
        QString num; num = num.setNum( counter );
        emit status( num + " landmarks have been matched." );
    }
    else if( counter == 0 )
    {
        emit status( "No landmarks have been matched." );
    }
}

bool TableauLayout::getPointsetLocation( const std::string& id, int space, double &x, double& y, double& z, int& index )
{
    bool result = false;

    const EW_Form3* form = m_dig3.get_spaces()[space]->get_form_data();
    if( form != 0 )
        for(unsigned int j=0; j<form->pointsets.size() && !result; ++j)
            if(form->pointsets[j].type == EW_Form3::TYPE_LANDMARK || form->pointsets[j].type == EW_Form3::TYPE_SEMI_LANDMARK)
            {
                if( form->pointsets[j].id == id )
                {
                    x = form->pointsets[j].locations[0];
                    y = form->pointsets[j].locations[1];
                    z = form->pointsets[j].locations[2];
                    index = j;
                    result = true;
                }
            }

    return result;
}

void TableauLayout::computeSpline()
{
    if( m_regular == false )
    {
        m_templateTopLandmarks->mapAllEnable( true );

        int size = m_templateTopLandmarks->childCount();
        for( int i = 0; i < size; ++i )
        {
            ViewTreeItem* vti = (ViewTreeItem*)m_templateTopLandmarks->child( i );
            if( vti->getType() == ViewTreeItem::LANDMARK_ITEM )
            {
                LandmarkItem* li = (LandmarkItem*)m_templateTopLandmarks->child( i );
                li->mapEnable( true );
            }
            else if( vti->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
            {
                SemiLandmarksTopItem* sli = (SemiLandmarksTopItem*)m_templateTopLandmarks->child( i );
                sli->mapEnable( true );
            }
        }
        return;
    }

#if TEMPLAND_TOOLKIT_BUILD
    if( !m_regular )
        return;

    if( m_regLmks.size() < 4 )
    {
        QString msg = QString("Not enough mapped landmarks, you currently have %l, you need atleast 4 mapped landmarks.").arg(m_regLmks.size());
        QMessageBox::critical(this, "Error", msg );
        return;
    }

    if( m_tps )
        delete m_tps;
    m_tps = new TPS;

    Matrix<double> templates;
    Matrix<double> targets;

    templates.ReSize( m_regLmks.size(), 3 );
    targets.ReSize( m_regLmks.size(), 3 );

    for( unsigned int i = 0; i < m_regLmks.size(); ++i )
    {
        MappedLmk ml = m_regLmks[i];
        templates.set( i, 0, ml.tempX );
        templates.set( i, 1, ml.tempY );
        templates.set( i, 2, ml.tempZ );

        targets.set( i, 0, ml.tarX );
        targets.set( i, 1, ml.tarY );
        targets.set( i, 2, ml.tarZ );

    }

    m_tps->LoadData( templates.GetMatrix(), targets.GetMatrix(), m_regLmks.size(), 3 );
    m_tps->PerformTPS();

    QString msg = "[Templand Node] ";
    QString msg2;
    bool cal = false;
    bool info = true;
    if( cal = m_tps->IsCalculated() )
        msg2 = "TPS calculated successfully.";
    else
    {
        msg2 = "TPS not calculated successfully.";
        info = false;
    }

    if( cal )
    {
        m_templateTopLandmarks->mapAllEnable( true );

        int size = m_templateTopLandmarks->childCount();
        for( int i = 0; i < size; ++i )
        {
            ViewTreeItem* vti = (ViewTreeItem*)m_templateTopLandmarks->child( i );
            if( vti->getType() != ViewTreeItem::LANDMARK_ITEM )
                continue;
            LandmarkItem* li = (LandmarkItem*)m_templateTopLandmarks->child( i );
            li->mapEnable( true );
        }
    }

    emit status( msg2 );
#if TEMPLAND_TOOLKIT_BUILD
    Logger::getInstance()->log( msg + msg2, info ? Logger::INFO : Logger::WARNING );
#else
    QMessageBox::warning( this, "Compute Spline", msg + msg2 );
#endif

#endif
}

void TableauLayout::matchSpline()
{
    matchAllLmk();
    computeSpline();
}

void TableauLayout::matchSplineWarp()
{
    matchAllLmk();
    computeSpline();
    mapAllLmk( 0, -1 );
}

void TableauLayout::pointType( int type)
{
    m_templateView->pointType( type );
    m_templateSliceView->pointType( type );
    m_targetView->pointType( type );
    m_targetSliceView->pointType( type );
}

void TableauLayout::changeBckgndColor()
{
    const unsigned char* rgb = m_targetView->getDig3View()->widget->get_background_color();
    const unsigned char* col = chooseColor(rgb);
    if(col && m_targetView->getDig3View())
    {
        m_targetView->getDig3View()->widget->set_background_color(col);
        m_targetSliceView->getDig3View()->widget->set_background_color(col);
        m_templateView->getDig3View()->widget->set_background_color(col);
        m_templateSliceView->getDig3View()->widget->set_background_color(col);
        delete[] col;

        tableauUpdated();
    }
}

void TableauLayout::loadViewTreeItems(const FormItem* formTreeItem, const EW_Form3& form)
{
    m_treesFull = false;
    for(int i=0; i<formTreeItem->childCount(); ++i)
    {
        ViewTreeItem* treeItem = (ViewTreeItem*)formTreeItem->child(i);
        switch(treeItem->getType())
        {
            case ViewTreeItem::SURFACES_ITEM:
            {
                SurfaceTopItem* surTopItem = (SurfaceTopItem*)treeItem;
                for(unsigned int j=0; j<form.surfaces.size(); ++j)
                    surTopItem->addSurface(form.surfaces[j].id.c_str(), form.surfaces[j].file.c_str(), true);
            }
            break;
            case ViewTreeItem::LANDMARKS_ITEM:
            {
                LandmarksTopItem* lmkTopItem = (LandmarksTopItem*)treeItem;

                for(unsigned int j=0; j<form.pointsets.size(); ++j)
                {
                    if( form.pointsets[j].type == EW_Form3::TYPE_LANDMARK )
                    {
                        lmkTopItem->assignLandmarksHere( 0 );
                        lmkTopItem->addLandmarkItem(QString(form.pointsets[j].id.c_str()));
                    }
                    else if( form.pointsets[j].type == EW_Form3::TYPE_SEMI_LANDMARK )
                    {
                        bool surfacePointset = true;
                        for( unsigned int k = 0; k < form.pointsets[j].relax_dims.size(); ++k )
                        {
                            if( form.pointsets[j].relax_dims[k] == 2 )
                            {
                                surfacePointset = true;
                                break;
                            }
                            else if( form.pointsets[j].relax_dims[k] == 1 )
                            {
                                surfacePointset = false;
                                break;
                            }
                        }
                        if( surfacePointset )
                        {
                            lmkTopItem->addSurfaceSemiLandmark( form.pointsets[j].id.c_str() );
                            lmkTopItem->assignLandmarksHere( (ViewTreeItem*)lmkTopItem->child( lmkTopItem->childCount() - 1 ) );
                        }
//                        else
//                            lmkTopItem->addCurveSemiLandmark();

                        unsigned int count = form.pointsets[j].locations.size();
                        if( count )
                            count /= 3;
                        for( unsigned int k = 0; k < count; ++k )
                            lmkTopItem->addLandmarkItem( "" );
                    }
                } // end for loop
            }
            break;
            default: break;
        }
    }

    m_treesFull = true;
}

void TableauLayout::targetFormUpdated()
{
    formUpdated( m_targetFormItem );
}

void TableauLayout::formUpdated(FormItem* formTreeItem)
{
    if(formTreeItem==m_targetFormItem && m_targetSaved)
    {
        m_targetSaved = false;
        m_trgtGrpBox->setTitle(m_trgtGrpBox->title()+"*");
        formTreeItem->setText(0, formTreeItem->text(0)+"*");
    }
    else if(formTreeItem==m_templateFormItem && m_templateSaved)
    {
        m_templateSaved = false;
        m_srcGrpBox->setTitle(m_srcGrpBox->title()+"*");
        formTreeItem->setText(0, formTreeItem->text(0)+"*");
    }
}

void TableauLayout::filmstripFrameNumberChanged( float number )
{
    QString str;
    str.setNum( number );
    m_frameCountLBL->setText( str );

    // update the form tree view with template and target forms
    int num = (int)number;
    m_templateFile = m_tableauList[num-1].space[0].form_filename.c_str();
    m_targetFile = m_tableauList[num-1].space[1].form_filename.c_str();
    loadForm( m_templateFormItem, true, false, false );
    loadForm( m_targetFormItem, true, false, false );

    // update the landmark views checkbox
    bool v = m_tableauList[num-1].space[1].show_lmks_in_main;
    bool s = m_tableauList[num-1].space[1].show_lmks_in_slice;
    m_targetView->getDig3View()->get_landmarks_item()->set_state(v);
    m_targetSliceView->getDig3View()->get_landmarks_item()->set_state(s);
    m_targetTopLandmarks->setCheckState( 1, v ? Qt::Checked : Qt::Unchecked );
    m_targetTopLandmarks->setCheckState( 2, s ? Qt::Checked : Qt::Unchecked );
    // do same for template
    v = m_tableauList[num-1].space[0].show_lmks_in_main;
    s = m_tableauList[num-1].space[0].show_lmks_in_slice;
    m_templateView->getDig3View()->get_landmarks_item()->set_state(v);
    m_templateSliceView->getDig3View()->get_landmarks_item()->set_state(s);
    m_templateTopLandmarks->setCheckState( 1, v ? Qt::Checked : Qt::Unchecked );
    m_templateTopLandmarks->setCheckState( 2, s ? Qt::Checked : Qt::Unchecked );

    // update the surface view checkbox
    v = s = true;
    if( m_tableauList[num-1].space[1].surface_settings.size() > 0 )
    {
        v = m_tableauList[num-1].space[1].surface_settings[0].show_in_main;
        s = m_tableauList[num-1].space[1].surface_settings[0].show_in_slice;
    }
    m_targetTopSurface->setCheckState( 1, v ? Qt::Checked : Qt::Unchecked );
    m_targetTopSurface->setCheckState( 2, s ? Qt::Checked : Qt::Unchecked );
    // do the same for target
    v = s = true;
    if( m_tableauList[num-1].space[0].surface_settings.size() > 0 )
    {
        v = m_tableauList[num-1].space[0].surface_settings[0].show_in_main;
        s = m_tableauList[num-1].space[0].surface_settings[0].show_in_slice;
    }
    m_templateTopSurface->setCheckState( 1, v ? Qt::Checked : Qt::Unchecked );
    m_templateTopSurface->setCheckState( 2, s ? Qt::Checked : Qt::Unchecked );
}

void TableauLayout::resetForm(FormItem* formTreeItem)
{
    int spaceIndex = formTreeItem==m_targetFormItem ? 1:0;

    m_dig3.get_spaces()[spaceIndex]->reset_form();

    m_treesFull = false;
    for(int i=0; i<formTreeItem->childCount(); ++i)
    {
        foreach(QTreeWidgetItem* item, formTreeItem->child(i)->takeChildren())
            delete item;
    }

    // reset the form text
    formTreeItem->setText( 0, "Form" );

    // reset the name in the specimen group or template group
    if( spaceIndex == 0 )
    {
        m_srcGrpBox->setTitle( "Template" );
    }
    else
    {
        m_trgtGrpBox->setTitle( "Specimen" );
    }

    formUpdated(formTreeItem);
    m_treesFull = true;
}

void TableauLayout::showFilmstrip( bool tog )
{
    m_filmShow->setText( tog ? "-" : "+" );
}

void TableauLayout::loadForm(FormItem* formTreeItem, bool loaded, bool focus, bool resetLmkViewState )
{
    QSettings settings("Evan", "Templand");
    QStringList fileSettings = settings.value("lastDirectory").toStringList();
    QString fsLastDir = "";
    if( fileSettings.size() > 0 )
        fsLastDir = fileSettings[0];

    QString fileName;
    if( loaded )
    {
        fileName = ( formTreeItem == m_targetFormItem ? m_targetFile : m_templateFile );
    }
    else
    {
        QFileDialog fileDialog(this, tr("Open a Form File"), fsLastDir, tr("Form files (*.frm)"));
        if( fileDialog.exec() )
        {
            fileName = fileDialog.selectedFiles()[0];
            QDir dir = fileDialog.directory();
            settings.setValue("lastDirectory",dir.absolutePath());
        }
    }

    if(fileName == "" || fileName.isEmpty())
        return;

    QFile file(fileName);
    if(!file.exists())
    {
#if TEMPLAND_TOOLKIT_BUILD
        Logger::getInstance()->log(QString("[Templand Node] Failed to load form! File '%1' does not exist!").arg(fileName), Logger::RUN_ERROR);
#else
        QMessageBox::critical( this, "Error", QString("[Templand Node] Failed to load form! File '%1' does not exist!").arg(fileName) );
#endif
        return;
    }
    int spaceIndex = formTreeItem==m_targetFormItem ? 1:0;

#ifdef _WIN32
        // for windows change the path slashes from forward to backward
        QString str = fileName.replace( '/', '\\' );
#else
        QString str = fileName.replace( '\\', '/' );
#endif

    EW_Form3 form;
    try
    {
        resetForm(formTreeItem);
        QApplication::setOverrideCursor(Qt::WaitCursor);
        emit status(QString("Loading Form, Please wait..."));
        form.read_file(fileName.toStdString().c_str());
        m_dig3.get_spaces()[spaceIndex]->set_form_data(&form);

        loadViewTreeItems(formTreeItem, form);

        // focus view for the first surface only
        if (m_dig3.get_spaces()[spaceIndex]->get_form_data()->surfaces.size() == 1) //Focus on first surface
        {
            bool found = false;
            for(int i=0; i<formTreeItem->childCount() || !found; ++i)
            {
                ViewTreeItem* treeItem = (ViewTreeItem*)formTreeItem->child(i);
                switch(treeItem->getType())
                {
                    case ViewTreeItem::SURFACES_ITEM:
                    {
                        SurfaceTopItem* surTopItem = (SurfaceTopItem*)treeItem;

                        if( surTopItem->childCount() <= 0 )
                            break;

                        SurfaceItem* sur = (SurfaceItem*)surTopItem->child( 0 );

                        sur->setSurfaceIndex(0);
                        if( focus )
                            focusView(formTreeItem, sur);

                        found = true;
                    }
                    break;
                    default: break;
                }
            }
        }

        emit status(QString("Form '%1' Loaded").arg(fileName), 2000);
        QString formID = fileName;//.right(fileName.length() - (fileName.lastIndexOf(QRegExp("[\\/]"))+1));
        if(spaceIndex)
        {
            m_targetFile = fileName;
            m_targetSaved = true;
            m_trgtGrpBox->setTitle(QString("Specimen - (%1)").arg(m_targetFile));
        }
        else
        {
            m_templateFile = fileName;
            m_templateSaved = true;
            m_srcGrpBox->setTitle(QString("Template - (%1)").arg(m_templateFile));
        }
        formTreeItem->setText(0, QString("Form (%1)").arg(formID));

        // add the landmark states
        updateTreeViewStates( spaceIndex );

        // refresh the views
        if( resetLmkViewState )
        {
            if( formTreeItem == m_targetFormItem )
            {
                m_targetView->getDig3View()->get_landmarks_item()->set_state(false);
                m_targetSliceView->getDig3View()->get_landmarks_item()->set_state(false);
                m_targetView->getDig3View()->get_landmarks_item()->set_state(true);
                m_targetSliceView->getDig3View()->get_landmarks_item()->set_state(true);

                m_targetTopSurface->setCheckState( 1, Qt::Checked );
                m_targetTopSurface->setCheckState( 2, Qt::Checked );

                m_targetTopLandmarks->setCheckState( 1, Qt::Checked );
                m_targetTopLandmarks->setCheckState( 2, Qt::Checked );
            }
            else
            {
                m_templateView->getDig3View()->get_landmarks_item()->set_state(false);
                m_templateSliceView->getDig3View()->get_landmarks_item()->set_state(false);
                m_templateView->getDig3View()->get_landmarks_item()->set_state(true);
                m_templateSliceView->getDig3View()->get_landmarks_item()->set_state(true);

                m_templateTopLandmarks->setCheckState( 1, Qt::Checked );
                m_templateTopLandmarks->setCheckState( 2, Qt::Checked );

                m_templateTopSurface->setCheckState( 1, Qt::Checked );
                m_templateTopSurface->setCheckState( 2, Qt::Checked );
            }
        }
    }
    catch(EW_Form3_IOError ex)
    {
        QMessageBox::critical(this, "IO Exception", QString("Failed to load form!\n- %1)").arg(ex.what()));
    }
    catch(std::exception &ex)
    {
        QMessageBox::critical(this, "IO Exception", QString("Failed to load form!\n- %1)").arg(ex.what()));
    }
    QApplication::restoreOverrideCursor();
}

void TableauLayout::saveFormAs(FormItem* formTreeItem)
{
    QString oldFileName = formTreeItem==m_targetFormItem ? m_targetFile : m_templateFile;
    if(formTreeItem == m_targetFormItem) m_targetFile = "";
    else               m_templateFile = "";

    if(!saveForm(formTreeItem))
    {
        if(formTreeItem == m_targetFormItem) m_targetFile = oldFileName;
        else               m_templateFile = oldFileName;
    }
}

bool TableauLayout::saveForm(FormItem* formTreeItem)
{
    int spaceIndex = formTreeItem==m_targetFormItem ? 1:0;

    QString fileName = spaceIndex? m_targetFile : m_templateFile;

    if(fileName == "" || fileName.isEmpty())
    {
        fileName =      QFileDialog::getSaveFileName(this, tr("Save Form"),
                                                            "Untitled.frm",
                                                             "Form files (*.frm)");
        if(fileName == "" || fileName.isEmpty())
            return false;
    }

    try
    {
        QApplication::setOverrideCursor(Qt::WaitCursor);
        emit status(QString("Saving Form, Please wait..."));

        //QFileInfo fi( fileName.left(fileName.length() - (fileName.lastIndexOf(QRegExp("[\\/]"))+1) ) );
#ifdef _WIN32
        // for windows change the path slashes from forward to backward
        QString str = fileName.replace( '/', '\\' );
#else
        QString str = fileName.replace( '\\', '/' );
#endif

        m_dig3.get_spaces()[spaceIndex]->get_form_data()->write_file( str.toStdString().c_str(), 0 );
        emit status(QString("Form %1 Saved").arg(fileName), 2000);
        if(spaceIndex)
        {
            m_targetFile = fileName;
            m_targetSaved = true;
            //m_savedAtleastOnce = true;
            m_trgtGrpBox->setTitle(QString("Specimen - (%1)").arg(m_targetFile));
        }
        else
        {
            m_templateFile = fileName;
            m_templateSaved = true;
            //m_savedAtleastOnce = true;
            m_srcGrpBox->setTitle(QString("Template - (%1)").arg(m_templateFile));
        }
        QString formID = fileName;//.right(fileName.length() - (fileName.lastIndexOf(QRegExp("[\\/]"))+1));
        formTreeItem->setText(0, QString("Form (%1)").arg(formID));
    }
    catch(EW_Form3_IOError ex)
    {
        QMessageBox::critical(this, "IO Exception", QString("Failed to save form!\n- %1)").arg(ex.what()));
        QApplication::restoreOverrideCursor();
        return false;
    }
    catch(std::exception &ex)
    {
        QMessageBox::critical(this, "IO Exception", QString("Failed to save form!\n- %1)").arg(ex.what()));
        QApplication::restoreOverrideCursor();
        return false;
    }

    QApplication::restoreOverrideCursor();

    //m_filmstripAdd->setDisabled( false );
    emit appendForm( true );

    return true;
}

void TableauLayout::newTableau()
{
    if( m_newTableau )
        return;

    m_newTableau = true;

    QDir d;

    QString s = d.absolutePath();

    s += "/Untitled.ltb";

//#ifdef _WIN32
//        // for windows change the path slashes from forward to backward
//        s = s.replace( '/', '\\' );
//#else
//        s = s.replace( '\\', '/' );
//#endif

    QObject::connect(&m_filmstrip, SIGNAL(frameIndexChange(int)), m_frameSlider, SLOT(setValue(int)));
    QObject::connect(&m_filmstrip, SIGNAL(frameIndexChange(int)), m_frameSBox, SLOT(setValue(int)));
    QObject::connect(&m_filmstrip, SIGNAL(frameNumberChange(float)), this, SLOT(filmstripFrameNumberChanged(float)));
    QObject::connect(m_frameSlider, SIGNAL(sliderMoved(int)), &m_filmstrip, SLOT(moveToFrame(int)));
    QObject::connect(m_frameSBox, SIGNAL(valueChanged(int)), &m_filmstrip, SLOT(moveToFrame(int)));
    QObject::connect(&m_filmstrip, SIGNAL(statusChanged(int)), this, SLOT(filmstripStatus(int)));

    //m_filmstripAdd->setDisabled( false );
    emit appendForm( true );
}

bool TableauLayout::loadTableau(QString loadedFile, bool loaded)
{
    bool result = true;
    QString fileName = loadedFile;

    QFile file(fileName);
    if(!file.exists())
    {
#if TEMPLAND_TOOLKIT_BUILD
        Logger::getInstance()->log(QString("[Templand Node] Failed to load tableau! File '%1' does not exist!").arg(fileName), Logger::RUN_ERROR);
#else
        QMessageBox::critical( this, "Error", QString("[Templand Node] Failed to load tableau! File '%1' does not exist!").arg(fileName) );
#endif
        result = false;
        return result;
    }

    try
    {
#ifdef _WIN32
        // for windows change the path slashes from forward to backward
        fileName = fileName.replace( '/', '\\' );
#else
        fileName = fileName.replace( '\\', '/' );
#endif
        std::vector<EW_Dig3_Tableau> tempTableaus;
        EW_Dig3_Tableau::read_file(&tempTableaus, fileName.toStdString().c_str());
        m_tableauList.clear();
        m_tableauList = tempTableaus;

        TableauDialog* loadDialog = new TableauDialog(this, fileName, tempTableaus.size());
        if(loaded ? true : loadDialog->exec())
        {
            QApplication::setOverrideCursor(Qt::WaitCursor);
            emit status(QString("Loading Tableau, Please wait..."));

            if( tempTableaus.size() > 0 )
            {
                if(loadDialog->loadTemplate())
                {
                    m_templateFile = tempTableaus[0].space[0].form_filename.c_str();
                    /*if(!m_templateFile.contains('\\') && !m_templateFile.contains('/')) //Relative form path
                        m_templateFile = fileName.left(fileName.lastIndexOf(QRegExp("[\\/]"))+1) + m_templateFile;*/
                    loadForm(m_templateFormItem, true);

                    // check if the template has surface or landmarks. If so then display the template views
                    const EW_Form3* form = m_dig3.get_spaces()[0]->get_form_data();
                    if( form->surfaces.size() || form->pointsets.size() )
                    {
                        m_srcGrpBox->setVisible(true);
                        m_templateItemTree->setVisible(true);
                    }
                }
                if(loadDialog->loadTarget())
                {
                    m_targetFile = tempTableaus[0].space[1].form_filename.c_str();
                    /*if(!m_targetFile.contains('\\') && !m_targetFile.contains('/')) //Relative form path
                        m_targetFile = fileName.left(fileName.lastIndexOf(QRegExp("[\\/]"))+1) + m_targetFile;*/
                    loadForm(m_targetFormItem, true);
                }
                if(loadDialog->loadSettings())
                {
                    m_dig3.load_tableau(&tempTableaus[0], 0, EW_Dig3::TABLEAU_SETTINGS );
                    m_dig3.load_tableau(&tempTableaus[0], 1, EW_Dig3::TABLEAU_SETTINGS );
                    m_dig3.load_tableau(&tempTableaus[0], 2, EW_Dig3::TABLEAU_SETTINGS );
                    m_dig3.load_tableau(&tempTableaus[0], 3, EW_Dig3::TABLEAU_SETTINGS );
                }
                if(loadDialog->loadView())
                {
                    m_dig3.load_tableau(&tempTableaus[0], 0, EW_Dig3::TABLEAU_VIEW );
                    m_dig3.load_tableau(&tempTableaus[0], 1, EW_Dig3::TABLEAU_VIEW );
                    m_dig3.load_tableau(&tempTableaus[0], 2, EW_Dig3::TABLEAU_VIEW );
                    m_dig3.load_tableau(&tempTableaus[0], 3, EW_Dig3::TABLEAU_VIEW );
                }
            }

            m_tableauFile = fileName;
            if(tempTableaus.size())
            {
                m_frameSlider->setMinimum(1);
                m_frameSlider->setMaximum(tempTableaus.size() > 0 ? tempTableaus.size() : 1);
                m_frameSBox->setMinimum(1);
                m_frameSBox->setMaximum(tempTableaus.size() > 0 ? tempTableaus.size() : 1);

                filmstripFrameNumberChanged( 1.0f );
            }
            setWindowTitle(QString("Templand - ") + m_tableauFile);
            m_tableauSaved = true;
            emit status(QString("File %1 Loaded").arg(m_tableauFile), 2000);

            if( tempTableaus.size() > 0 )
            {
                m_filmstrip.AssignContent( &m_dig3, tempTableaus );
            }
//
//            // update the landmark views checkbox
//            bool v = m_tableauList[0].space[1].show_lmks_in_main;
//            bool s = m_tableauList[0].space[1].show_lmks_in_slice;
//            m_targetTopLandmarks->setCheckState( 1, v ? Qt::Checked : Qt::Unchecked );
//            m_targetTopLandmarks->setCheckState( 2, s ? Qt::Checked : Qt::Unchecked );
//            // do same for template
//            v = m_tableauList[0].space[0].show_lmks_in_main;
//            s = m_tableauList[0].space[0].show_lmks_in_slice;
//            m_templateTopLandmarks->setCheckState( 1, v ? Qt::Checked : Qt::Unchecked );
//            m_templateTopLandmarks->setCheckState( 2, s ? Qt::Checked : Qt::Unchecked );


            QObject::connect(&m_filmstrip, SIGNAL(frameIndexChange(int)), m_frameSlider, SLOT(setValue(int)));
            QObject::connect(&m_filmstrip, SIGNAL(frameIndexChange(int)), m_frameSBox, SLOT(setValue(int)));
            QObject::connect(&m_filmstrip, SIGNAL(frameNumberChange(float)), this, SLOT(filmstripFrameNumberChanged(float)));
            QObject::connect(m_frameSlider, SIGNAL(sliderMoved(int)), &m_filmstrip, SLOT(moveToFrame(int)));
            QObject::connect(m_frameSBox, SIGNAL(valueChanged(int)), &m_filmstrip, SLOT(moveToFrame(int)));
            QObject::connect(&m_filmstrip, SIGNAL(statusChanged(int)), this, SLOT(filmstripStatus(int)));

            //m_filmstripAdd->setDisabled( false );
            //m_filmstripReplace->setDisabled( tempTableaus.size() < 1 );
            //m_filmstripDelete->setDisabled( tempTableaus.size() < 1 );
            emit appendForm( true );
            emit replaceForm( tempTableaus.size() );
            emit deleteForm( tempTableaus.size() );
            emit swapPrevForm( false );
            emit swapNextForm( tempTableaus.size() > 1 );
            m_tblStart->setDisabled( tempTableaus.size() < 2 );
            m_tblEnd->setDisabled( tempTableaus.size() < 2 );
            m_tblLeftE->setDisabled( tempTableaus.size() < 2 );
            m_tblLeft->setDisabled( tempTableaus.size() < 2 );
            m_tblRight->setDisabled( tempTableaus.size() < 2 );
            m_tblListRightE->setDisabled( tempTableaus.size() < 2 );

            m_filmFirstFrame->setDisabled( tempTableaus.size() < 2 );
            m_filmPlayBackward->setDisabled( tempTableaus.size() < 2 );
            m_filmPlay->setDisabled( tempTableaus.size() < 2 );
            m_filmPause->setDisabled( tempTableaus.size() < 2 );
            m_filmStop->setDisabled( tempTableaus.size() < 2 );
            m_filmLastFrame->setDisabled( tempTableaus.size() < 2 );

            m_filmStepPrev->setDisabled( tempTableaus.size() < 2 );
            m_filmStepNext->setDisabled( tempTableaus.size() < 2 );
            m_secFrameTXT->setDisabled( tempTableaus.size() < 2 );
            m_stepsFrameTXT->setDisabled( tempTableaus.size() < 2 );

            m_savedAtleastOnce = true;
        }
        else
            result = false;
        delete loadDialog;
    }
    catch(EW_Form3_IOError ex)
    {
        QMessageBox::critical(this, "IO Exception", QString("Failed to load tableau!\n- %1)").arg(ex.what()));
        result = false;
    }
    catch(std::exception &ex)
    {
        QMessageBox::critical(this, "IO Exception", QString("Failed to load tableau!\n- %1)").arg(ex.what()));
        result = false;
    }
    QApplication::restoreOverrideCursor();
    return result;
}

void TableauLayout::saveTableauAs()
{
    QString oldFile = m_tableauFile;
    m_tableauFile = "";
    if(!saveTableau())
        m_tableauFile = oldFile;
}

bool TableauLayout::saveTableau()
{
    if( !m_savedAtleastOnce )
        m_tableauFile = "";

    if(m_tableauFile == "" || m_tableauFile.isEmpty())
    {
        m_tableauFile = QFileDialog::getSaveFileName(this, tr("Save Tableau"),
                                                            "Untitled.ltb",
                                                             "Landmarking tableau files (*.ltb)");
        if(m_tableauFile == "" || m_tableauFile.isEmpty())
            return false;
    }

    try
    {
        emit status(QString("Saving Tableau, Please wait..."));

        QString fileID = m_tableauFile.left(m_tableauFile.lastIndexOf('.'));
        if(m_templateFile == "" || m_templateFile.isEmpty())
            m_templateFile = QString(fileID+"_0.frm");
        if(!saveForm(m_templateFormItem))
            return false;
        if(m_targetFile == "" || m_targetFile.isEmpty())
            m_targetFile = QString(fileID+"_1.frm");
        if(!saveForm(m_targetFormItem))
            return false;

        QString templatePath = m_templateFile;/*(m_templateFile.left(m_templateFile.lastIndexOf(QRegExp("[\\/]"))) ==
                                m_tableauFile.left(m_tableauFile.lastIndexOf(QRegExp("[\\/]")))) ?
                                m_templateFile.right(m_templateFile.length() - (m_templateFile.lastIndexOf(QRegExp("[\\/]"))+1))
                                : m_templateFile;*/
        QString targetPath = m_targetFile;/*(m_targetFile.left(m_targetFile.lastIndexOf(QRegExp("[\\/]"))) ==
                                m_tableauFile.left(m_tableauFile.lastIndexOf(QRegExp("[\\/]")))) ?
                                m_targetFile.right(m_targetFile.length() - (m_targetFile.lastIndexOf(QRegExp("[\\/]"))+1))
                                : m_targetFile;*/

        QApplication::setOverrideCursor(Qt::WaitCursor);

#ifdef _WIN32
        // for windows change the path slashes from forward to backward
        m_tableauFile = m_tableauFile.replace( '/', '\\' );
#else
        m_tableauFile = m_tableauFile.replace( '\\', '/' );
#endif

        EW_Dig3_Tableau::write_file(m_tableauFile.toStdString().c_str(), false, &m_tableauList);


        setWindowTitle(QString("Templand - ") + m_tableauFile);
        m_tableauSaved = true;
        m_savedAtleastOnce = true;
        emit status(QString("File %1 Saved").arg(m_tableauFile), 2000);
    }
    catch(EW_Form3_IOError ex)
    {
        QMessageBox::critical(this, "IO Exception", QString("Failed to save tableau!\n- %1)").arg(ex.what()));
        QApplication::restoreOverrideCursor();
        return false;
    }
    catch(std::exception &ex)
    {
        QMessageBox::critical(this, "IO Exception", QString("Failed to save tableau!\n- %1)").arg(ex.what()));
        QApplication::restoreOverrideCursor();
        return false;
    }

    QApplication::restoreOverrideCursor();
    return true;
}

void TableauLayout::pauseFilmstip()
{
    m_filmstrip.pause();

    m_filmPlayBackward->setDisabled( false );
    m_filmPlay->setDisabled( false );
    //m_filmPause->setDisabled( true );
}

void TableauLayout::stopFilmstip()
{
    m_filmstrip.stop();

    emit swapPrevForm( false );
    emit swapNextForm( true );

    m_filmPlayBackward->setDisabled( false );
    m_filmPlay->setDisabled( false );
    //m_filmPause->setDisabled( true );
}

void TableauLayout::forwardFilmstip()
{
    m_filmstrip.playForward();
    m_filmstrip.play();

    m_filmPlayBackward->setDisabled( true );
    m_filmPlay->setDisabled( true );
    m_filmPause->setDisabled( false );
}

void TableauLayout::backwardFilmstip()
{
    m_filmstrip.playBackward();
    m_filmstrip.play();

    m_filmPlayBackward->setDisabled( true );
    m_filmPlay->setDisabled( true );
    m_filmPause->setDisabled( false );
}

void TableauLayout::firstFrameFilmstrip()
{
    m_filmstrip.prevFrame();

    m_filmPlayBackward->setDisabled( false );
    m_filmPlay->setDisabled( false );
    m_filmPause->setDisabled( true );
}

void TableauLayout::lastFrameFilmstrip()
{
    m_filmstrip.nextFrame();

    m_filmPlayBackward->setDisabled( false );
    m_filmPlay->setDisabled( false );
    m_filmPause->setDisabled( true );
}

void TableauLayout::filmstripStatus( int status )
{
    if( status == FilmstripPlayerControls::PLAYING )
    {
//        m_filmstripAdd->setDisabled( true );
//        m_filmstripDelete->setDisabled( true );
//        m_filmstripReplace->setDisabled( true );

        emit appendForm( false );
        emit replaceForm( false );
        emit deleteForm( false );

        emit swapPrevForm( false );
        emit swapNextForm( false );
    }
    else if( status == FilmstripPlayerControls::STOPPED )
    {
//        m_filmstripAdd->setDisabled( false );
//        m_filmstripDelete->setDisabled( false );
//        m_filmstripReplace->setDisabled( false );

        emit appendForm( true );
        emit replaceForm( true );
        emit deleteForm( true );

        emit swapPrevForm( true );
        emit swapNextForm( true );
    }
    else if( status == FilmstripPlayerControls::PAUSED )
    {
//        m_filmstripAdd->setDisabled( false );
//        m_filmstripDelete->setDisabled( false );
//        m_filmstripReplace->setDisabled( false );

        emit appendForm( true );
        emit replaceForm( true );
        emit deleteForm( true );

        emit swapPrevForm( true );
        emit swapNextForm( true );
    }
}

void TableauLayout::addTableau( int index )
{
    if( !m_savedAtleastOnce )
    {
        QMessageBox::StandardButton ret;
        ret = QMessageBox::warning(this, tr("Save?"),
                            tr("Tableau file needs to be saved before adding Tableau frames.\nDo you want to save your changes?"),
                            QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);
        if (ret == QMessageBox::Save)
        {
            m_tableauFile = "";
            if( saveTableau() )
            {
                QObject::connect(&m_filmstrip, SIGNAL(frameIndexChange(int)), m_frameSlider, SLOT(setValue(int)));
                QObject::connect(&m_filmstrip, SIGNAL(frameIndexChange(int)), m_frameSBox, SLOT(setValue(int)));
                QObject::connect(&m_filmstrip, SIGNAL(frameNumberChange(float)), this, SLOT(filmstripFrameNumberChanged(float)));
                QObject::connect(m_frameSlider, SIGNAL(sliderMoved(int)), &m_filmstrip, SLOT(moveToFrame(int)));
                QObject::connect(m_frameSBox, SIGNAL(valueChanged(int)), &m_filmstrip, SLOT(moveToFrame(int)));
                QObject::connect(&m_filmstrip, SIGNAL(statusChanged(int)), this, SLOT(filmstripStatus(int)));
            }
            else
            {
                return;
            }
        }
        else if (ret == QMessageBox::Cancel || ret == QMessageBox::Discard )
        {
            return;
        }
    }

    EW_Dig3_Tableau tabl;
    m_dig3.save_tableau(&tabl, 0, 1, 2, 3);
    int n = 0;
    if( index == -1 )
    {
        m_tableauList.push_back( tabl );
        n = m_tableauList.size() - 1;
    }
    else
    {
         m_tableauList.insert( m_tableauList.begin() + index, tabl );
         n = index;
    }
    int size = m_tableauList.size();

    QString templatePath = m_templateFile;/*(m_templateFile.left(m_templateFile.lastIndexOf(QRegExp("[\\/]"))) ==
                            m_tableauFile.left(m_tableauFile.lastIndexOf(QRegExp("[\\/]")))) ?
                            m_templateFile.right(m_templateFile.length() - (m_templateFile.lastIndexOf(QRegExp("[\\/]"))+1))
                            : m_templateFile;*/
    QString targetPath = m_targetFile;/*(m_targetFile.left(m_targetFile.lastIndexOf(QRegExp("[\\/]"))) ==
                            m_tableauFile.left(m_tableauFile.lastIndexOf(QRegExp("[\\/]")))) ?
                            m_targetFile.right(m_targetFile.length() - (m_targetFile.lastIndexOf(QRegExp("[\\/]"))+1))
                            : m_targetFile;*/

    m_tableauList[n].space[0].form_filename = templatePath.toStdString().c_str();
    m_tableauList[n].space[1].form_filename = targetPath.toStdString().c_str();

    m_filmstrip.AssignContent( &m_dig3, m_tableauList  );


    if(size > 1)
    {
        m_frameSlider->setMinimum(1);
        m_frameSlider->setMaximum(size);
        m_frameSBox->setMinimum(1);
        m_frameSBox->setMaximum(size);
    }
    else
    {
        m_frameSlider->setMinimum(0);
        m_frameSlider->setMaximum(0);
        m_frameSBox->setMinimum(1);
        m_frameSBox->setMaximum(1);
    }

    m_frameSlider->setValue( n + 1 );
    m_frameSBox->setValue( n + 1 );
    filmstripFrameNumberChanged( n + 1 );

    // activate the filmstrip controls
    m_filmFirstFrame->setDisabled( m_tableauList.size() < 2 );
    m_filmPlayBackward->setDisabled( m_tableauList.size() < 2 );
    m_filmPlay->setDisabled( m_tableauList.size() < 2 );
    m_filmPause->setDisabled( m_tableauList.size() < 2 );
    m_filmStop->setDisabled( m_tableauList.size() < 2 );
    m_filmLastFrame->setDisabled( m_tableauList.size() < 2 );

    m_filmStepPrev->setDisabled( m_tableauList.size() < 2 );
    m_filmStepNext->setDisabled( m_tableauList.size() < 2 );
    m_secFrameTXT->setDisabled( m_tableauList.size() < 2 );
    m_stepsFrameTXT->setDisabled( m_tableauList.size() < 2 );

//    m_filmstripDelete->setDisabled( false );
//    m_filmstripReplace->setDisabled( false );

    emit replaceForm( true );
    emit deleteForm( true );

    emit swapPrevForm( false );
    emit swapNextForm( m_tableauList.size() > 1 );

    m_tblStart->setDisabled( m_tableauList.size() < 2 );
    m_tblEnd->setDisabled( m_tableauList.size() < 2 );
    m_tblLeftE->setDisabled( m_tableauList.size() < 2 );
    m_tblLeft->setDisabled( m_tableauList.size() < 2 );
    m_tblRight->setDisabled( m_tableauList.size() < 2 );
    m_tblListRightE->setDisabled( m_tableauList.size() < 2 );
}

void TableauLayout::replaceTableau()
{
    int v = m_frameSBox->value() - 1;

    m_dig3.save_tableau( &m_tableauList[v], 0, 1, 2, 3);

    QString templatePath = m_templateFile;/*(m_templateFile.left(m_templateFile.lastIndexOf(QRegExp("[\\/]"))) ==
                            m_tableauFile.left(m_tableauFile.lastIndexOf(QRegExp("[\\/]")))) ?
                            m_templateFile.right(m_templateFile.length() - (m_templateFile.lastIndexOf(QRegExp("[\\/]"))+1))
                            : m_templateFile;*/
    QString targetPath = m_targetFile;/*(m_targetFile.left(m_targetFile.lastIndexOf(QRegExp("[\\/]"))) ==
                            m_tableauFile.left(m_tableauFile.lastIndexOf(QRegExp("[\\/]")))) ?
                            m_targetFile.right(m_targetFile.length() - (m_targetFile.lastIndexOf(QRegExp("[\\/]"))+1))
                            : m_targetFile;*/


    m_tableauList[v].space[0].form_filename = templatePath.toStdString().c_str();
    m_tableauList[v].space[1].form_filename = targetPath.toStdString().c_str();

    m_filmstrip.AssignContent( &m_dig3, m_tableauList );

    // activate the filmstrip controls
    m_filmFirstFrame->setDisabled( ( m_tableauList.size() < 2 ) );
    m_filmPlayBackward->setDisabled( ( m_tableauList.size() < 2 ) );
    m_filmPlay->setDisabled( ( m_tableauList.size() < 2 ) );
    m_filmPause->setDisabled( ( m_tableauList.size() < 2 ) );
    m_filmStop->setDisabled( ( m_tableauList.size() < 2 ) );
    m_filmLastFrame->setDisabled( ( m_tableauList.size() < 2 ) );
}

void TableauLayout::deleteTableau()
{
    int v = m_frameSlider->value() - 1;

    m_tableauList.erase( m_tableauList.begin() + v );

    m_filmstrip.AssignContent( &m_dig3, m_tableauList);

    if(m_tableauList.size())
    {
        m_frameSlider->setMinimum(1);
        m_frameSlider->setMaximum(m_tableauList.size());
        m_frameSBox->setMinimum(1);
        m_frameSBox->setMaximum(m_tableauList.size());
    }

    m_frameSlider->setValue( 0 );
    m_frameSBox->setValue( 0 );

    if( m_tableauList.size() > 1 )
    {
        // activate the filmstrip controls
        m_filmFirstFrame->setDisabled( false );
        m_filmPlayBackward->setDisabled( false );
        m_filmPlay->setDisabled( false );
        m_filmPause->setDisabled( false );
        m_filmStop->setDisabled( false);
        m_filmLastFrame->setDisabled( false );

        m_tblStart->setDisabled( false );
        m_tblEnd->setDisabled( false );
        m_tblLeftE->setDisabled( false );
        m_tblLeft->setDisabled( false );
        m_tblRight->setDisabled( false );
        m_tblListRightE->setDisabled( false );
    }
    else
    {
        // activate the filmstrip controls
        m_filmFirstFrame->setDisabled( true );
        m_filmPlayBackward->setDisabled( true );
        m_filmPlay->setDisabled( true );
        m_filmPause->setDisabled( true );
        m_filmStop->setDisabled( true );
        m_filmLastFrame->setDisabled( true );

        //m_filmstripDelete->setDisabled( true );
        emit deleteForm( false );

        m_tblStart->setDisabled( true );
        m_tblEnd->setDisabled( true );
        m_tblLeftE->setDisabled( true );
        m_tblLeft->setDisabled( true );
        m_tblRight->setDisabled( true );
        m_tblListRightE->setDisabled( true );
    }
}

void TableauLayout::moveTableauToPrev()
{
    // move right
    int v = m_frameSlider->value() - 1;

    if( v == 0 )
        return;

    EW_Dig3_Tableau replacement = m_tableauList[v - 1];
    m_tableauList[v - 1] = m_tableauList[v];
    m_tableauList[v] = replacement;

    m_filmstrip.AssignContent( &m_dig3, m_tableauList );

    m_frameSlider->setValue( v );
    m_frameSBox->setValue( v );
    filmstripFrameNumberChanged( v );
}

void TableauLayout::moveTableauToNext()
{
    // move left
    int v = m_frameSlider->value() - 1;

    if( v == (int)m_tableauList.size() - 1 )
        return;

    EW_Dig3_Tableau replacement = m_tableauList[v + 1];
    m_tableauList[v + 1] = m_tableauList[v];
    m_tableauList[v] = replacement;

    m_filmstrip.AssignContent( &m_dig3, m_tableauList );

    m_frameSlider->setValue( v + 2 );
    m_frameSBox->setValue( v + 2 );
    filmstripFrameNumberChanged( v + 2 );
}

void TableauLayout::changeSliceColor(ViewTree* itemTree)
{
    const EW_Dig3_View* surfaceView = itemTree==m_targetItemTree ?  m_targetView->getDig3View() : m_templateView->getDig3View();
    const EW_Dig3_View* sliceView = itemTree==m_targetItemTree ?  m_targetSliceView->getDig3View() : m_templateSliceView->getDig3View();
    int sliceIndex = itemTree==m_targetItemTree ? 3:1;
    if(!surfaceView->get_slice_items() || !sliceView->get_slice_items())
        return;

    const unsigned char* rgb = surfaceView->get_slice_items()[sliceIndex]->get_color();
    const unsigned char* col = chooseColor(rgb);
    if(col)
    {
        surfaceView->get_slice_items()[sliceIndex]->set_color(col);
        sliceView->get_slice_items()[sliceIndex]->set_color(col);
        delete[] col;

        tableauUpdated();
    }
}

void TableauLayout::changeSliceClipR(ViewTree* itemTree)
{
    QString itemData = m_tableauFile;

    if(m_frameSlider->value() >0 && m_frameSlider->value()<=(int)m_tableauList.size())
    {
        bool ok = false;
        double clipRatio = m_tableauList[m_frameSlider->value()-1].slice_clip_ratio;
        clipRatio = QInputDialog::getDouble(this, tr("Slice Clip Ratio"),
                                            tr("Enter Slice Clip Ratio: "), clipRatio, 0.001, 100, 3, &ok);
        if(ok)
            m_tableauList[m_frameSlider->value()-1].slice_clip_ratio = clipRatio;
    }
}

void TableauLayout::changeLmkColor(FormItem* itemForm)
{
    View3Qt* surfaceWidget = itemForm==m_targetFormItem ?  m_targetView : m_templateView;
    View3Qt* sliceWidget = itemForm==m_targetFormItem ?  m_targetSliceView: m_templateSliceView;

    if(!surfaceWidget->getDig3View()->get_landmarks_item() || !sliceWidget->getDig3View()->get_landmarks_item())
        return;

    const unsigned char* rgb = surfaceWidget->getDig3View()->get_landmarks_item()->get_color();
    const unsigned char* col = chooseColor(rgb);
    if(col)
    {
        surfaceWidget->getDig3View()->get_landmarks_item()->set_color(col);
        sliceWidget->getDig3View()->get_landmarks_item()->set_color(col);

        //Asssign the higlight color to the inverse
        QColor result;
        result.setRgb(255-col[0], 255-col[1], 255-col[2]);
        int ir, ig, ib;
        result.getRgb(&ir, &ig, &ib);
        unsigned char hCol[3];
        hCol[0] = (unsigned char)ir;
        hCol[1] = (unsigned char)ig;
        hCol[2] = (unsigned char)ib;
        surfaceWidget->set_highlight_color(hCol);
        sliceWidget->set_highlight_color(hCol);

        delete[] col;

        tableauUpdated();
    }
}

bool TableauLayout::addSurface(SurfaceItem* treeItem, FormItem* itemForm)
{
    bool result = false;
    try
    {
        QApplication::setOverrideCursor(Qt::WaitCursor);
        int space = itemForm==m_targetFormItem ? 1 : 0;
        QString formFile = itemForm==m_targetFormItem ? m_targetFile : m_templateFile;

        QString surfacePath = treeItem->getSurfaceFile();
        /*surfacePath = ( surfacePath.left(surfacePath.lastIndexOf(QRegExp("[\\/]"))) ==
                        formFile.left(formFile.lastIndexOf(QRegExp("[\\/]")))) ?
                        surfacePath.right(surfacePath.length() - (surfacePath.lastIndexOf(QRegExp("[\\/]"))+1)) : surfacePath;*/

#ifdef _WIN32
        // for windows change the path slashes from forward to backward
        surfacePath = surfacePath.replace( '/', '\\' );
#else
        surfacePath = surfacePath.replace( '\\', '/' );
#endif

        EW_Form3_Surface fs;
        fs.id = treeItem->getSurfaceID().toStdString().c_str();
        fs.file = surfacePath.toStdString().c_str();
        bool replaced;
        m_dig3.get_spaces()[space]->set_form_surface(&replaced, &fs);//treeItem->getSurfaceID().toStdString().c_str(), surfacePath.toStdString().c_str());

        if (m_dig3.get_spaces()[space]->get_form_data()->surfaces.size() == 1) //Focus on first surface
        {
            treeItem->setSurfaceIndex(0);
            focusView(itemForm, treeItem);
        }

        formUpdated(itemForm);
        QApplication::restoreOverrideCursor();

        //m_filmstripAdd->setDisabled( false );
        emit appendForm( true );

        result = true;
    }
    catch(EW_Surface3_IOError ex)
    {
        QMessageBox::critical(this, "IO Exception", QString("Failed to load surface!\n- %1)").arg(ex.what()));
        result = false;
    }
    QApplication::restoreOverrideCursor();

    return result;
}

void TableauLayout::changeSurfaceColor(SurfaceItem* surItem, FormItem* itemForm, bool front)
{
    const EW_Dig3_View* surfaceView = itemForm==m_targetFormItem ?  m_targetView->getDig3View() : m_templateView->getDig3View();
    const EW_Dig3_View* sliceView = itemForm==m_targetFormItem ?  m_targetSliceView->getDig3View() : m_templateSliceView->getDig3View();
    int surIndex = surItem->getSurfaceIndex();
    if(!surfaceView->get_surface_items() || !sliceView->get_surface_items())
        return;

    const unsigned char *rgb = front? surfaceView->get_surface_items()[surIndex]->get_front_color() :
                                      surfaceView->get_surface_items()[surIndex]->get_back_color() ;
    const unsigned char* col = chooseColor(rgb);
    if(col)
    {
        if(front)
        {
            surfaceView->get_surface_items()[surIndex]->set_front_color(col);
            sliceView->get_surface_items()[surIndex]->set_front_color(col);
        }
        else
        {
            surfaceView->get_surface_items()[surIndex]->set_back_color(col);
            sliceView->get_surface_items()[surIndex]->set_back_color(col);
        }
        delete[] col;

        tableauUpdated();
    }
}

void TableauLayout::applyItemChange(ViewTree* itemTree, QTreeWidgetItem* treeItem, int column)
{
    if(!m_treesFull)
        return;
    bool on = treeItem->checkState(column) != Qt::Unchecked;
    const EW_Dig3_View* surfaceView = itemTree==m_targetItemTree ?  m_targetView->getDig3View() : m_templateView->getDig3View();
    const EW_Dig3_View* sliceView = itemTree==m_targetItemTree ?  m_targetSliceView->getDig3View() : m_templateSliceView->getDig3View();

    switch(((ViewTreeItem*)treeItem)->getType())
    {
        case ViewTreeItem::SURFACES_ITEM:
            for(int i=0; i<treeItem->childCount(); ++i)
            {
                if( column == 1 )
                    surfaceView->get_surface_items()[i]->set_state(on);
                else
                    sliceView->get_surface_items()[i]->set_state(on);
                treeItem->child(i)->setCheckState(column, treeItem->checkState(column));
            }
        break;
        case ViewTreeItem::SURFACE_ITEM:
        {
            SurfaceItem* surItem = (SurfaceItem*)treeItem;
            int surIndex = surItem->getSurfaceIndex();
            if(surIndex != -1)
            {
                if( column == 1 )
                    surfaceView->get_surface_items()[surIndex]->set_state(on);
                else
                    sliceView->get_surface_items()[surIndex]->set_state(on);
            }
        }
        break;
        case ViewTreeItem::SLICE_ITEM:
        {
            if( column == 1 )
            {
                if(surfaceView->get_slice_items())
                    surfaceView->get_slice_items()[itemTree==m_targetItemTree? 3:1]->set_state(on);
            }
            else
            {
                if( sliceView->get_slice_items() )
                    sliceView->get_slice_items()[itemTree==m_targetItemTree ? 3 : 1 ]->set_state( on );
            }
        }
        break;
        case ViewTreeItem::LANDMARKS_ITEM:
        {
            if( column == 1 )
                surfaceView->get_landmarks_item()->set_state(on);
            else
                sliceView->get_landmarks_item()->set_state(on);
        }
        break;
        case ViewTreeItem::SEMILANDMARKS_ITEM:
        {
            if( column == 1 )
                surfaceView->get_landmarks_item()->set_state(on);
            else
                sliceView->get_landmarks_item()->set_state(on);
        }
        break;
        default: break;
    }
}

void TableauLayout::deleteLmk(FormItem* itemForm, int i)
{
    std::string id;
    if( itemForm==m_targetFormItem )
    {
        ViewTreeItem* vti = (ViewTreeItem*)m_targetTopLandmarks->child( i );
        if( vti->getType() == ViewTreeItem::LANDMARK_ITEM )
            id = ((LandmarkItem*)vti)->getLmkID().toStdString();
        else if( vti->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
            id = ((SemiLandmarksTopItem*)vti)->getLmkID().toStdString();
    }
    else if( itemForm==m_templateFormItem )
    {
        ViewTreeItem* vti = (ViewTreeItem*)m_templateTopLandmarks->child( i );
        if( vti->getType() == ViewTreeItem::LANDMARK_ITEM )
            id = ((LandmarkItem*)vti)->getLmkID().toStdString();
        else if( vti->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
            id = ((SemiLandmarksTopItem*)vti)->getLmkID().toStdString();
    }

    if( id.empty() )
        return;

    const EW_Form3* form = m_dig3.get_spaces()[ (itemForm==m_targetFormItem) ? 1 : 0 ]->get_form_data();
    int index = i;
    bool found = false;
    if( form != 0 )
        for(unsigned int j=0; j<form->pointsets.size() && !found; ++j)
            if(form->pointsets[j].type == EW_Form3::TYPE_LANDMARK || form->pointsets[j].type == EW_Form3::TYPE_SEMI_LANDMARK)
            {
                if( form->pointsets[j].id == id )
                {
                    index = j;
                    found = true;
                }
            }

    if( !found )
        return;

    m_dig3.get_spaces()[itemForm==m_targetFormItem ? 1:0]->remove_form_pointset(index);
    formUpdated(itemForm);
}

void TableauLayout::deleteLmk(FormItem* patch, ViewTreeItem* semi, int index )
{
    std::string id = ((SemiLandmarksTopItem*)patch)->getLmkID().toStdString();

    const EW_Form3* form = m_dig3.get_spaces()[ 1 ]->get_form_data();
    if( form != 0 )
        for(unsigned int j=0; j<form->pointsets.size(); ++j)
            if( form->pointsets[j].type == EW_Form3::TYPE_SEMI_LANDMARK )
            {
                if( form->pointsets[j].id == id )
                {
                    for( int n = 0; n < form->pointsets[j].n; ++n )
                    {
                        if( n == index )
                        {
                            EW_Form3_PointSet ps = form->pointsets[j];
                            // delete the x y z
                            ps.locations.erase( ps.locations.begin() + n * 3 );
                            ps.locations.erase( ps.locations.begin() + n * 3 );
                            ps.locations.erase( ps.locations.begin() + n * 3 );

                            // delet the relax dims
                            if( (int)ps.relax_dims.size() > n )
                                ps.relax_dims.erase( ps.relax_dims.begin() + n );

                            // delete the relax params
                            if( (int)ps.relax_params.size() > n * 3 + 3 )
                            {
                                ps.relax_params.erase( ps.relax_params.begin() + n * 3 );
                                ps.relax_params.erase( ps.relax_params.begin() + n * 3 );
                                ps.relax_params.erase( ps.relax_params.begin() + n * 3 );
                            }

                            // decrease the pointset count
                            --ps.n;

                            bool b = false;
                            m_dig3.get_spaces()[ 1 ]->set_form_pointset(&b, &ps);
                            return;
                        }
                    }
                }
            }
}

void TableauLayout::stateLmk(FormItem* form, ViewTreeItem* item, int index, int newState )
{
    if( item->getType() == ViewTreeItem::LANDMARKS_ITEM )
    {
        // loop through all the Landmark children and change state to fixed
        for( int i = 0; i < item->childCount(); ++i )
        {
            ViewTreeItem* vti = (ViewTreeItem*)item->child(i);
            if( vti->getType() == ViewTreeItem::LANDMARK_ITEM )
            {
                LandmarkItem* li = (LandmarkItem*)vti;
                QString id = li->getLmkID();

                changeState( id.toStdString(), newState );
            }
        }

        //update the tree
        updateTreeViewStates( 1 );
    }
    else if( item->getType() == ViewTreeItem::LANDMARK_ITEM )
    {
        LandmarkItem* li = (LandmarkItem*)item;
        QString id = li->getLmkID();

        changeState( id.toStdString(), newState );

        //update the tree
        updateTreeViewStates( 1 );
    }
    else if( item->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
    {
        SemiLandmarksTopItem* sti = (SemiLandmarksTopItem*)item;
        QString id = sti->getLmkID();

        changeState( id.toStdString(), newState );

        //update the tree
        updateTreeViewStates( 1 );
    }
}

void TableauLayout::changeState( const std::string& id, int state )
{
    for( unsigned int i = 0; i < m_dig3.get_spaces()[ 1 ]->get_form_data()->pointsets.size(); ++i )
    {
        if( m_dig3.get_spaces()[ 1 ]->get_form_data()->pointsets[i].id == id )
        {
            // take copy of the pointset
            EW_Form3_PointSet ps = m_dig3.get_spaces()[ 1 ]->get_form_data()->pointsets[i];

            // change the state to new state
            ps.state = state;

            // insert pointset with new state
            bool b = false;
            m_dig3.get_spaces()[1]->set_form_pointset(&b, &ps);

            break;
        }
    }
}

void TableauLayout::landmarkMoved( const QString& id, int space )
{
    //update the tree
    updateTreeViewStates( space );
}

void TableauLayout::renameLmk(FormItem* form, ViewTreeItem* item, int index, const QString& newID)
{
    std::string oldID;
    // get the old name
    if( item->getType() == ViewTreeItem::LANDMARK_ITEM )
    {
        LandmarkItem* li = (LandmarkItem*)item;
        oldID = li->getLmkID().toStdString();
    }
    else if( item->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
    {
        SemiLandmarksTopItem* sti = (SemiLandmarksTopItem*)item;
        oldID = sti->getLmkID().toStdString();
    }

    // check existing landmark if new names exists
    bool exists = false;
    for( int i = 0; i < m_targetTopLandmarks->childCount() && !exists; ++i )
    {
        ViewTreeItem* vti = (ViewTreeItem*)m_targetTopLandmarks->child( i );

        if( vti->getType() == ViewTreeItem::LANDMARK_ITEM )
        {
            LandmarkItem* li = (LandmarkItem*)vti;
            exists = ( li->getLmkID() == newID );
        }
        else if( vti->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
        {
            SemiLandmarksTopItem* sti = (SemiLandmarksTopItem*)vti;
            exists = ( sti->getLmkID() == newID );
        }
    }

    // display duplication error message and quit
    if( exists )
    {
        QString msg = newID;
        QMessageBox::critical( this, "Error", msg + " name already exists. Please choose another name." );
        return;
    }

    // now rename the pointsets if unique
    for( unsigned int i = 0; i < m_dig3.get_spaces()[ 1 ]->get_form_data()->pointsets.size(); ++i )
    {
        if( m_dig3.get_spaces()[ 1 ]->get_form_data()->pointsets[i].id == oldID )
        {
            // take copy of the pointset
            EW_Form3_PointSet ps = m_dig3.get_spaces()[ 1 ]->get_form_data()->pointsets[i];

            ps.id = newID.toStdString();

            // delete the old pointset with the old name
            m_dig3.get_spaces()[1]->remove_form_pointset(i);

            // insert pointset with new name
            bool b = false;
            m_dig3.get_spaces()[1]->set_form_pointset(&b, &ps);

            // update the UI text and internal ID
            if( item->getType() == ViewTreeItem::LANDMARK_ITEM )
            {
                LandmarkItem* li = (LandmarkItem*)item;
                li->setLmkID( newID );
                QString state = stateLabel( ps.state );
                state += " ";
                state += newID;
                li->setText( 0, state );
            }
            else if( item->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
            {
                SemiLandmarksTopItem* sti = (SemiLandmarksTopItem*)item;
                sti->setLmkID( newID );
                QString state = stateLabel( ps.state );
                state += " ";
                state += newID;
                sti->setText( 0, state );
            }

            // update landmark index after pointset deletion and insertion
            for( int i = 0; i < m_targetTopLandmarks->childCount(); ++i )
            {
                ViewTreeItem* vti = (ViewTreeItem*)m_targetTopLandmarks->child( i );

                if( vti->getType() == ViewTreeItem::LANDMARK_ITEM )
                {
                    double x, y, z;
                    int index = -1;
                    LandmarkItem* li = (LandmarkItem*)vti;
                    if( getPointsetLocation( li->getLmkID().toStdString(), 1, x, y, z, index ) )
                        li->setLmkIndex( index );
                }
                else if( vti->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
                {
                    double x, y, z;
                    int index = -1;
                    SemiLandmarksTopItem* sti = (SemiLandmarksTopItem*)vti;
                    if( getPointsetLocation( sti->getLmkID().toStdString(), 1, x, y, z, index ) )
                        sti->setLmkIndex( index );
                }
            }

            // finally informt he end user
            QString msg = oldID.c_str();
            msg += " Landmark has been renamed to: ";
            msg += newID;
            emit status( msg );
        }
    }
}

void TableauLayout::highlightItem(ViewTree* itemTree, QTreeWidgetItem* treeItem)
{
    if(!m_treesFull)
        return;

    const EW_Dig3_View* surfaceView = itemTree==m_targetItemTree ?  m_targetView->getDig3View() : m_templateView->getDig3View();
    const EW_Dig3_View* sliceView = itemTree==m_targetItemTree ?  m_targetSliceView->getDig3View() : m_templateSliceView->getDig3View();
    surfaceView->widget->clear_highlight();
    sliceView->widget->clear_highlight();

    LandmarksTopItem* other = itemTree==m_targetItemTree ? m_templateTopLandmarks : m_targetTopLandmarks;
    LandmarksTopItem* same = itemTree==m_targetItemTree ? m_targetTopLandmarks : m_templateTopLandmarks;

    switch(((ViewTreeItem*)treeItem)->getType())
    {
        case ViewTreeItem::LANDMARK_ITEM:
        {
            LandmarkItem* lmkItem = (LandmarkItem*)treeItem;
            int index = lmkItem->getLmkIndex();

            std::string id = lmkItem->getLmkID().toStdString();
            const EW_Form3* form = m_dig3.get_spaces()[ (itemTree==m_targetItemTree) ? 1 : 0 ]->get_form_data();
            if( form != 0 )
                for(unsigned int j=0; j<form->pointsets.size(); ++j)
                    if(form->pointsets[j].type == EW_Form3::TYPE_LANDMARK)
                    {
                        if( form->pointsets[j].id == id )
                        {
                            index = j;
                            break;
                        }
                    }

            surfaceView->get_landmarks_item()->set_highlight(index, 0);
            sliceView->get_landmarks_item()->set_highlight(index, 0);

            // finally informt he end user
            QString msg = lmkItem->getLmkID();
            msg += " has been selected.";
            emit status( msg );

            if( other->childCount() && other->doesChildExist( lmkItem->getLmkID(), "" ) == true && m_selectPointsInViews == true )
            {
                // now highlight the landmark IF exists in other tree view
                other->select( lmkItem->getLmkID() );
            }
            else
            {
                // now highlight the landmark IF exists in other tree view
                same->select( lmkItem->getLmkID() );
            }
        }
        break;
        case ViewTreeItem::SEMILANDMARKS_ITEM:
        {
            SemiLandmarksTopItem* lmkItem = (SemiLandmarksTopItem*)treeItem;
            int index = lmkItem->getLmkIndex();

            std::string id = lmkItem->getLmkID().toStdString();
            const EW_Form3* form = m_dig3.get_spaces()[ (itemTree==m_targetItemTree) ? 1 : 0 ]->get_form_data();
            if( form != 0 )
                for(unsigned int j=0; j<form->pointsets.size(); ++j)
                    if(form->pointsets[j].type == EW_Form3::TYPE_SEMI_LANDMARK)
                    {
                        if( form->pointsets[j].id == id )
                        {
                            index = j;
                            break;
                        }
                    }

            surfaceView->get_landmarks_item()->set_highlight(index, 0);
            sliceView->get_landmarks_item()->set_highlight(index, 0);

            // finally informt he end user
            QString msg = lmkItem->getLmkID();
            msg += " has been selected.";
            emit status( msg );

            if( other->childCount() && other->doesChildExist( lmkItem->getLmkID(), "" ) == true && m_selectPointsInViews == true )
            {
                // now highlight the semilandmark patch IF exists in other tree view
                other->select( lmkItem->getLmkID() );
            }
            else
            {
                // now highlight the semilandmark patch IF exists in other tree view
                same->select( lmkItem->getLmkID() );
            }
        }
        break;
        case ViewTreeItem::SEMILANDMARK_ITEM:
        {
            SemiLandmarkItem* lmkItem = (SemiLandmarkItem*)treeItem;
            QTreeWidgetItem* parent = ((QTreeWidgetItem*)lmkItem)->parent();
            int index = ((SemiLandmarksTopItem*)parent)->getLmkIndex();
            int within = lmkItem->getLmkIndex();

            std::string id = ((SemiLandmarksTopItem*)parent)->getLmkID().toStdString();
            const EW_Form3* form = m_dig3.get_spaces()[ (itemTree==m_targetItemTree) ? 1 : 0 ]->get_form_data();
            if( form != 0 )
                for(unsigned int j=0; j<form->pointsets.size(); ++j)
                    if(form->pointsets[j].type == EW_Form3::TYPE_SEMI_LANDMARK)
                    {
                        if( form->pointsets[j].id == id )
                        {
                            index = j;
                            break;
                        }
                    }

            surfaceView->get_landmarks_item()->set_highlight(index, within * 3 );
            sliceView->get_landmarks_item()->set_highlight(index, within * 3 );

            // finally informt he end user
            QString msg = lmkItem->text( 0 );
            msg += " in ";
            msg += id.c_str();
            msg += " has been selected.";
            emit status( msg );

            if( other->childCount() && other->doesChildExist(id.c_str(), lmkItem->text(0)) == true && m_selectPointsInViews == true )
            {
                // now highlight the semilandmark patch IF exists in other tree view
                other->select( id.c_str(), lmkItem->text(0) );
            }
            else
            {
                // now highlight the semilandmark patch IF exists in other tree view
                same->select( id.c_str(), lmkItem->text(0) );
            }
        }
        break;
        default: break;
    }
}

void TableauLayout::landmarkSelected( const QString& id, int index, int indexWithin, int space )
{
    LandmarksTopItem* lti = (space == 0) ? m_templateTopLandmarks : m_targetTopLandmarks;
    ViewTree* itemTree = (space == 0) ? m_templateItemTree : m_targetItemTree;

    for(int i = 0; i < lti->childCount(); ++i )
    {
        ViewTreeItem* vti = (ViewTreeItem*)lti->child(i);
        if( vti->getType() == ViewTreeItem::LANDMARK_ITEM )
        {
            if( ((LandmarkItem*)vti)->getLmkID() == id )
            {
                highlightItem( itemTree, vti );
                break;
            }
        }
        else if( vti->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
        {
            if( ((SemiLandmarksTopItem*)vti)->getLmkID() == id )
            {
                /*if( indexWithin == 0 )
                    highlightItem( itemTree, vti );
                else
                    */highlightItem( itemTree, vti->child( indexWithin ) );

                break;
            }
        }
    }
}

bool TableauLayout::hasStateLabel( const QString& lbl, QString& lblWithoutState )
{
    if( lbl.contains( "[" ) && lbl.contains( "]" ) )
    {
        lblWithoutState = lbl.right( lbl.length() - 4 );
        return true;
    }
    return false;
}

QString TableauLayout::stateLabel( int state )
{
    if( state == EW_Form3::STATE_SET )
    {
        return "[F]";
    }
    else if( state == EW_Form3::STATE_PROJECTED )
    {
        return "[P]";
    }
    else if( state == EW_Form3::STATE_WARPED )
    {
        return "[W]";
    }
    else if( state == EW_Form3::STATE_OPTIMIZED )
    {
        return "[S]";
    }
    return "[-]";
}

void TableauLayout::updateTreeViewStates( int space )
{
    LandmarksTopItem* lti = ( space == 0 ) ? m_templateTopLandmarks : m_targetTopLandmarks;

    for( int i = 0; i < lti->childCount(); ++i )
    {
        QString lbl;
        ViewTreeItem* vti = (ViewTreeItem*)lti->child(i);
        if( vti->getType() == ViewTreeItem::LANDMARK_ITEM )
            lbl = ((LandmarkItem*)vti)->getLmkID();
        else if( vti->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
            lbl = ((SemiLandmarksTopItem*)vti)->getLmkID();

        QString lblWithout;

        if( hasStateLabel( lbl, lblWithout ) == false )
            lblWithout = lbl;

        int index = -1;
        double x = 0.0, y = 0.0, z = 0.0;

        if( getPointsetLocation( lblWithout.toStdString(), space, x, y, z, index ) )
        {
            int state = m_dig3.get_spaces()[space]->get_form_data()->pointsets[index].state;

            QString finalLBL = stateLabel( state );
            finalLBL += " ";
            finalLBL += lblWithout;
            lti->child(i)->setText( 0, finalLBL );
        } // end if
    } // end for loop
}

void TableauLayout::changeLmksSymbol(FormItem* itemForm, EW_View3_Landmarks::SymbolType sym)
{
    const EW_Dig3_View* surfaceView = itemForm==m_targetFormItem ?  m_targetView->getDig3View() : m_templateView->getDig3View();
    const EW_Dig3_View* sliceView = itemForm==m_targetFormItem ?  m_targetSliceView->getDig3View() : m_templateSliceView->getDig3View();
    surfaceView->get_landmarks_item()->set_symbol(sym);
    sliceView->get_landmarks_item()->set_symbol(sym);
    tableauUpdated();
}

void TableauLayout::moveToFrame(int frameIndex)
{
    if(frameIndex > 0 && frameIndex <= (int)m_tableauList.size())
    {
        m_dig3.load_tableau(&m_tableauList[frameIndex-1], 0, EW_Dig3::TABLEAU_VIEW );
        m_dig3.load_tableau(&m_tableauList[frameIndex-1], 1, EW_Dig3::TABLEAU_VIEW );
        m_dig3.load_tableau(&m_tableauList[frameIndex-1], 2, EW_Dig3::TABLEAU_VIEW );
        m_dig3.load_tableau(&m_tableauList[frameIndex-1], 3, EW_Dig3::TABLEAU_VIEW );
    }

    if( !m_filmstrip.IsPlaying() )
    {
        if( frameIndex == 1 )
        {
            emit swapPrevForm( false );
            emit swapNextForm( false );
        }
        else if( frameIndex == (int)m_tableauList.size() )
        {
            emit swapPrevForm( true );
            emit swapNextForm( false );
        }
        else
        {
            emit swapPrevForm( true );
            emit swapNextForm( true );
        }
    }

}

void TableauLayout::focusView(FormItem* itemForm, ViewTreeItem* treeItem)
{
    View3Qt* surfaceWidget = itemForm==m_targetFormItem ?  m_targetView : m_templateView;
    View3Qt* sliceWidget = itemForm==m_targetFormItem ?  m_targetSliceView : m_templateSliceView;
    EW_Transform3 tr;

    switch(treeItem->getType())
    {
        case ViewTreeItem::FORM_ITEM:
        {
            EW_Bbox3 b;
            m_dig3.get_spaces()[itemForm==m_targetFormItem ? 1 : 0]->get_bbox(&b);
            if (surfaceWidget->get_bbox_tr(&tr, &b, true, true))
                surfaceWidget->getDig3View()->view_post_compose(&tr);
            if (sliceWidget->get_bbox_tr(&tr, &b, true, true))
                sliceWidget->getDig3View()->view_post_compose(&tr);
        }
        break;
        case ViewTreeItem::SLICE_ITEM:
        {
            surfaceWidget = treeItem->getParentTree()==m_targetItemTree ?  m_targetView : m_templateView;
            sliceWidget = treeItem->getParentTree()==m_targetItemTree ?  m_targetSliceView : m_templateSliceView;
            const EW_Bbox3* b = surfaceWidget->getDig3View()->get_slice_items()[treeItem->getParentTree()==m_targetItemTree ? 3:1]->get_bbox();
            if (surfaceWidget->get_bbox_tr(&tr, b, true, true))
                surfaceWidget->getDig3View()->view_post_compose(&tr);
            if (sliceWidget->get_bbox_tr(&tr, b, true, true))
                sliceWidget->getDig3View()->view_post_compose(&tr);
        }
        break;
        case ViewTreeItem::LANDMARKS_ITEM:
        {
            const EW_Bbox3* b = surfaceWidget->getDig3View()->get_landmarks_item()->get_bbox();
            if (surfaceWidget->get_bbox_tr(&tr, b, true, true))
                surfaceWidget->getDig3View()->view_post_compose(&tr);
            if (sliceWidget->get_bbox_tr(&tr, b, true, true))
                sliceWidget->getDig3View()->view_post_compose(&tr);
        }
        break;
        case ViewTreeItem::SURFACE_ITEM:
        {
            SurfaceItem* surItem = (SurfaceItem*)treeItem;
            EW_Dig3_Space *sp = m_dig3.get_spaces()[itemForm==m_targetFormItem ? 1 : 0];
            if (sp->get_form_data()->surfaces.size() && surItem->getSurfaceIndex()<(int)sp->get_form_data()->surfaces.size())
            {
                const EW_Bbox3 *b = surfaceWidget->getDig3View()->get_surface_items()[surItem->getSurfaceIndex()]->get_bbox();
                if (surfaceWidget->get_bbox_tr(&tr, b, true, true))
                    surfaceWidget->getDig3View()->view_post_compose(&tr);
                if (sliceWidget->get_bbox_tr(&tr, b, true, true))
                    sliceWidget->getDig3View()->view_post_compose(&tr);
            }
        }
        break;
        default: break;
    }

    tableauUpdated();
}

void TableauLayout::secPerFrame( const QString& text )
{
    if( text.isEmpty () )
        return;
    bool ok;
    float sec = text.toFloat( &ok );
    if( ok )
        m_filmstrip.secPerFrame( sec );
    else
        QMessageBox::critical( this, "Format Error", "Invalid input should be blank or positive Real number." );
}

void TableauLayout::stepPerFrame( const QString& text )
{
    if( text.isEmpty () )
        return;
    bool ok;
    int step = text.toInt( &ok );
    if( ok )
        m_filmstrip.stepPerFrame( (float)step );
    else
        QMessageBox::critical( this, "Format Error", "Invalid input should be blank or positive integer." );
}

void TableauLayout::stepPrevFrameFilmstrip()
{
    m_filmstrip.prevFrameStep();
}

void TableauLayout::stepNextFrameFilmstrip()
{
    m_filmstrip.nextFrameStep();
}

void TableauLayout::closeEvent(QCloseEvent *event)
{
    if(!m_tableauSaved)
    {
        QMessageBox::StandardButton ret;
        ret = QMessageBox::warning(this, tr("Save?"),
                            tr("The landmark tableau file <b>'%1'</b> has been modified.\nDo you want to save your changes?")
                            .arg(m_tableauFile),
                            QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);
        if (ret == QMessageBox::Save)
            saveTableau();
        else if (ret == QMessageBox::Cancel)
        {
            event->ignore();
            return;
        }

    }
    if(!m_targetSaved)
    {
        QMessageBox::StandardButton ret;
        ret = QMessageBox::warning(this, tr("Save?"),
                            tr("The specimen form <b>'%1'</b> has been modified.\nDo you want to save your changes?")
                            .arg(m_targetFile),
                            QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);
        if (ret == QMessageBox::Save)
            saveForm(m_targetFormItem);
        else if (ret == QMessageBox::Cancel)
        {
            event->ignore();
            return;
        }
    }
    if(!m_templateSaved)
    {
        QMessageBox::StandardButton ret;
        ret = QMessageBox::warning(this, tr("Save?"),
                            tr("The template form <b>'%1'</b> has been modified.\nDo you want to save your changes?")
                            .arg(m_templateFile),
                            QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);
        if (ret == QMessageBox::Save)
            saveForm(m_templateFormItem);
        else if (ret == QMessageBox::Cancel)
        {
            event->ignore();
            return;
        }
    }
    event->accept();
}

void TableauLayout::tblMoveToStart()
{
    if( m_tableauList.size() > 1 )
        m_filmstrip.startFrame();
}

void TableauLayout::tblMoveToEnd()
{
    if( m_tableauList.size() > 1 )
        m_filmstrip.endFrame();
}

void TableauLayout::tblMoveLeftE()
{
    if( m_tableauList.size() > 1 )
    {
        m_filmstrip.prevFrame();
        m_filmstrip.prevFrame();
    }
}

void TableauLayout::tblMoveLeft()
{
    if( m_tableauList.size() > 1 )
        m_filmstrip.prevFrame();
}

void TableauLayout::tblMoveRight()
{
    if( m_tableauList.size() > 1 )
        m_filmstrip.nextFrame();
}

void TableauLayout::tblMoveRightE()
{
    if( m_tableauList.size() > 1 )
    {
        m_filmstrip.nextFrame();
        m_filmstrip.nextFrame();
    }
}

//void TableauLayout::tblNew( FormItem* item )
//{
//    resetForm( item );
//}

QString TableauLayout::getNewFormFilename()
{
    QString file = m_targetFile;
    if( m_tableauList.size() > 0 )
        file = m_tableauList[ m_tableauList.size()-1 ].space[1].form_filename.c_str();

    if( file.contains( "_c" ))
    {
        QString b = file.left( file.lastIndexOf("_c") );
        QString a = file.right( file.length() - file.lastIndexOf("_c") - 2 );
        QString c = a.left( a.lastIndexOf(".frm") );

        bool ok = false;
        int count = c.toInt( &ok );

        bool exists = true;
        QString finalname;
        while( exists )
        {
            count++;
            finalname = b + "_c";
            QString num; num.setNum( count );
            finalname += num;
            finalname += ".frm";
            exists = QFile::exists(finalname);
        }
        return finalname;
    }
    else
    {
        QString b = file.left( file.lastIndexOf(".frm") );

        int count = 0;

        bool exists = true;
        QString finalname;
        while( exists )
        {
            count++;
            finalname = b + "_c";
            QString num; num.setNum( count );
            finalname += num;
            finalname += ".frm";
            exists = QFile::exists(finalname);
        }
        return finalname;
    }
    return "";
}

void TableauLayout::tblAppend( FormItem* item )
{
    if( m_targetFile != "" && !m_targetSaved )
    {
        QString f = getNewFormFilename();
        if( f != "" )
        {
            m_targetFile = f;
        }
        saveForm( m_targetFormItem );
    }

    addTableau();
    // now save the tableau file
    saveTableau();
}

void TableauLayout::tblInsert( FormItem* item )
{
    if( m_targetFile != "" && !m_targetSaved)
    {
        QString f = getNewFormFilename();
        if( f != "" )
        {
            m_targetFile = f;
        }
        saveForm( m_targetFormItem );
    }

    bool ok;
    int max = m_tableauList.size() ? m_tableauList.size() : 1;
    int num = QInputDialog::getInteger( NULL, "Insert Tableau", "Specify index to insert tableau", 1, 1, max, 1, &ok);
    if (ok)
    {
        addTableau( num - 1 );
    }
    // now save the tableau file
    saveTableau();
}

void TableauLayout::tblReplace( FormItem* item )
{
    if( m_targetFile != "" && !m_targetSaved)
    {
        QString f = getNewFormFilename();
        if( f != "" )
        {
            m_targetFile = f;
        }
        saveForm( m_targetFormItem );
    }

    // replace
    replaceTableau();
    // now save the tableau ifle
    saveTableau();
}

void TableauLayout::tblSwapPrev()
{
    moveTableauToPrev();
    // now save the tableau file
    saveTableau();
}

void TableauLayout::tblSwapNext()
{
    moveTableauToNext();
    // now save the tableau file
    saveTableau();
}

void TableauLayout::tblDelete( FormItem* item )
{
    // delete
    deleteTableau();
    // now save the tablea file
    saveTableau();
}
