#include "LandmarksTopItem.h"
#include "LandmarkItem.h"
#include "SemiLandmarksTopItem.h"

void LandmarksTopItem::addLandmarkItem(const QString& id)
{
    if( !m_assignhere )
    {
        LandmarkItem* lmkChild = new LandmarkItem(this);
        lmkChild->setText(0, id);
        lmkChild->setLmkLabel( id );
        lmkChild->setLmkID( id );
        lmkChild->setLmkIndex(childCount()-1);
        setCheckState(1, Qt::Checked);
        setExpanded(true);
        treeWidget()->clearSelection();
        lmkChild->setSelected(true);
        if( m_templateTreeView )
        {
            lmkChild->addMapLmk();
            connect(lmkChild, SIGNAL(mapLmkAction(int)), this, SLOT(signalMap(int)));
        }
        else
        {
            lmkChild->deleteLmk();
            lmkChild->addRename();
            //lmkChild->addRegisterLmk();
            lmkChild->addProjectLmk();
            lmkChild->addSlideLmk();
            lmkChild->addState();
//            connect(lmkChild, SIGNAL(regLmkAction(int)), this, SLOT(signalReg(int)));
            connect(lmkChild, SIGNAL(projectLmkAction(FormItem*, ViewTreeItem*, int)), this, SLOT(project(FormItem*, ViewTreeItem*, int)));
            connect(lmkChild, SIGNAL(renameLmkAction(FormItem*, ViewTreeItem*, int, const QString&)), this, SLOT(rename(FormItem*, ViewTreeItem*, int, const QString&)));
            connect(lmkChild, SIGNAL(stateAction(FormItem*, ViewTreeItem*, int, int)), this, SLOT(state(FormItem*, ViewTreeItem*, int, int)));
            connect(lmkChild, SIGNAL(slideLmkAction(FormItem*, ViewTreeItem*, int, bool)), this, SLOT(slide(FormItem*, ViewTreeItem*, int, bool)));
        }
        emit lmkAdded(m_parentForm);
        connect(lmkChild, SIGNAL(lmkDeleteAction(int)), this, SLOT(signalDelete(int)));

        treeWidget()->resizeColumnToContents( 0 );
    }
    else
    {
        SemiLandmarksTopItem* sl = (SemiLandmarksTopItem*)m_assignhere;
        sl->addSemiLandmarkItem();
    }
}

void LandmarksTopItem::signalDelete(int index)
{
    emit lmkDeleted(m_parentForm,index);
    QTreeWidgetItem * childItem = child(index);
    removeChild(childItem);
    delete childItem;
    for(int i=0; i<childCount(); ++i)
    {
        ViewTreeItem* vti = (ViewTreeItem*)child(i);
        if( vti->getType() == LANDMARK_ITEM )
        {
            LandmarkItem* lmkItem = (LandmarkItem*)vti;
            lmkItem->setLmkIndex(i);
        }
        else if( vti->getType() == SEMILANDMARKS_ITEM )
        {
            SemiLandmarksTopItem* lmkItem = (SemiLandmarksTopItem*)vti;
            lmkItem->setLmkIndex(i);
        }
    }
}

void LandmarksTopItem::signalDelete(FormItem* semipatch, ViewTreeItem* semilmk, int index)
{
    emit lmkDeletedChild(semipatch, semilmk, index);
}

void LandmarksTopItem::enableMapChild( int index, bool enable )
{
    int size = childCount();
    for( int i = 0; i < size; ++i )
    {
        LandmarkItem* lmi = (LandmarkItem*)child( i );
        if( lmi->getLmkIndex() == index )
        {
            lmi->mapEnable( enable );
        } // end if
    }// end for loop
}

void LandmarksTopItem::enableRegChild( int index, bool enable )
{
    int size = childCount();
    for( int i = 0; i < size; ++i )
    {
        LandmarkItem* lmi = (LandmarkItem*)child( i );
        if( lmi->getLmkIndex() == index )
        {
            lmi->regEnable( enable );
        } // end if
    }// end for loop
}

void LandmarksTopItem::signalMap(int index)
{
    emit lmkMapped(m_parentForm,index);
}

void LandmarksTopItem::signalMapAll(int index)
{
    emit lmkMappedAll(m_parentForm, index);
}

void LandmarksTopItem::signalReg(int index)
{
    emit lmkRegister(m_parentForm,index);
}

QString LandmarksTopItem::getSelectedLandmarkID()
{
    if( m_templateTreeView )
    {
        for( int i = 0; i < childCount(); ++i )
        {
            ViewTreeItem* vti = (ViewTreeItem*)child(i);
            if( vti->getType() == LANDMARK_ITEM && vti->isSelected() )
            {
                LandmarkItem* lti = (LandmarkItem*)vti;
                return lti->getLmkID();
            }
        }
    }

    return "";
}

bool LandmarksTopItem::doesChildExist( const QString& parentid, const QString& childid )
{
    bool found = false;
    // first check if the item exists in the tree
    for( int i = 0; i < childCount() && !found; ++i )
    {
        ViewTreeItem* vti = (ViewTreeItem*)child(i);
        if( vti->getType() == ViewTreeItem::LANDMARK_ITEM )
        {
            LandmarkItem* li = (LandmarkItem*)vti;
            found = ( li->getLmkID() == parentid );
        }
        // deselect the semilandmark children
        else if( vti->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
        {
            SemiLandmarksTopItem* sli = (SemiLandmarksTopItem*)child(i);
            if( sli->getLmkID() == parentid )
            {
                for( int c = 0; c < sli->childCount() && !found; ++c )
                {
                    found = ( sli->child( c )->text(0) == childid );
                }
            }
        }
    }
    return found;
}

void LandmarksTopItem::select( const QString& parentid, const QString& childid )
{
    // is the child available?
    if( doesChildExist( parentid, childid) == false )
        return;

    blockSignals( true );

    // deselect the parent if it is selected.
    setSelected( false );

    for( int i = 0; i < childCount(); ++i )
    {
        ViewTreeItem* vti = (ViewTreeItem*)child(i);
        if( vti->getType() == ViewTreeItem::LANDMARK_ITEM )
        {
            LandmarkItem* li = (LandmarkItem*)vti;
            child(i)->setSelected( li->getLmkID() == parentid );
        }
        // deselect the semilandmark children
        else if( vti->getType() == ViewTreeItem::SEMILANDMARKS_ITEM )
        {
            SemiLandmarksTopItem* sli = (SemiLandmarksTopItem*)child(i);
            if( sli->getLmkID() == parentid )
            {
                sli->setSelected( (childid == "") );

                for( int c = 0; c < sli->childCount(); ++c )
                {
                    sli->child( c )->setSelected( sli->child( c )->text(0) == childid );
                }
            }
            else
            {
                sli->setSelected( false );

                for( int c = 0; c < sli->childCount(); ++c )
                {
                    sli->child( c )->setSelected( false );
                }
            }
        }
    }
    blockSignals( false );
}

void LandmarksTopItem::addSemiLandmarkSet( int t, const QString& suggestedName )
{
    int count = 1;
    for( int i = 0; i < childCount(); ++i )
    {
        ViewTreeItem* vti = (ViewTreeItem*)child( i );
        if( vti->getType() == SEMILANDMARKS_ITEM )
            count++;
    }
    QString num; num.setNum( count );
    QString lmkID = (t == 0 ) ? "Surface SemiLandmarks " : "Curve SemiLandmarks ";
    lmkID += num;
    IdDialog* detailsDialog = new IdDialog(0, lmkID, lmkID);
    detailsDialog->hideLabel();
    QString id = suggestedName;
    if( id.isEmpty() )
    {
        if( !detailsDialog->exec() )
            return;
        id = detailsDialog->getID();
    }

    SemiLandmarksTopItem* sti = new SemiLandmarksTopItem(this);
    sti->setLmkIndex(childCount()-1);
    sti->templateTreeView( m_templateTreeView );
    sti->setSemiType( t == 0 ? SemiLandmarksTopItem::SURFACE_SEMILANDMARK_ITEM : SemiLandmarksTopItem::CURVE_SEMILANDMARK_ITEM );
    sti->setText(0, id);
    sti->setLmkID( id );
    sti->addLmksHere(true);
    setExpanded(true);
    treeWidget()->clearSelection();
    sti->setSelected(true);
    if( !m_templateTreeView )
    {
        sti->deleteLmk();
        sti->addRename();
        sti->projectLmk();
        sti->slideLmks();
        sti->fixLmk();
    }
    else
    {
        sti->mapLmk();
        connect(sti, SIGNAL(lmkMap(int)), this, SLOT(signalMap(int)));
    }
    connect(sti, SIGNAL(lmkDeleteAction(int)), this, SLOT(signalDelete(int)));
    connect(sti, SIGNAL(lmkDeleteAction(FormItem* , ViewTreeItem* , int)), this, SLOT(signalDelete(FormItem* , ViewTreeItem* , int)));
    connect(sti, SIGNAL(lmkProject(FormItem*, ViewTreeItem*, int)), this, SLOT(project(FormItem*, ViewTreeItem*, int)));
    connect(sti, SIGNAL(lmkRenameAction(FormItem*, ViewTreeItem*, int, const QString&)), this, SLOT(rename(FormItem*, ViewTreeItem*, int, const QString&)));
    connect(sti, SIGNAL(lmkSlide(FormItem*, ViewTreeItem*, int, bool)), this, SLOT(slide(FormItem*, ViewTreeItem*, int, bool)));
    connect(sti, SIGNAL(lmkState(FormItem*, ViewTreeItem*, int, int)), this, SLOT(state(FormItem*, ViewTreeItem*, int, int)));
    treeWidget()->resizeColumnToContents( 0 );
    assignLandmarksHere( sti );
    assignLandmarksHere( sti );
}

void LandmarksTopItem::assignLandmarksHere( ViewTreeItem* vti )
{
    m_assignhere = vti;
    QString id = "";
    int semitype = -1;

    if( vti == 0 )
    {
        for( int i = 0; i < childCount(); ++i )
        {
            ViewTreeItem* vti = (ViewTreeItem*)child( i );
            if( vti->getType() == SEMILANDMARKS_ITEM )
            {
                SemiLandmarksTopItem* s = (SemiLandmarksTopItem*)vti;
                s->toggleAdd( false );
            }
        }

//        if( addAction )
//            addAction->setChecked( true );
    }
    else
    {
        for( int i = 0; i < childCount(); ++i )
        {
            ViewTreeItem* vti = (ViewTreeItem*)child( i );
            if( vti->getType() == SEMILANDMARKS_ITEM )
            {
                SemiLandmarksTopItem* s = (SemiLandmarksTopItem*)vti;
                if( s != m_assignhere )
                    s->toggleAdd( false );
                else
                {
                    id = s->text( 0 );
                    s->toggleAdd( true );
                    semitype = s->getSemiType() == SemiLandmarksTopItem::SURFACE_SEMILANDMARK_ITEM ? 0 : 1;
                }
            }
        }

//        if( addAction )
//            addAction->setChecked( false );
    }

    emit addLandmarkHere( id, semitype );
}
