#ifndef SEMILANDMARKSTOPITEM_H
#define SEMILANDMARKSTOPITEM_H

#include "FormItem.h"
#include "ew/view3_landmarks.h"
#include "LandmarksTopItem.h"

#include <QInputDialog>

class LandmarksTopItem;

class SemiLandmarksTopItem : public ViewTreeItem
{
Q_OBJECT

public:
    enum SemiLandmarkItemTypes  {SURFACE_SEMILANDMARK_ITEM, CURVE_SEMILANDMARK_ITEM};

private:
    bool m_templateTreeView;
    SemiLandmarkItemTypes m_semitype;
    QAction* addAction;
    QAction* slideAction;
    QAction* projAction;
    QAction* m_mapAction;
    int m_lmkIndex;
    QString m_lmkID;
    QAction* m_renAction;
    QAction* m_fixAction;

    SemiLandmarksTopItem() : ViewTreeItem(SEMILANDMARKS_ITEM), m_templateTreeView(false), m_semitype(SURFACE_SEMILANDMARK_ITEM), addAction(0), slideAction(0), projAction(0), m_mapAction(0), m_lmkIndex(-1), m_renAction(0), m_fixAction(0)
    {fillContextMenu();}

    QString createAlphaNumeric( unsigned int idnum );

public:
    SemiLandmarksTopItem(LandmarksTopItem* parent) :  ViewTreeItem(parent, SEMILANDMARKS_ITEM), m_templateTreeView(false), m_semitype(SURFACE_SEMILANDMARK_ITEM), addAction(0), slideAction(0), projAction(0), m_mapAction(0), m_lmkIndex(-1), m_renAction(0), m_fixAction(0)
    {fillContextMenu();}

    void fillContextMenu()
    {
    }

    void mapEnable( bool enable )
    {
        if( m_mapAction )
        {
            m_mapAction->setEnabled( enable );
        }
    }

    void setLmkIndex(int i) {m_lmkIndex = i;}
    int getLmkIndex() const {return m_lmkIndex;}

    void deleteLmk()
    {
        QAction* deleteAction = new QAction("Delete", m_contextMenu);
        m_contextMenu->addAction(deleteAction);
        connect(deleteAction, SIGNAL(triggered()), this, SLOT(lmkDeleted()));
    }

    void projectLmk()
    {
        projAction = new QAction("Project", m_contextMenu);
        m_contextMenu->addAction(projAction);
        connect(projAction, SIGNAL(triggered()), this, SLOT(lmkProject()));
    }

    void addRename()
    {
        m_renAction = new QAction("Rename", m_contextMenu);
        m_contextMenu->addAction(m_renAction);
        connect(m_renAction, SIGNAL(triggered()), this, SLOT(renameLmk()));
    }

    void mapLmk()
    {
        m_mapAction = new QAction("Warp to Specimen", m_contextMenu);
        m_contextMenu->addAction(m_mapAction);
        connect(m_mapAction, SIGNAL(triggered()), this, SLOT(lmkMap()));
        mapEnable( false );
    }

    void fixLmk()
    {
        m_fixAction = new QAction("Fix SemilLndmark", m_contextMenu);
        m_contextMenu->addAction(m_fixAction);
        connect(m_fixAction, SIGNAL(triggered()), this, SLOT(fix()));
    }

    void projectEnable( bool enable )
    {
        if( projAction )
        {
            projAction->setEnabled( enable );
        }
    }

    void addLmksHere(bool checked = false)
    {
        if( !m_templateTreeView )
        {
            if( !addAction )
            {
                addAction = new QAction("Current", m_contextMenu);
                addAction->setCheckable( true );
                addAction->setChecked( checked );
                m_contextMenu->addAction(addAction);
                connect(addAction, SIGNAL(toggled(bool)), this, SLOT(addHere(bool)));
            }
        }
    }

    void slideLmks()
    {
        if( !m_templateTreeView )
        {
            if( !slideAction )
            {
                slideAction = new QAction("Slide", m_contextMenu);
                //slideAction->setCheckable( true );
                //slideAction->setChecked( false );
                m_contextMenu->addAction(slideAction);
                //connect(slideAction, SIGNAL(toggled(bool)), this, SLOT(slide(bool)));
                connect(slideAction, SIGNAL(triggered()), this, SLOT(slide()));
            }
        }
    }

    void toggleAdd(bool tog)
    {
        if( addAction )
            addAction->setChecked( tog );
    }

    void templateTreeView( bool temp ) {m_templateTreeView = temp; }

    void setSemiType( SemiLandmarkItemTypes t ) {m_semitype = t;}
    SemiLandmarkItemTypes getSemiType() const {return m_semitype;}

    void setLmkID( const QString& lmkID ) { m_lmkID = lmkID; }
    QString getLmkID() {return m_lmkID;}

public slots:
    void addHere(bool tog)
    {
        ((LandmarksTopItem*)m_parentForm)->assignLandmarksHere( tog ? this : 0 );
    }

    void slide( bool tog = true )
    {
        emit lmkSlide( m_parentForm, this, 0, tog );
    }
    void lmkDeleted()
    {
        ((LandmarksTopItem*)m_parentForm)->assignLandmarksHere( 0 );
        emit lmkDeleteAction(m_lmkIndex);
    }
    void lmkDeletedChild( FormItem* semipatch, ViewTreeItem* semi, int  semiID );
    void addSemiLandmarkItem();
    void lmkProject() { emit lmkProject(m_parentForm, this, 0); }
    void lmkMap() { emit lmkMap( m_lmkIndex ); }
    void renameLmk()
    {
        bool ok;
        QString title = QInputDialog::getText( NULL, "SemiLandmark", "Enter SemiLandmark new name", QLineEdit::Normal, getLmkID(), &ok);
        if (ok)
        {
            emit lmkRenameAction( m_parentForm, this, m_lmkIndex, title );
        }
    }
    void fix() { emit lmkState( m_parentForm, this, m_lmkIndex, EW_Form3::STATE_SET ); }

signals:
    void lmkDeleteAction(int);
    void lmkDeleteAction( FormItem*, ViewTreeItem*, int );
    void lmkProject(FormItem*, ViewTreeItem*, int);
    void lmkMap(int);
    void lmkSlide( FormItem*, ViewTreeItem*, int, bool );
    void lmkRenameAction(FormItem*, ViewTreeItem*, int, const QString& newID );
    void lmkState( FormItem*, ViewTreeItem*, int, int );
};

#endif // SEMILANDMARKSTOPITEM_H
