// wdkg 2008-2010

#ifndef EW_CURVE3_H_INCLUDED
#define EW_CURVE3_H_INCLUDED 1

#include <vector>

// XXX auto try xxx.gz first, any error -> xxx
// XXX ? if filename to write ends in gz, compress automatically, otherwise not
// XXX more specific error info (line)
// XXX check not large file
// XXX more specific error classes (no file, io error, format error)
// XXX read compressed
// XXX ? mmap cached format, check for *.cache
// XXX other formats (ply), more general obj support, cache format
// XXX ?auto-detect format, ?auto cache support
// XXX ? uniquify vertices
// XXX write curve
// XXX obj: relative index

/// Piecewise-linear Curve

/// EW_Curve3 encapsulates data for piecewise-linear curves in
/// @f$\mathbb{R}^3@f$.
///
/// EW_Curve3 is a data structure that supports default construction, copy
/// construction, assignment and equality comparison.
///
/// EW_Curve3 is declared in @c ew/curve3.h.
///
/// @anchor duplicated_points
/// The data consists of a list of points and a list of edges.

class EW_Curve3 {
public:
  std::vector<float> points;
  std::vector<int> edges;
  void reset();
  void read_file(const char *file);
  inline void swap(EW_Curve3 &s);
  bool operator==(const EW_Curve3 &a) const;
  inline bool operator!=(const EW_Curve3 &a) const;
};

inline void swap(EW_Curve3 &s1, EW_Curve3 &s2);

/// @var std::vector<float> EW_Curve3::points
/// This vector has size 3 times the number of points.
/// Each consecutive triple defines a point.

/// @var std::vector<int> EW_Curve3::edges
/// This vector has size 2 times the number of edges.
/// The elements are indices of the points list, and are in the range
/// [0, number of points).
/// Each consecutive pair defines an edge.

/// @fn void EW_Curve3::reset()
/// This frees any current data and sets the object to the initial state.

/// @fn void EW_Curve3::read_file(const char *file)
/// This reads the surface contained in the @a file.
/// The following surface formats are supported:
///   - obj format
///     Only these types of record are currently supported:
///     - v
///     - l with non-negative arguments
///
/// @param file
/// The name of the file to read.
/// @exception std::bad_alloc
/// @exception EW_Curve3_IOError

/// @fn inline void EW_Curve3::swap(EW_Curve3 &s)
/// Swap data between 2 curves without copying.
/// @param s
///   The surface to swap with @c this.

/// @fn inline bool EW_Curve3::operator==(const EW_Curve3 &a) const
/// This compares member by member.

/// @fn inline bool EW_Curve3::operator!=(const EW_Curve3 &a) const
/// This compares member by member.

// The rest of this file is private.

inline void
EW_Curve3::swap(EW_Curve3 &s)
{
  points.swap(s.points);
  edges.swap(s.edges);
}

inline void
swap(EW_Curve3 &s1, EW_Curve3 &s2)
{
  s1.swap(s2);
}

inline bool
EW_Curve3::operator!=(const EW_Curve3 &a) const
{
  return !operator==(a);
}

#endif
