// wdkg 2008-2010

#ifndef EW_DATAFLOW_ARRAY3_H_INCLUDED
#define EW_DATAFLOW_ARRAY3_H_INCLUDED 1

#include "ew/bbox3.h"
#include "ew/dataflow_network.h"
#include "ew/dataflow_node.h"

// Point Array Node Base Class

// EW_Dataflow_Array3 is the base class for nodes containing a 3 dimensional
// array of points in @f$\mathbb{R}^3@f$.
//
// EW_Dataflow_Array3 is a class without assignment or comparison.
// There are private member variables.
//
// EW_Dataflow_Array3 is declared in @c ew/dataflow_array3.h.

class EW_Dataflow_Array3 : public EW_Dataflow_Node {
public:
  inline const int *get_size() const;
  inline const EW_Bbox3 *get_bbox() const;
//XXX  inline double get_scale() const;
protected:
  EW_Dataflow_Array3(EW_Dataflow_Network *nw, int ndeps, const char *n);
  mutable const int *OutpSize;
//XXX  mutable double OutpScale;
private:
  void update_bbox() const;
  mutable EW_Bbox3 OutpBbox;
  mutable EW_Dataflow_Network::cycle_t update_cycle_bbox;
  const bool DbgOnArr;
};

// @fn const int *EW_Dataflow_Array3::get_size()
// This will force the network into an updating phase.
// The pointer is valid until the node or a dependency of the node is changed.
// @return
//   A pointer to an integer array of length 3 containing the dimensions of
//   the point array.

// @fn const EW_Bbox3 *EW_Dataflow_Array3::get_bbox()
// The pointer is valid until the node or a dependency of the node is changed.
// @return
//   A pointer to the bounding box of the points in the array.

// The rest of this file is private.

inline const int *
EW_Dataflow_Array3::get_size() const
{
  EW_Dataflow_Node::update();
  return OutpSize;
}

#ifdef XXX
inline double
EW_Dataflow_Array3::get_scale() const
{
  EW_Dataflow_Node::update();
  return OutpScale;
}
#endif

inline const EW_Bbox3 *
EW_Dataflow_Array3::get_bbox() const
{
  EW_Dataflow_Network::cycle_t v = EW_Dataflow_Node::get_version();
  if (update_cycle_bbox < v) {
    update_bbox();
    update_cycle_bbox = EW_Dataflow_Node::network->get_cycle();
  }
  return &OutpBbox;
}

#endif
