// wdkg 2008-2010

#ifndef EW_DATAFLOW_ARRAY3_EXPLICIT_H_INCLUDED
#define EW_DATAFLOW_ARRAY3_EXPLICIT_H_INCLUDED 1

#include "ew/dataflow_node.h"
#include "ew/dataflow_array3.h"
#include "ew/transform3.h"

class EW_Dataflow_Network;

// Explicit Point Array Node

// EW_Dataflow_Array3_Explicit is a node containing an exlicitly specified
// regular array of points.
//
// EW_Dataflow_Array3_Explicit is a class without assignment or comparison.
// There are private member variables.
//
// EW_Dataflow_Array3_Explicit is declared in @c ew/dataflow_array3_explicit.h.
//
// Initially and when reset, the node has no points.
// In an explicit array the sizes are parameters.
// An explicit array is valid iff it has some points.
//
// Nodes of this class are input only: other nodes may depend on them but they
// never depend on other nodes.
//XXX make points

class EW_Dataflow_Array3_Explicit : public EW_Dataflow_Array3 {
public:
  explicit EW_Dataflow_Array3_Explicit(EW_Dataflow_Network *nw);
  virtual void reset();
  void set_explicit_size(const int *n);
  inline const int *get_explicit_size() const;
  void set_stretch(const double *str);
  inline const double *get_stretch() const;
  void set_mapping(const EW_Transform3 *tr);
  inline const EW_Transform3 *get_mapping() const;
private:
  virtual void update_helper() const;
  int InpSize[3];
  double InpStretch[3];
  EW_Transform3 InpMapping;
  const bool DbgOn;
};

// @fn explicit EW_Dataflow_Array3_Explicit::EW_Dataflow_Array3_Explicit(
//  EW_Dataflow_Network *nw)
// This creates an explicit array node.
// @param nw
//   The network this node should belong to.

// These are named to disambiguate EW_Dataflow_Array3::get_size().

// @fn void EW_Dataflow_Array3_Explicit::set_explicit_size(const int *n)
// This sets the size of the array.
// Each component of the size must be positive.
// 2 dimensional arrays are constucted by setting the last dimension to 1.
// Use EW_Dataflow_Node::reset to set all sizes to 0.
// @param n
//   This points to an array of size 3 containing the new sizes.

// @fn const int *EW_Dataflow_Array3_Explicit::get_explicit_size() const
// @return
//   A pointer to the current array size.

// @fn void EW_Dataflow_Array3_Explicit::set_stretch(const double *str)
// str points to an array of size 3
// XXX interpretation

// @fn const double *EW_Dataflow_Array3_Explicit::get_stretch() const
// @return
//   A pointer to the current stretch parameters.

// @fn void EW_Dataflow_Array3_Explicit::set_mapping(const EW_Transform3 *tr)
// XXX interpretation

// @fn const EW_Transform3 *EW_Dataflow_Array3_Explicit::get_mapping() const
// @return
//   A pointer to the current mapping parameter.

// The rest of this file is private.

inline const int *
EW_Dataflow_Array3_Explicit::get_explicit_size() const
{
  return InpSize;
}

inline const double *
EW_Dataflow_Array3_Explicit::get_stretch() const
{
  return InpStretch;
}

inline const EW_Transform3 *
EW_Dataflow_Array3_Explicit::get_mapping() const
{
  return &InpMapping;
}

#endif
