// wdkg 2008-2010

#ifndef EW_DATAFLOW_NOTIFIER_H_INCLUDED
#define EW_DATAFLOW_NOTIFIER_H_INCLUDED 1

#include "ew/dataflow_network.h"
#include "ew/dataflow_notifierD.h"
#include "ew/dataflow_notifier_state.h"

// XXX doc as a way of tracing network
// Class EW_Dataflow_Notifier is a utility class to provide a callback when
// anything changes in the network.
// Inherit this class, implement callback notifier_handle_cb() and call
// register() to get notified when any changes are made
// to the dataflow network.
// The notifier is intended for use when flow is controlled by an event loop.
// The handler should do nothing more than set a flag or schedule later
// processing with an idle handler.
// Typically notifier_handle_cb() will call an idle handler.
// For efficiency, it gets called before changes are instantiated in the
// network.
// The notifier is automatically unregistered when triggered.
// It should not re-register itself, but its idle handler can.

// This is designed to be privately inherited, so use redundent member names to
// make inadvertent name clashes less likely.

// Operations on the member variables must be coordinated with operations on
// EW_Dataflow_Notifier_State member variables.
// These are delegated to EW_Dataflow_Notifier_State, except for the
// initialization of IsRegistered.

// The actual declaration is in dataflow_notifierD.h.

// The rest of this file is private.

inline
EW_Dataflow_Notifier::EW_Dataflow_Notifier(EW_Dataflow_Network &network) :
 State(network.NotifierState),
// Next uninitialized
 IsRegistered(false)
{
}

inline
EW_Dataflow_Notifier::~EW_Dataflow_Notifier()
{
  State.extract(*this);
}

inline bool
EW_Dataflow_Notifier::notifier_is_registered() const
{
  return IsRegistered;
}

inline void
EW_Dataflow_Notifier::notifier_register()
{
  State.append(*this);
}

inline void
EW_Dataflow_Notifier::notifier_unregister()
{
  State.extract(*this);
}

#endif
