// wdkg 2010

#ifndef EW_DATAFLOW_NOTIFIER_STATE_H_INCLUDED
#define EW_DATAFLOW_NOTIFIER_STATE_H_INCLUDED 1

#include "ew/dataflow_notifierD.h"

// This class is private to the library.
// Only EW_Dataflow_Network and EW_Dataflow_Notifier can access it.

// This is to allow network-wide EW_Dataflow_Notifier state to be kept in
// EW_Dataflow_Network without EW_Dataflow_Notifier having access to all of
// EW_Dataflow_Network.
// It is like static data for EW_Dataflow_Notifier, except stored in
// EW_Dataflow_Network.

class EW_Dataflow_Notifier_State {
private:
  friend class EW_Dataflow_Network;
  friend class EW_Dataflow_Notifier;
// EW_Dataflow_Network
  inline EW_Dataflow_Notifier_State();
  inline ~EW_Dataflow_Notifier_State();
  inline void handle_list();
  inline bool list_is_empty() const;
// EW_Dataflow_Notifier
  inline void append(EW_Dataflow_Notifier &notifier);
  inline void extract(EW_Dataflow_Notifier &notifier);
// really private
  EW_Dataflow_Notifier *First;
  EW_Dataflow_Notifier *Last;
// undefined
  EW_Dataflow_Notifier_State(const EW_Dataflow_Notifier_State &rhs);
  EW_Dataflow_Notifier_State &operator=(const EW_Dataflow_Notifier_State &rhs);
};

inline
EW_Dataflow_Notifier_State::EW_Dataflow_Notifier_State() :
 First(0),
 Last(0)
{
}

inline
EW_Dataflow_Notifier_State::~EW_Dataflow_Notifier_State()
{
// Mark handlers as unregistered, so they don't try to unregister themselves
// later, in case of out of order destruction.
  for (EW_Dataflow_Notifier *n = First; n != 0; n = n->Next) {
    n->IsRegistered = false;
  }
}

inline bool
EW_Dataflow_Notifier_State::list_is_empty() const
{
  return (First == 0);
}

inline void
EW_Dataflow_Notifier_State::handle_list()
{
// It used to be allowed for handlers to re-register immediately.
// It is disallowed now for efficiency.
  while (First != 0) {
    First->notifier_handle_cb();
    First->IsRegistered = false;
    First = First->Next;
  }
  Last = 0;
}

inline void
EW_Dataflow_Notifier_State::append(EW_Dataflow_Notifier &notifier)
{
  if (notifier.IsRegistered) {
    return;
  }
  notifier.IsRegistered = true;
  notifier.Next = 0;
  if (First == 0) {
    First = Last = &notifier;
  } else {
    Last->Next = &notifier;
    Last = &notifier;
  }
}

inline void
EW_Dataflow_Notifier_State::extract(EW_Dataflow_Notifier &notifier)
{
  if (!notifier.IsRegistered) {
    return;
  }
  notifier.IsRegistered = false;
  if (First == &notifier) {
    First = notifier.Next;
    if (Last == &notifier) {
      Last = 0;
    }
  } else {
    for (EW_Dataflow_Notifier *n = First; n != 0; n = n->Next) {
      if (n->Next == &notifier) {
        n->Next = notifier.Next;
        if (notifier.Next == 0) {
          Last = n;
        }
        break;
      }
    }
  }
}

#endif
