// wdkg 2008-2010

#ifndef EW_DATAFLOW_SURFACE3_EXPLICIT_H_INCLUDED
#define EW_DATAFLOW_SURFACE3_EXPLICIT_H_INCLUDED 1

#include <string>
#include "ew/dataflow_node.h"
#include "ew/dataflow_surface3.h"
#include "ew/surface3.h"

class EW_Dataflow_Network;

//XXX ? uncache

/// Explicit Surface Node

/// EW_Dataflow_Surface3_Explicit is a node containing an explicitly specified
/// surface.
///
/// EW_Dataflow_Surface3_Explicit objects may not be copied or compared.
/// There are private member variables.
///
/// EW_Dataflow_Surface3_Explicit is declared in
/// @c ew/dataflow_surface3_explicit.h.
///
/// Nodes of this class do not depend on other nodes, but other nodes can
/// depend on them.
/// They are valid if they have some faces.
///
/// Initially and when reset, the node is empty.

class EW_Dataflow_Surface3_Explicit : public EW_Dataflow_Surface3 {
public:
  explicit EW_Dataflow_Surface3_Explicit(EW_Dataflow_Network *nw);
  virtual void reset();
  void set_data(EW_Surface3 *data);
  inline const EW_Surface3 *get_data() const;
  inline const char *get_filename() const;
private:
  friend class EW_Dataflow_Network;
// EW_Dataflow_Network
  std::string filename;
  bool in_cache;
// private
  virtual void update_helper() const;
  EW_Surface3 InpData;
  const bool DbgOn;
};

/// @fn EW_Dataflow_Surface3_Explicit::EW_Dataflow_Surface3_Explicit(
///  EW_Dataflow_Network *nw)
/// This creates an explicit surface node.
/// @param nw
///   The network this node should belong to.

/// @fn void EW_Dataflow_Surface3_Explicit::set_data(EW_Surface3 *data)
/// This transfers the surface data from the EW_Surface3 object to the node.
/// Afterwards, the EW_Surface3 object is reset.
/// @pre This node must not be a cached node.
/// @param data
///   The object containing the surface data.
//
/// @fn inline const EW_Surface3 *EW_Dataflow_Surface3_Explicit::get_data(
///  ) const
/// @return
///   A pointer to the internal surface data.

/// @fn const char *EW_Dataflow_Surface3_Explicit::get_filename() const
/// @return
///   If this node was created by EW_Dataflow_Network::cached_surface, the
///   filename, otherwise the empty string.

// The rest of this file is private.

inline const char *
EW_Dataflow_Surface3_Explicit::get_filename() const
{
  return filename.c_str();
}

inline const EW_Surface3 *
EW_Dataflow_Surface3_Explicit::get_data() const
{
  return &InpData;
}

#endif
