// wdkg 2008-2010

#ifndef EW_DIG3_SPACE_H_INCLUDED
#define EW_DIG3_SPACE_H_INCLUDED 1

#include <string>
#include <vector>
#include "ew/dataflow_form3.h"

class EW_Bbox3;
class EW_Dataflow_Curve3_Explicit;
class EW_Dataflow_Form3;
class EW_Dataflow_Network;
class EW_Dataflow_Surface3_Explicit;
class EW_Dig3;
class EW_Dig3_View;
class EW_Form3;
class EW_Form3_Embedding;
class EW_Form3_PointSet;
class EWDebug;

// XXX for now, forms must be empty or have a filename (last load and save,
// not reset, but changes ok) before save records can be recorded

/// Space Element

/// EW_Dig3_Space implements a space element of EW_Dig3.
/// There are 2 spaces, the template space and the specimen space.
///
/// EW_Dig3_Space is a class without assignment or comparison.
/// There are private member variables.
///
/// EW_Dig3_Space is declared in ew/dig3.h.

// EW_Dig3 has no concept of filename, but EW_Dig3_Tableau must have filenames
// for the forms3's.
// This does not keep track of form3 filenames.
// When creating a save record, these must be filled in.
// When loading a save record, the form3 files must be read if necessary and
// loaded first.

class EW_Dig3_Space {
public:
  inline const EW_Dataflow_Form3 *get_form_node();
  void reset_form();
  void set_form_data(const EW_Form3 *form_data);
  inline const EW_Form3 *get_form_data() const;
//XXX version cnt, or dirty flag
//XXX ??? filename of form, ?if not saved, ?dirty
  int set_form_curve(bool *replaced, const EW_Form3_Curve *cr);
  void remove_form_curve(int n);
  int set_form_surface(bool *replaced, const EW_Form3_Surface *sr);
  void remove_form_surface(int n);
  inline int set_form_pointset(bool *replaced, const EW_Form3_PointSet *ps);
  inline void set_form_pointset_location(int n, int i, const double *loc);
  inline void set_form_pointset_relax(int n, int i, int rdim,
   const double *rparam);
  inline void remove_form_pointset(int n);
  inline int set_form_embedding(bool *replaced, const EW_Form3_Embedding *em);
  inline void remove_form_embedding(int n);
  int get_curve_of_pointset(int ps) const;
  int get_surface_of_pointset(int ps) const;
  void get_bbox(EW_Bbox3 *b) const;
  inline const EW_Dataflow_Curve3_Explicit *const *get_curve_nodes() const;
  inline const EW_Dataflow_Surface3_Explicit *const *get_surface_nodes() const;
  inline int get_index() const;
  EW_Dig3 *const dig3;
  EW_Dataflow_Network *const network;
private:
  friend class EW_Dig3;
  friend class EW_Dig3_View;
// EW_Dig3
  EW_Dig3_Space(EW_Dig3 *dig3, int index);
  ~EW_Dig3_Space();
  EW_Dataflow_Form3 *form_node;
// EW_Dig3 and EW_Dig3_View
  std::vector<const EW_Dataflow_Curve3_Explicit *> curve_nodes;
  std::vector<const EW_Dataflow_Surface3_Explicit *> surface_nodes;
// private
  const int index;
  const EWDebug &Dbg;
  const bool DbgOn;
  const int DbgN;
  const std::string DbgStr;
public:
// private to library
  const char *const DbgId;
private:
// undefined
  EW_Dig3_Space(const EW_Dig3_Space &rhs);
  EW_Dig3_Space &operator=(const EW_Dig3_Space &rhs);
};

/// @fn inline const EW_Dataflow_Form3 *EW_Dig3_Space::get_form_node()
/// This returns a pointer to the EW_Dataflow_Form3 node managed by the
/// EW_Dig3_Space instance.
/// This node can be changed using EW_Dig3_Space wrappers for the
/// EW_Dataflow_Form3 methods.

/// @fn void EW_Dig3_Space::reset_form()
/// This set the space to an empty form.

/// @fn void EW_Dig3_Space::set_form_data(const EW_Form3 *form_data)
/// This sets the space to a new form.
/// @param form_data
///   The form data to copy.

/// @fn const EW_Form3 *EW_Dig3_Space::get_form_data() const
/// @return
///   A pointer to the EW_From3 data of the space.

/// @fn int EW_Dig3_Space::set_form_curve(bool *replaced,
///  const EW_Form3_Curve *cr)
/// This adds or replaces a curve in the space.
/// @param[out] replaced
///   @c true if the curve replaced an existing curve.
/// @param cr
///   A pointer to the curve data to copy.
/// @return
///   The index of the curve.

/// @fn void EW_Dig3_Space::remove_form_curve(int n)
/// This deletes a curve from the space.
/// @param n
///   The index of the curve to delete.

/// @fn int EW_Dig3_Space::set_form_surface(bool *replaced,
///  const EW_Form3_Surface *sr)
/// This adds or replaces a surface in the space.
/// @param[out] replaced
///   @c true if the surface replaced an existing surface.
/// @param sr
///   A pointer to the surface data to copy.
/// @return
///   The index of the surface.

/// @fn void EW_Dig3_Space::remove_form_surface(int n)
/// This deletes a surface from the space.
/// @param n
///   The index of the surface to delete.

/// @fn inline int EW_Dig3_Space::set_form_pointset(bool *replaced,
///  const EW_Form3_PointSet *ps)
/// This adds or replaces a pointset in the space.
/// @param[out] replaced
///   @c true if the point set replaced an existing point set.
/// @param ps
///   A pointer to the point set data to copy.
/// @return
///   The index of the point set.

/// @fn inline void EW_Dig3_Space::set_form_pointset_location(int n, int i,
///  const double *loc);
/// This changes the coordinates of an element of the point set.
/// @param n
///   The index of the point set.
/// @param i
///   The index of the element in the point set.
/// @param[in] loc
///   A pointer to the new coordinates of this entry.

/// @fn inline void EW_Dig3_Space::set_form_pointset_relax(int n, int i,
///  int rdim, const double *rparam);
/// This changes the relaxation parameters of an element of the point set.
/// @param n
///   The index of the point set.
/// @param i
///   The index of the element in the point set.
/// @param[in] rdim
///   The new relax_dims of this entry.
/// @param[in] rparam
///   A pointer to the new relax_params of this entry.

/// @fn inline void EW_Dig3_Space::remove_form_pointset(int n)
/// This deletes a point set from the space.
/// @param n
///   The index of the point set to delete.

/// @fn inline int EW_Dig3_Space::set_form_embedding(bool *replaced,
///  const EW_Form3_Embedding *em)
/// This adds or replaces a embedding in the space.
/// @param[out] replaced
///   @c true if the embedding replaced an existing embedding.
/// @param em
///   A pointer to the embedding data to copy.
/// @return
///   The index of the embedding.

/// @fn inline void EW_Dig3_Space::remove_form_embedding(int n)
/// This deletes a embedding from the space.
/// @param n
///   The index of the embedding to delete.

// Doxygen comment without long line or g++ multiline comment warning.
/*!
 * @fn inline const EW_Dataflow_Curve3_Explicit *const \
 *  *EW_Dig3_Space::get_curve_nodes() const
 * @return
 *   The curve node index.
 *   The i'th entry is a pointer to the curve node corresponding to the i'th
 *   curve, or zero if this curve has no data.
 */

// Doxygen comment without long line or g++ multiline comment warning.
/*!
 * @fn inline const EW_Dataflow_Surface3_Explicit *const \
 *  *EW_Dig3_Space::get_surface_nodes() const
 * @return
 *   The surface node index.
 *   The i'th entry is a pointer to the surface node corresponding to the i'th
 *   surface, or zero if this surface has no data.
 */

/// @fn int EW_Dig3_Space::get_curve_of_pointset(int ps) const
/// This finds the curve a pointset is embedded in if any.
/// @param ps
///   The index of the pointset.
/// @return
///   The index of the curve, or -1.

/// @fn int EW_Dig3_Space::get_surface_of_pointset(int ps) const
/// This finds the surface a pointset is embedded in if any.
/// @param ps
///   The index of the pointset.
/// @return
///   The index of the surface, or -1.

/// @fn void EW_Dig3_Space::get_bbox(EW_Bbox3 *b) const
/// This calculates the bounding box of all elements of the space's form.
/// @param[out] b
///   Where to store the bounding box.

/// @fn int EW_Dig3_Space::get_index() const
/// @return
///   The index of this space element in the space index of #dig3.
///   0 if this is the template space, 1 if this is the specimen space.

/// @var EW_Dataflow_Network *const EW_Dig3_Space::network
/// This is a pointer to the EW_Dataflow_Network that the #dig3 is associated
/// with.

/// @var EW_Dig3 *const EW_Dig3_Space::dig3
/// This is a pointer to the EW_Dig3 instance that this space element belongs
/// to.

// The rest of this file is private.

inline const EW_Form3 *
EW_Dig3_Space::get_form_data() const
{
  return form_node->get_data();
}

inline const EW_Dataflow_Curve3_Explicit *const *
EW_Dig3_Space::get_curve_nodes() const
{
  return curve_nodes.empty() ? 0 : &curve_nodes[0];
}

inline const EW_Dataflow_Surface3_Explicit *const *
EW_Dig3_Space::get_surface_nodes() const
{
  return surface_nodes.empty() ? 0 : &surface_nodes[0];
}

inline int
EW_Dig3_Space::get_index() const
{
  return index;
}

inline const EW_Dataflow_Form3 *
EW_Dig3_Space::get_form_node()
{
  return form_node;
}

inline int
EW_Dig3_Space::set_form_pointset(bool *replaced, const EW_Form3_PointSet *ps)
{
  return form_node->set_pointset(replaced, ps);
}

inline void
EW_Dig3_Space::set_form_pointset_location(int n, int i, const double *loc)
{
  form_node->set_pointset_location(n, i, loc);
}

inline void
EW_Dig3_Space::set_form_pointset_relax(int n, int i, int rdim,
 const double *rparam)
{
  form_node->set_pointset_relax(n, i, rdim, rparam);
}

inline void
EW_Dig3_Space::remove_form_pointset(int n)
{
  form_node->remove_pointset(n);
}

inline int
EW_Dig3_Space::set_form_embedding(bool *replaced, const EW_Form3_Embedding *em)
{
  return form_node->set_embedding(replaced, em);
}

inline void
EW_Dig3_Space::remove_form_embedding(int n)
{
  form_node->remove_embedding(n);
}

#endif
