// wdkg 2008-2010

#ifndef EW_DIG3_TABLEAU_H_INCLUDED
#define EW_DIG3_TABLEAU_H_INCLUDED 1

#include <stdexcept>
#include <string>
#include <vector>
#include "ew/dig3_tableau_space.h"
#include "ew/transform3.h"

//XXX auto try xxx.gz first, any error -> xxx
//XXX ?? pointer to form3 as alternative to filename, so can have tableau
//        records in memory without first saving form file
//XXX id=* or more generally patterns
//XXX check when apply support mismatches
//XXX space: line width, translucency, outline, cross, ...
//XXX ? move clip ratio to space
//XXX ? view clip ratio (large to avoid clipping)
//XXX ? move bg to space
//XXX allow 4 views to be not present (-1)

/// Viewing State Record

/// EW_Dig3_Tableau contains a partial record of the state of an instance of
/// EW_Dig3.
///
/// EW_Dig3_Tableau is a data structure that supports default construction,
/// copy construction, assignment and equality comparison.
///
/// EW_Dig3_Tableau is declared in @c ew/dig3_tableau.h.
///
/// EW_Dig3_Tableau records the EW_Dig3 space forms indirectly as file names.
/// It does not record the EW_Dig3 views, their widgets or their linkages.
/// It records the settings of one each of:
///   - a template main view
///   - a template slice view
///   - a specimen main view
///   - a specimen slice view
///
/// Some settings will be assumed to be common to some or all of the windows.
/// It does not record state that is not controlled by the library, such as the
/// window sizes, file browser history or transient state such as the selected
/// item, window configuration, or gui modal state.
///
/// EW_Dig3_Tableau lists are represented in save files read by #read_file and
/// #write_file.
/// Here is an example:
///   - @ref example.sav
/// The representation is exact except that filenames in the form files are
/// abbreviated to relative filenames in common situations were possible, but
/// in EW_Dig3_Tableau they are always in canonical form.

class EW_Dig3_Tableau {
public:
  EW_Dig3_Tableau_Space space[2];
  double slice_clip_ratio;
  unsigned char bg[3];
  EW_Dig3_Tableau();
  void reset();
  static void read_file(std::vector<EW_Dig3_Tableau> *outp,
   const char *file);
  static void write_file(const char *file, bool compress,
   const std::vector<EW_Dig3_Tableau> *inp);
  bool operator==(const EW_Dig3_Tableau &a) const;
  inline bool operator!=(const EW_Dig3_Tableau &a) const;
};

/// @var EW_Dig3_Tableau_Space EW_Dig3_Tableau::space[2]
/// The template and specimen space states.

/// @var double EW_Dig3_Tableau::slice_clip_ratio
/// The slice clip ratio.
/// The default is 0.01.

/// @var unsigned char EW_Dig3_Tableau::bg[3]
/// The window background.
/// The default is [0, 0, 0].

/// @fn EW_Dig3_Tableau::EW_Dig3_Tableau()
/// This sets intrinsic member variables to their default values.

/// @fn EW_Dig3_Tableau::reset()
/// This resets the object to its initial state.

/// @fn void EW_Dig3_Tableau::read_file(std::vector<EW_Dig3_Tableau> *outp,
///  const char *file)
/// This reads a save file.
/// The file can be compressed with gzip compression.
/// @param[in] outp
///   The vector to store the records read in.
/// @param file
///   The file name.
/// @exception std::bad_alloc
/// @exception EW_Dig3_Tableau_IOError

//XXX ??make this take an array, not a vector
/// @fn void EW_Dig3_Tableau::write_file(const char *file, bool compress,
///  const std::vector<EW_Dig3_Tableau> *inp)
/// This writes a save file.
/// @param file
///   The file name.
/// @param compress
///   If @c true, write the file compressed with gzip compression.
/// @param[in] inp
///   The vector of records to write.
/// @exception std::bad_alloc
/// @exception EW_Dig3_Tableau_IOError

/// @fn inline bool EW_Dig3_Tableau::operator==(const EW_Dig3_Tableau &a) const
/// This compares member by member.

/// @fn inline bool EW_Dig3_Tableau::operator!=(const EW_Dig3_Tableau &a) const
/// This compares member by member.

// The rest of this file is private.

inline
EW_Dig3_Tableau::EW_Dig3_Tableau() :
 slice_clip_ratio(0.01)
{
  bg[0] = 0;
  bg[1] = 0;
  bg[2] = 0;
}

inline bool
EW_Dig3_Tableau::operator!=(const EW_Dig3_Tableau &a) const
{
  return !operator==(a);
}

#endif
