// wdkg 2008-2010

#ifndef EW_DIG3_TABLEAU_CURVE_H_INCLUDED
#define EW_DIG3_TABLEAU_CURVE_H_INCLUDED 1

#include <string>

/// Curve Element

/// EW_Dig3_Tableau_Curve contains data for a curve element of
/// EW_Dig3_Tableau.
///
/// EW_Dig3_Tableau_Curve is a data structure that supports default
/// construction, copy construction, assignment and equality comparison.
///
/// EW_Dig3_Tableau_Curve is declared in @c ew/dig3_tableau_curve.h.

class EW_Dig3_Tableau_Curve {
public:
  EW_Dig3_Tableau_Curve();
  std::string id;
  bool show_in_main;
  bool show_in_slice;
  unsigned char col[3];
//XXX linewidth
  inline bool operator==(const EW_Dig3_Tableau_Curve &a) const;
  inline bool operator!=(const EW_Dig3_Tableau_Curve &a) const;
};

/// @var std::string EW_Dig3_Tableau_Curve::id
/// The EW_Form3_Curve::id of the curve this field should apply to.

/// @var bool EW_Dig3_Tableau_Curve::show_in_main
/// @c true if the curve should be displayed in the main view.
/// The default is @c true.

/// @var bool EW_Dig3_Tableau_Curve::show_in_slice
/// @c true if the curve should be displayed in the slice view.
/// The default is @c true.

/// @var unsigned char EW_Dig3_Tableau_Curve::col[3]
/// The colour of the curve.
/// The default is [128, 128, 128].

/// @fn EW_Dig3_Tableau_Curve::EW_Dig3_Tableau_Curve()
/// This sets intrinsic member variables to their default values.

/// @fn inline bool EW_Dig3_Tableau_Curve::operator==(
///  const EW_Dig3_Tableau_Curve &a) const
/// This compares member by member.

/// @fn inline bool EW_Dig3_Tableau_Curve::operator!=(
///  const EW_Dig3_Tableau_Curve &a) const
/// This compares member by member.

// The rest of this file is private.

inline
EW_Dig3_Tableau_Curve::EW_Dig3_Tableau_Curve() :
 show_in_main(true),
 show_in_slice(true)
{
  col[0] = 128;
  col[1] = 128;
  col[2] = 128;
}

inline bool
EW_Dig3_Tableau_Curve::operator==(const EW_Dig3_Tableau_Curve &a) const
{
  return
   id == a.id &&
   show_in_main == a.show_in_main &&
   show_in_slice == a.show_in_slice &&
   col[0] == a.col[0] &&
   col[1] == a.col[1] &&
   col[2] == a.col[2];
}

inline bool
EW_Dig3_Tableau_Curve::operator!=(const EW_Dig3_Tableau_Curve &a) const
{
  return !operator==(a);
}

#endif
