// wdkg 2008-2010

#ifndef EW_DIG3_TABLEAU_SURFACE_H_INCLUDED
#define EW_DIG3_TABLEAU_SURFACE_H_INCLUDED 1

#include <string>

/// Surface Element

/// EW_Dig3_Tableau_Surface contains data for a surface element of
/// EW_Dig3_Tableau.
///
/// EW_Dig3_Tableau_Surface is a data structure that supports default
/// construction, copy construction, assignment and equality comparison.
///
/// EW_Dig3_Tableau_Surface is declared in @c ew/dig3_tableau_surface.h.

class EW_Dig3_Tableau_Surface {
public:
  EW_Dig3_Tableau_Surface();
  std::string id;
  bool show_in_main;
  bool show_in_slice;
  unsigned char front_col[3];
  unsigned char back_col[3];
  inline bool operator==(const EW_Dig3_Tableau_Surface &a) const;
  inline bool operator!=(const EW_Dig3_Tableau_Surface &a) const;
};

/// @var std::string EW_Dig3_Tableau_Surface::id
/// The EW_Form3_Surface::id of the surface this field should apply to.

/// @var bool EW_Dig3_Tableau_Surface::show_in_main
/// @c true if the surface should be displayed in the main view.
/// The default is @c true.

/// @var bool EW_Dig3_Tableau_Surface::show_in_slice
/// @c true if the surface should be displayed in the slice view.
/// The default is @c true.

/// @var unsigned char EW_Dig3_Tableau_Surface::front_col[3]
/// The colour of the front side of the surface.
/// The default is [128, 128, 128].

/// @var unsigned char EW_Dig3_Tableau_Surface::back_col[3]
/// The colour of the back side of the surface.
/// The default is [128, 128, 128].

/// @fn EW_Dig3_Tableau_Surface::EW_Dig3_Tableau_Surface()
/// This sets intrinsic member variables to their default values.

/// @fn inline bool EW_Dig3_Tableau_Surface::operator==(
///  const EW_Dig3_Tableau_Surface &a) const
/// This compares member by member.

/// @fn inline bool EW_Dig3_Tableau_Surface::operator!=(
///  const EW_Dig3_Tableau_Surface &a) const
/// This compares member by member.

// The rest of this file is private.

inline
EW_Dig3_Tableau_Surface::EW_Dig3_Tableau_Surface() :
 show_in_main(true),
 show_in_slice(true)
{
  front_col[0] = 128;
  front_col[1] = 128;
  front_col[2] = 128;
  back_col[0] = 128;
  back_col[1] = 128;
  back_col[2] = 128;
}

inline bool
EW_Dig3_Tableau_Surface::operator==(const EW_Dig3_Tableau_Surface &a) const
{
  return
   id == a.id &&
   show_in_main == a.show_in_main &&
   show_in_slice == a.show_in_slice &&
   front_col[0] == a.front_col[0] &&
   front_col[1] == a.front_col[1] &&
   front_col[2] == a.front_col[2] &&
   back_col[0] == a.back_col[0] &&
   back_col[1] == a.back_col[1] &&
   back_col[2] == a.back_col[2];
}

inline bool
EW_Dig3_Tableau_Surface::operator!=(const EW_Dig3_Tableau_Surface &a) const
{
  return !operator==(a);
}

#endif
