// wdkg 2008-2010

#ifndef EW_DIG3_VIEW_H_INCLUDED
#define EW_DIG3_VIEW_H_INCLUDED 1

#include <vector>
#include "ew/dataflow_form3.h"
#include "ew/form3.h"
#include "ew/transform3.h"
#include "ew/view3_image.h"
#include "ew/view3_landmarks.h"

class EW_Dataflow_Array3_Explicit;
class EW_Dataflow_Image3;
class EW_Dataflow_Network;
class EW_Dig3;
class EW_Dig3_Space;
class EW_View3_Curve;
class EW_View3_Surface;
class EW_View3_Widget;

/// View Element

/// EW_Dig3_View implements a view element of EW_Dig3.
///
/// EW_Dig3_View is a class without assignment or comparison.
/// There are private member variables.
///
/// EW_Dig3_View is declared in ew/dig3_view.h.
///
/// An instance of this class controls an EW_View3_Widget instance.
/// The settings and item settings of this widget, except for the view mapping,
/// may be directly manipulated.
/// The toolkit interaction of this widget should be handled by the inheriting
/// class of the widget.
/// Otherwise, this widget should not be directly manipulated.
/// Only one view element should be associated with a given widget.

//XXX doc and enforce conditions on links (set_link, destroy)

class EW_Dig3_View {
public:
  EW_Dig3_View(EW_Dig3 *dig3, int space, int slice_mode,
   EW_View3_Widget *widget);
  ~EW_Dig3_View();
  //XXX accessor funs for view_link view_link_tr
  void set_slice_mode(int sm);
  inline int get_slice_mode() const;
  void set_space(int sp);
  inline int get_space() const;
  void set_link(EW_Dig3_View *v, const EW_Transform3 *tr);
  void set_view_mapping(const EW_Transform3 *tr);
  void view_post_compose(const EW_Transform3 *tr);
  inline EW_View3_Image *const *get_slice_items() const;
  inline EW_View3_Curve *const *get_curve_items() const;
  inline EW_View3_Surface *const *get_surface_items() const;
  inline EW_View3_Landmarks *get_landmarks_item() const;
  inline int get_index() const;
  EW_View3_Widget *const widget;
  EW_Dataflow_Network *const network;
  EW_Dig3 *const dig3;
private:
  friend class EW_Dig3_Space;
  friend class EW_Dig3;
// EWDig3 and EW_Dig3_Space
  std::vector<EW_View3_Curve *> curve_item_index;
  std::vector<EW_View3_Surface *> surface_item_index;
// EWDig3
  std::vector<EW_View3_Image *> slice_item_index;
// private
  void sync_view();
  int index;
  int space;
  int slice_mode;
  EW_Dig3_View *view_link;
  EW_Transform3 view_link_tr;
  EW_Dataflow_Array3_Explicit *slice_array_node;
  EW_Dataflow_Image3 *slice_image_node;
  EW_View3_Landmarks *landmarks_item;
  const EWDebug &Dbg;
  const bool DbgOn;
  const int DbgN;
  const std::string DbgStr;
public:
// private to library
  const char *const DbgId;
private:
// undefined
  EW_Dig3_View(const EW_Dig3_View &rhs);
  EW_Dig3_View &operator=(const EW_Dig3_View &rhs);
};

/// @fn EW_Dig3_View::EW_Dig3_View(EW_Dig3 *dig3, int space, int slice_mode,
///  EW_View3_Widget *widget)
/// This creates a view element of the @a dig3, and associates it with the
/// @a widget.
/// @param dig3
///   The EW_Dig3 instance this view element will belong to.
/// @param space
///   Which space of the @a dig3 this view element will initially display.
///   Either 0 or 1.
/// @param slice_mode
///   Whether this will be a slice view.
///   Either 0 or 1.
/// @param widget
///   The EW_View3_Widget this view element will manage.

/// @fn EW_Dig3_View::~EW_Dig3_View()
/// This removes the view from #dig3, disassociates it from #widget and
/// destroys it.
/// The #widget is not destroyed.

/// @fn void EW_Dig3_View::set_slice_mode(int sm)
/// @param sm
///   Whether this view should now be a slice view.
///   Either 0 or 1.

/// @fn int EW_Dig3_View::get_slice_mode() const
/// @return
///   Whether this is a slice view.
///   Either 0 or 1.

/// @fn void EW_Dig3_View::set_space(int sp)
/// @param sp
///   The index of the space this view should now be associated with.
///   Either 0 or 1.

/// @fn int EW_Dig3_View::get_space() const
/// @return
///   The index of the space this view is currently associated with.
///   Either 0 or 1.

/// @fn void EW_Dig3_View::set_link(EW_Dig3_View *v, const EW_Transform3 *tr)
/// This sets up a link between the view mapping of this view and of another
/// one.
/// Links must be disjoint cycles.
/// The composition of the transforms should be the identity for sensible
/// results.
/// @param v
///   The view to link to.
/// @param tr
///   The transform that should be left-composed with this view's view mapping
///   to get the link views view mapping.

/// @fn void EW_Dig3_View::set_view_mapping(const EW_Transform3 *tr)
/// This operates on the view mapping of #widget, and synchronizes with other
/// views and with slices as necessary.
/// @param tr
///   As for EW_View3_Widget::set_view_mapping().

/// @fn void EW_Dig3_View::view_post_compose(const EW_Transform3 *tr)
/// This operates on the view mapping of #widget, and synchronizes with other
/// views and with slices as necessary.
/// @param tr
///   As for EW_View3_Widget::view_post_compose().

/// @fn inline EW_View3_Image *const *EW_Dig3_View::get_slice_items() const
/// @return
///   The slice item index.
///   The i'th entry is the item displaying the slice of the i'th view of the
///   #dig3, or zero if the i'th view is not in slice mode, or is not
///   associated with the same space.

/// @fn EW_View3_Curve *const *EW_Dig3_View::get_curve_items() const
/// @return
///   The curve item index.
///   The i'th entry is the item displaying the i'th curve of the space this
///   view is currently associated with.

/// @fn EW_View3_Surface *const *EW_Dig3_View::get_surface_items() const
/// @return
///   The surface item index.
///   The i'th entry is the item displaying the i'th surface of the space this
///   view is currently associated with.

/// @fn EW_View3_Landmarks *const *EW_Dig3_View::get_landmarks_item() const
/// @return
///   The landmarks item.

/// @fn int EW_Dig3_View::get_index() const
/// @return
///   The current index of this view element in the view index of #dig3.

/// @var EW_View3_Widget *const EW_Dig3_View::widget
/// This is a pointer to the EW_View3_Widget that this view element is
/// associated with.

/// @var EW_Dataflow_Network *const EW_Dig3_View::network
/// This is a pointer to the EW_Dataflow_Network that the #dig3 and #widget
/// are associated with.

/// @var EW_Dig3 *const EW_Dig3_View::dig3
/// This is a pointer to the EW_Dig3 instance that this view element belongs
/// to.

// The rest of this file is private.

inline int
EW_Dig3_View::get_slice_mode() const
{
  return slice_mode;
}

inline int
EW_Dig3_View::get_space() const
{
  return space;
}

inline EW_View3_Image *const *
EW_Dig3_View::get_slice_items() const
{
  return slice_item_index.empty() ? 0: &slice_item_index[0];
}

inline EW_View3_Curve *const *
EW_Dig3_View::get_curve_items() const
{
  return curve_item_index.empty() ? 0: &curve_item_index[0];
}

inline EW_View3_Surface *const *
EW_Dig3_View::get_surface_items() const
{
  return surface_item_index.empty() ? 0: &surface_item_index[0];
}

inline EW_View3_Landmarks *
EW_Dig3_View::get_landmarks_item() const
{
  return landmarks_item;
}

inline int
EW_Dig3_View::get_index() const
{
  return index;
}

#endif
