// wdkg 2008-2010

#ifndef EW_FORM3_CURVE_H_INCLUDED
#define EW_FORM3_CURVE_H_INCLUDED 1

#include <string>

/// Curve Element

/// EW_Form3_Curve contains data for a curve element of EW_Form3.
///
/// EW_Form3_Curve is a data structure that supports default construction,
/// copy construction, assignment and equality comparison.
///
/// EW_Form3_Curve is declared in @c ew/form3_curve.h.

class EW_Form3_Curve {
public:
  std::string id;
  std::string file;
  int state;
  inline EW_Form3_Curve();
  inline bool operator==(const EW_Form3_Curve &a) const;
  inline bool operator!=(const EW_Form3_Curve &a) const;
};

/// @var std::string EW_Form3_Curve::id
/// The id of this element used as a key to refer to this element and as a
/// label.

/// @var std::string EW_Form3_Curve::file
/// The canonical name of the file containing the data for this element, or an
/// empty string indicating that there is no associated file.

/// @var int EW_Form3_Curve::state
/// A code indicating the digitizing state of the curve.
/// Its value is a EW_Form3::StateType.
/// The default is 0.

/// @fn EW_Form3_Curve::EW_Form3_Curve()
/// This sets intrinsic member variables to their default values.

/// @fn inline bool EW_Form3_Curve::operator==(const EW_Form3_Curve &a) const
/// This compares member by member.

/// @fn inline bool EW_Form3_Curve::operator!=(const EW_Form3_Curve &a) const
/// This compares member by member.

// The rest of this file is private.

inline
EW_Form3_Curve::EW_Form3_Curve() :
 state(0)
{
}

inline bool
EW_Form3_Curve::operator==(const EW_Form3_Curve &a) const
{
  return id == a.id && file == a.file && state == a.state;
}

inline bool
EW_Form3_Curve::operator!=(const EW_Form3_Curve &a) const
{
  return !operator==(a);
}

#endif
