// wdkg 2008-2010

#ifndef EW_FORM3_EMBEDDING_H_INCLUDED
#define EW_FORM3_EMBEDDING_H_INCLUDED 1

#include <string>

/// Embedding Relation

/// EW_Form3_Embedding contains data for an embedding relation of EW_Form3.
/// It is used to indicate subset relationships between point, curve and
/// surface elements.
///
/// EW_Form3_Embedding is a data structure that supports default construction,
/// copy construction, assignment and equality comparison.
///
/// EW_Form3_Embedding is declared in @c ew/form3_embedding.h.

class EW_Form3_Embedding {
public:
  std::string subset_id;
  std::string superset_id;
  inline bool operator==(const EW_Form3_Embedding &a) const;
  inline bool operator!=(const EW_Form3_Embedding &a) const;
};

/// @var std::string EW_Form3_Embedding::subset_id
/// The id of the lower dimensional element of the embedding.

/// @var std::string EW_Form3_Embedding::superset_id
/// The id of the higher dimensional element of the embedding.

/// @fn inline bool EW_Form3_Embedding::operator==(
///  const EW_Form3_Embedding &a) const
/// This compares member by member.

/// @fn inline bool EW_Form3_Embedding::operator!=(
///  const EW_Form3_Embedding &a) const
/// This compares member by member.

// The rest of this file is private.

inline bool
EW_Form3_Embedding::operator==(const EW_Form3_Embedding &a) const
{
  return subset_id == a.subset_id && superset_id == a.superset_id;
}

inline bool
EW_Form3_Embedding::operator!=(const EW_Form3_Embedding &a) const
{
  return !operator==(a);
}

#endif
