// wdkg 2008-2010

#ifndef EW_FORM3_REFLECTION_H_INCLUDED
#define EW_FORM3_REFLECTION_H_INCLUDED 1

#include <string>

/// Reflection Relation

/// EW_Form3_Reflection contains data for a reflection relation of EW_Form3.
/// Only symmetries where one element is on the left and one on the right,
/// or self symmetries for elements symbolically in the center plane, are
/// supported.
///
/// EW_Form3_Reflection is a data structure that supports default construction,
/// copy construction, assignment and equality comparison.
///
/// EW_Form3_Reflection is declared in @c ew/form3_reflection.h.

class EW_Form3_Reflection {
public:
  std::string left_id;
  std::string right_id;
  inline bool operator==(const EW_Form3_Reflection &a) const;
  inline bool operator!=(const EW_Form3_Reflection &a) const;
};

/// @var std::string EW_Form3_Reflection::left_id
/// The id of the left element of the symmetry.

/// @var std::string EW_Form3_Reflection::right_id
/// The id of the right element of the symmetry.

/// @fn inline bool EW_Form3_Reflection::operator==(
///  const EW_Form3_Reflection &a) const
/// This compares member by member.

/// @fn inline bool EW_Form3_Reflection::operator!=(
///  const EW_Form3_Reflection &a) const
/// This compares member by member.

// The rest of this file is private.

inline bool
EW_Form3_Reflection::operator==(const EW_Form3_Reflection &a) const
{
  return left_id == a.left_id && right_id == a.right_id;
}

inline bool
EW_Form3_Reflection::operator!=(const EW_Form3_Reflection &a) const
{
  return !operator==(a);
}

#endif
