// wdkg 2008-2010

#ifndef EW_FORM3_SURFACE_H_INCLUDED
#define EW_FORM3_SURFACE_H_INCLUDED 1

#include <string>

/// Surface Element

/// EW_Form3_Surface contains data for a surface element of EW_Form3.
///
/// EW_Form3_Surface is a data structure that supports default construction,
/// copy construction, assignment and equality comparison.
///
/// EW_Form3_Surface is declared in @c ew/form3_surface.h.

class EW_Form3_Surface {
public:
  std::string id;
  std::string file;
  inline bool operator==(const EW_Form3_Surface &a) const;
  inline bool operator!=(const EW_Form3_Surface &a) const;
};

/// @var std::string EW_Form3_Surface::id
/// The id of this element used as a key to refer to this element and as a
/// label.

/// @var std::string EW_Form3_Surface::file
/// The canonical name of the file containing the data for this element.
/// An empty string indicates that there is no associated file.

/// @fn inline bool EW_Form3_Surface::operator==(
///  const EW_Form3_Surface &a) const
/// This compares member by member.

/// @fn inline bool EW_Form3_Surface::operator!=(
///  const EW_Form3_Surface &a) const
/// This compares member by member.

// The rest of this file is private.

inline bool
EW_Form3_Surface::operator==(const EW_Form3_Surface &a) const
{
  return id == a.id && file == a.file;
}

inline bool
EW_Form3_Surface::operator!=(const EW_Form3_Surface &a) const
{
  return !operator==(a);
}

#endif
