// wdkg 2008-2010

#ifndef EW_FORM3_VOLUME_H_INCLUDED
#define EW_FORM3_VOLUME_H_INCLUDED 1

#include <string>

/// Volume Element

/// EW_Form3_Volume contains data for a volume element of EW_Form3.
///
/// EW_Form3_Volume is a data structure that supports default construction,
/// copy construction, assignment and equality comparison.
///
/// EW_Form3_Volume is declared in @c ew/form3_volume.h.

class EW_Form3_Volume {
public:
  std::string id;
  std::string file;
  inline bool operator==(const EW_Form3_Volume &a) const;
  inline bool operator!=(const EW_Form3_Volume &a) const;
};

/// @var std::string EW_Form3_Volume::id
/// The id of this element used as a key to refer to this element and as a
/// label.

/// @var std::string EW_Form3_Volume::file
/// The canonical name of the file containing the data for this element.
/// An empty string indicates that there is no associated file.

/// @fn inline bool EW_Form3_Volume::operator==(const EW_Form3_Volume &a) const
/// This compares member by member.

/// @fn inline bool EW_Form3_Volume::operator!=(const EW_Form3_Volume &a) const
/// This compares member by member.

// The rest of this file is private.

inline bool
EW_Form3_Volume::operator==(const EW_Form3_Volume &a) const
{
  return id == a.id && file == a.file;
}

inline bool
EW_Form3_Volume::operator!=(const EW_Form3_Volume &a) const
{
  return !operator==(a);
}

#endif
