// wdkg 2008-2010

#ifndef EW_SURFACE3_H_INCLUDED
#define EW_SURFACE3_H_INCLUDED 1

#include <vector>

// XXX auto try xxx.gz first, any error -> xxx
// XXX ? if filename to write ends in gz, compress automatically, otherwise not
// XXX support texture maps, normals
//XXX more specific error info (line)
//XXX mmap (check ws terminated, o/w copy and add z) or incremental read
// to use readahead (profile)
//XXX check not large file
//XXX more specific error classes (no file, io error, format error)
//XXX read compressed
//XXX ? mmap cached format, check for *.cache
//XXX other formats (ply), more general obj support, cache format
//XXX ?auto-detect format, ?auto cache support
//XXX other formats (ply), more general obj support, cache format
// XXX ? uniquify vertices
// XXX silhouette, flat, wireframe, text map, normals, ...
// XXX write surface
// XXX obj: relative index, vn, vt, usemtl (or mtl file)
// more specific error info (line)
// check not large file

/// Triangulated Surface

/// EW_Surface3 encapsulates data for triangulated surfaces in
/// @f$\mathbb{R}^3@f$.
///
/// EW_Surface3 is a data structure that supports default construction, copy
/// construction, assignment and equality comparison.
///
/// EW_Surface3 is declared in @c ew/surface3.h.
///
/// @anchor duplicated_points
/// The data consists of a list of points and a list of faces.
/// If a point is shared by several faces, the surface normal will be avaraged
/// while rendering as a smooth surface.
/// It this is not the intent, the point should be duplicated as several
/// entries in the list of points, all with the same coordinates.

class EW_Surface3 {
public:
  std::vector<float> points;
  std::vector<int> faces;
  void reset();
  void read_file(const char *file);
  inline void swap(EW_Surface3 &s);
  bool operator==(const EW_Surface3 &a) const;
  inline bool operator!=(const EW_Surface3 &a) const;
};

inline void swap(EW_Surface3 &s1, EW_Surface3 &s2);

/// @var std::vector<float> EW_Surface3::points
/// This vector has size 3 times the number of points.
/// Each consecutive triple defines a point.

/// @var std::vector<int> EW_Surface3::faces
/// This vector has size 3 times the number of faces
/// The elements are indices of the points list, and are in the range
/// [0, number of points).
/// Each consequtive triple defines a face.

/// @fn void EW_Surface3::reset()
/// This frees any current data and sets the object to the initial state.

/// @fn void EW_Surface3::read_file(const char *file)
/// This reads the surface contained in the @a file.
/// The following surface formats are supported:
///   - obj format, for example @ref tetrahedron.obj.
///     The identify of the obj vertices is preserved, so an obj with
///     duplicated vertices, as well as being bloated, will disable smooth
///     rendering.
///     See @ref duplicated_points "these comments about duplicated points".
///     Only these types of record are currently supported:
///     - v
///     - f with non-negative arguments
///     Normals and texture mapping are currently ignored.
///
/// @param file
/// The name of the file to read.
/// @exception std::bad_alloc
/// @exception EW_Surface3_IOError

/// @fn inline void EW_Surface3::swap(EW_Surface3 &s)
/// Swap data between 2 surfaces without copying.
/// @param s
///   The surface to swap with @c this.

/// @fn inline bool EW_Surface3::operator==(const EW_Surface3 &a) const
/// This compares member by member.

/// @fn inline bool EW_Surface3::operator!=(const EW_Surface3 &a) const
/// This compares member by member.

// The rest of this file is private.

inline void
EW_Surface3::swap(EW_Surface3 &s)
{
  points.swap(s.points);
  faces.swap(s.faces);
}

inline void
swap(EW_Surface3 &s1, EW_Surface3 &s2)
{
  s1.swap(s2);
}

inline bool
EW_Surface3::operator!=(const EW_Surface3 &a) const
{
  return !operator==(a);
}

#endif
