// wdkg 2008-2010

#ifndef EW_VIEW3_CURVE_H_INCLUDED
#define EW_VIEW3_CURVE_H_INCLUDED 1

#include "ew/dataflow_network.h"
#include "ew/view3_item.h"

class EW_Bbox3;
class EW_Dataflow_Curve3;
class EW_View3_Widget;

/// Curve Item

/// EW_View3_Curve is an EW_View3_Widget item to display a
/// EW_Dataflow_Curve3, a piecewise-linear curve.
///
/// EW_View3_Curve is a class without assignment or comparison.
/// There are private member variables.
///
/// EW_View3_Curve is declared in @c ew/view3_curve.h.

class EW_View3_Curve : public EW_View3_Item {
public:
  explicit EW_View3_Curve(EW_View3_Widget *view);
  void set_curve(const EW_Dataflow_Curve3 *cur);
  inline const EW_Dataflow_Curve3 *get_curve() const;
  virtual const EW_Bbox3 *get_bbox();
  void set_color(const unsigned char *rgb);
  inline const unsigned char *get_color() const;
  void set_linewidth(double w);
  inline double get_linewidth() const;
private:
  virtual ~EW_View3_Curve();
  virtual bool check_dataflow();
  virtual void clear_highlight();
  virtual void prepare();
  virtual void render();
  const EW_Dataflow_Curve3 *curve;
  EW_Dataflow_Network::cycle_t curve_checked_version;
  unsigned char color[3];
  double linewidth;
  unsigned int cur_dlist;
  unsigned int dlist;
  const bool DbgOn;
};

/// @fn explicit EW_View3_Curve::EW_View3_Curve(EW_View3_Widget *view)
/// This creates an item.
/// @param view
///   The view widget the item should belong to.

/// @fn EW_View3_Curve::~EW_View3_Curve()
/// This destroys the item.

/// @fn void EW_View3_Curve::set_curve(const EW_Dataflow_Curve3 *cur)
/// @param cur
///   A pointer to the new curve node to display, or 0.

/// @fn inline const EW_Dataflow_Curve3 *EW_View3_Curve::get_curve(
///  ) const
/// @return
///   A pointer to the current curve node being displayed, or 0.

/// @fn void EW_View3_Curve::set_color(const unsigned char *rgb)
/// @param[in] rgb
///   This points to an array of size 3 containing the new colour for the
///   curve.

/// @fn const unsigned char *EW_View3_Curve::get_color() const
/// @return
///   A pointer to the current colour for the curve.

//XXX linewidth
// The rest of this file is private.

inline const EW_Dataflow_Curve3 *
EW_View3_Curve::get_curve() const
{
  return curve;
}

inline const unsigned char *
EW_View3_Curve::get_color() const
{
  return color;
}

inline double
EW_View3_Curve::get_linewidth() const
{
  return linewidth;
}

#endif
