// wdkg 2008-2010

#ifndef EW_VIEW3_IMAGE_H_INCLUDED
#define EW_VIEW3_IMAGE_H_INCLUDED 1

#include "ew/bbox3.h"
#include "ew/dataflow_network.h"
#include "ew/view3_item.h"

class EW_Dataflow_Image3;
class EW_View3_Widget;

// XXXexplicit about regular/non regular
// XXX implement
/// Image Item

/// EW_View3_Image is an EW_View3_Widget item to display a EW_Dataflow_Image3,
/// a raster image projected onto an array of points.
///
/// EW_View3_Image is a class without assignment or comparison.
/// There are private member variables.
///
/// EW_View3_Image is declared in @c ew/view3_image.h.
///
/// This is barely implemented, and currently just displays a solid rectangle,
/// on a 2x2x1 rectangular array of points.
///
/// The default colour is [128, 128, 128].

class EW_View3_Image : public EW_View3_Item {
public:
  explicit EW_View3_Image(EW_View3_Widget *view);
  void set_image(const EW_Dataflow_Image3 *im);
  inline const EW_Dataflow_Image3 *get_image() const;
  virtual const EW_Bbox3 *get_bbox();
  void set_color(const unsigned char *rgb);
  inline const unsigned char *get_color() const;
private:
  virtual ~EW_View3_Image();
  virtual bool check_dataflow();
  virtual void clear_highlight();
  virtual void prepare();
  virtual void render();
  const EW_Dataflow_Image3 *image;
  EW_Dataflow_Network::cycle_t image_checked_version;
  unsigned char color[3];
  unsigned int dlist;
// XXX temporary
  EW_Bbox3 bbox;
  const bool DbgOn;
};

/// @fn explicit EW_View3_Image::EW_View3_Image(EW_View3_Widget *view)
/// This creates an item.
/// @param view
///   The view widget the item should belong to.

/// @fn EW_View3_Image::~EW_View3_Image()
/// This destroys the item.

/// @fn void EW_View3_Image::set_image(const EW_Dataflow_Image3 *im)
/// @param im
///   A pointer to the new image node to display, or 0.

/// @fn inline const EW_Dataflow_Image3 *EW_View3_Image::get_image() const
/// @return
///   A pointer to the current image node being displayed, or 0.

/// @fn void EW_View3_Image::set_color(const unsigned char *rgb)
/// @param[in] rgb
///   This points to an array of size 3 containing the new colour for the
///   image.

/// @fn const unsigned char *EW_View3_Image::get_color() const
/// @return
///   A pointer to the current colour for the image.

// The rest of this file is private.

inline const EW_Dataflow_Image3 *
EW_View3_Image::get_image() const
{
  return image;
}

inline const unsigned char *
EW_View3_Image::get_color() const
{
  return color;
}

#endif
