// wdkg 2008-2010

#ifndef EW_VIEW3_ITEM_H_INCLUDED
#define EW_VIEW3_ITEM_H_INCLUDED 1

#include <string>

#include "ew/view3_widget.h"

class EW_Bbox3;
class EW_Dataflow_Network;

/// Item Base Class

/// EW_View3_Item is the base class for items displayed in EW_View_widget.
///
/// EW_View3_Item is a class without assignment or comparison.
/// There are private member variables.
///
/// Items must be created with @c new and then belong to the EW_View3_Widget.
/// They must only be deleted indirectly with the #destroy method.
/// Remaining items will be automatically deleted when the EW_View3_Widget is
/// destroyed.
///
/// EW_View3_Item is declared in @c ew/view3_item.h.

class EW_View3_Item {
public:
  inline bool get_state() const;
  void set_state(bool st);
  virtual const EW_Bbox3 *get_bbox() = 0;
  inline int get_index() const;
  inline void destroy();
  EW_View3_Widget *const view;
  EW_Dataflow_Network *const network;
protected:
  EW_View3_Item(EW_View3_Widget *view, const char *dbg_final_class_name);
  virtual ~EW_View3_Item();
  inline void redraw_view_later();
  inline void set_this_highlight_item();
  bool prepared;
private:
  friend class EW_View3_Widget;
// EW_View3_Widget
  virtual bool check_dataflow() = 0;
  virtual void clear_highlight() = 0;
  virtual void prepare() = 0;
  virtual void render() = 0;
  inline bool get_prepared() const;
  inline void set_index(int n);
// private
  bool state;
  int index;
protected:
  const EWDebug &Dbg;
private:
  const bool DbgOnItem;
  const int DbgN;
  std::string DbgStr;
public:
// private to library
  const char *const DbgId;
private:
// undefined
  EW_View3_Item(const EW_View3_Item &rhs);
  EW_View3_Item &operator=(const EW_View3_Item &rhs);
};

/// @fn EW_View3_Item::destroy()
/// This removes the item from the widget and destroys it.

/// @fn bool EW_View3_Item::get_state() const
/// An item is displayed iff the state is @c true.
/// @return
///   The current state of the item.

/// @fn void EW_View3_Item::set_state(bool st)
/// @param st
///   The new state of the item.

/// @fn const EW_Bbox3 *EW_View3_Item::get_bbox()
/// @return
///   A pointer to a bounding box for the item as currently being displayed.

/// @fn int EW_View3_Item::get_index() const
/// @return
///   The current index of the item in the widget's item list.

/// @var EW_View3_Widget *const EW_View3_Item::view
/// This points to the EW_View3_Widget the item belongs to.

/// @var EW_Dataflow_Network *const EW_View3_Item::network
/// This points to the EW_Dataflow_Network the view widget is associated with.

// The rest of this file is private.

inline void
EW_View3_Item::redraw_view_later()
{
  view->redraw_later();
}

inline void
EW_View3_Item::set_this_highlight_item()
{
  view->set_highlight_item(this);
}

inline bool
EW_View3_Item::get_state() const
{
  return state;
}

inline bool
EW_View3_Item::get_prepared() const
{
  return prepared;
}

inline int
EW_View3_Item::get_index() const
{
  return index;
}

inline void
EW_View3_Item::destroy()
{
  delete this;
}

inline void
EW_View3_Item::set_index(int n)
{
  index = n;
}

#endif
