// wdkg 2009-2010

#ifndef EW_VIEW3_LANDMARKS_H_INCLUDED
#define EW_VIEW3_LANDMARKS_H_INCLUDED 1

#include "ew/dataflow_network.h"
#include "ew/view3_item.h"

class EW_Bbox3;
class EW_Dataflow_Form3;
class EW_View3_Widget;

//XXX ?later highlight by id
/// Landmarks Item

/// EW_View3_Landmarks is an EW_View3_Widget item to display the landmarks in a
/// EW_Dataflow_Form3.
///
/// EW_View3_Landmarks is a class without assignment or comparison.
/// There are private member variables.
///
/// EW_View3_Landmarks is declared in @c ew/view3_landmarks.h.

//XXX labels, select color, select n, radius, relax, symbol(relaxdims)
//XXX settings(lmk-type)
class EW_View3_Landmarks : public EW_View3_Item {
public:
  explicit EW_View3_Landmarks(EW_View3_Widget *view);
  void set_form(const EW_Dataflow_Form3 *frm);
  inline const EW_Dataflow_Form3 *get_form() const;
  virtual const EW_Bbox3 *get_bbox();
  void set_color(const unsigned char *rgb);
  inline const unsigned char *get_color() const;
  void set_highlight(int ps, int i);
  inline int get_highlight_ps() const;
  inline int get_highlight_i() const;
  enum SymbolType {
/// A cross symbol.
    SYMBOL_CROSS,
/// A circle symbol.
    SYMBOL_CIRCLE,
/// A dot symbol.
    SYMBOL_DOT
  };
  static const int N_SYMBOLS = 3;
  static const char *const symbol_names[];
  void set_symbol(int sym);
  inline int get_symbol() const;
private:
  virtual ~EW_View3_Landmarks();
  virtual bool check_dataflow();
  virtual void clear_highlight();
  virtual void prepare();
  virtual void render();
  const EW_Dataflow_Form3 *form;
  EW_Dataflow_Network::cycle_t form_checked_version;
  int symbol;
  unsigned char color[3];
  unsigned int dlist;
  int highlight_ps;
  int highlight_i;
  const bool DbgOn;
};

/// @fn explicit EW_View3_Landmarks::EW_View3_Landmarks(EW_View3_Widget *view)
/// This creates an item.
/// @param view
///   The view widget the item should belong to.

/// @fn EW_View3_Landmarks::~EW_View3_Landmarks()
/// This destroys the item.

/// @fn void EW_View3_Landmarks::set_form(const EW_Dataflow_Form3 *frm)
/// @param frm
///   A pointer to the new form node to display, or 0.

/// @fn inline const EW_Dataflow_Form3 *EW_View3_Landmarks::get_form() const
/// @return
///   A pointer to the current form node being displayed, or 0.

/// @fn void EW_View3_Landmarks::set_color(const unsigned char *rgb)
/// @param[in] rgb
///   This points to an array of size 3 containing the new colour for the
///   landmark symbols.

/// @fn const unsigned char *EW_View3_Landmarks::get_color() const
/// @return
///   A pointer to the current colour for the landmark symbols.

/// @enum EW_View3_Landmarks::SymbolType
/// These are the possible values for #set_symbol and #get_symbol.
/// The first, with value zero, is #SYMBOL_CROSS.

/// @var const int EW_View3_Landmarks::N_SYMBOLS
/// They number of possible #SymbolType values.

/// @var static const char *const EW_View3_Landmarks::symbol_names[]
/// They names of the #SymbolType choices.

/// @fn void EW_View3_Landmarks::set_symbol(int sym);
/// @param sym
///   The new landmark symbol.

/// @fn int EW_View3_Landmarks::get_symbol() const;
/// @return
///   The current landmark symbol.

/// @fn void EW_View3_Landmarks::set_highlight(int ps, int i);
/// @param ps
///   The pointset of the landmark to highlight.
/// @param i
///   The index of the landmark within the pointset of the landmark to
///   highlight.

/// @fn inline int EW_View3_Landmarks::get_highlight_ps() const;
/// @return
///   The currently highlighted pointset index, or -1.

/// @fn inline int EW_View3_Landmarks::get_highlight_i() const;
/// @return
///   The currently highlighted landmark index within the pointset, or -1.

// The rest of this file is private.

inline const EW_Dataflow_Form3 *
EW_View3_Landmarks::get_form() const
{
  return form;
}

inline const unsigned char *
EW_View3_Landmarks::get_color() const
{
  return color;
}

inline int
EW_View3_Landmarks::get_symbol() const
{
  return symbol;
}

inline int
EW_View3_Landmarks::get_highlight_ps() const
{
  return highlight_ps;
}

inline int
EW_View3_Landmarks::get_highlight_i() const
{
  return highlight_i;
}

#endif
