// wdkg 2008-2010

#ifndef EW_VIEW3_SURFACE_H_INCLUDED
#define EW_VIEW3_SURFACE_H_INCLUDED 1

#include "ew/dataflow_network.h"
#include "ew/view3_item.h"

class EW_Bbox3;
class EW_Dataflow_Surface3;
class EW_View3_Widget;

/// Surface Item

/// EW_View3_Surface is an EW_View3_Widget item to display a
/// EW_Dataflow_Surface3, a triangulated surface.
///
/// EW_View3_Surface is a class without assignment or comparison.
/// There are private member variables.
///
/// EW_View3_Surface is declared in @c ew/view3_surface.h.

class EW_View3_Surface : public EW_View3_Item {
public:
  explicit EW_View3_Surface(EW_View3_Widget *view);
  void set_surface(const EW_Dataflow_Surface3 *sur);
  inline const EW_Dataflow_Surface3 *get_surface() const;
  virtual const EW_Bbox3 *get_bbox();
  void set_front_color(const unsigned char *rgb);
  inline const unsigned char *get_front_color() const;
  void set_back_color(const unsigned char *rgb);
  inline const unsigned char *get_back_color() const;
//XXX not currently supported
  enum RenderModeType {
    MODE_SMOOTH,
    MODE_SILHOUETTE,
    MODE_WIREFRAME
  };
  static const int N_MODES = 3;
  static const char *const mode_names[];
  void set_render_mode(int m);
  inline int get_render_mode() const;
  void set_wireframe_linewidth(double w);
  inline double get_wireframe_linewidth() const;
private:
  virtual ~EW_View3_Surface();
  virtual bool check_dataflow();
  virtual void clear_highlight();
  virtual void prepare();
  virtual void render();
  const EW_Dataflow_Surface3 *surface;
  EW_Dataflow_Network::cycle_t surface_checked_version;
  int render_mode;
  unsigned char front_color[3];
  unsigned char back_color[3];
  double wireframe_linewidth;
  unsigned int sur_dlist;
  unsigned int dlist;
  const bool DbgOn;
};

/// @fn explicit EW_View3_Surface::EW_View3_Surface(EW_View3_Widget *view)
/// This creates an item.
/// @param view
///   The view widget the item should belong to.

/// @fn EW_View3_Surface::~EW_View3_Surface()
/// This destroys the item.

/// @fn void EW_View3_Surface::set_surface(const EW_Dataflow_Surface3 *sur)
/// @param sur
///   A pointer to the new surface node to display, or 0.

/// @fn inline const EW_Dataflow_Surface3 *EW_View3_Surface::get_surface(
///  ) const
/// @return
///   A pointer to the current surface node being displayed, or 0.

/// @fn void EW_View3_Surface::set_front_color(const unsigned char *rgb)
/// @param[in] rgb
///   This points to an array of size 3 containing the new colour for the front
///   of the surface.

/// @fn const unsigned char *EW_View3_Surface::get_front_color() const
/// @return
///   A pointer to the current colour for the front of the surface.

/// @fn void EW_View3_Surface::set_back_color(const unsigned char *rgb)
/// @param[in] rgb
///   This points to an array of size 3 containing the new colour for the back
///   of the surface.

/// @fn const unsigned char *EW_View3_Surface::get_back_color() const
/// @return
///   A pointer to the current colour for the back of the surface.

// The rest of this file is private.

inline const EW_Dataflow_Surface3 *
EW_View3_Surface::get_surface() const
{
  return surface;
}

inline const unsigned char *
EW_View3_Surface::get_front_color() const
{
  return front_color;
}

inline const unsigned char *
EW_View3_Surface::get_back_color() const
{
  return back_color;
}

inline double
EW_View3_Surface::get_wireframe_linewidth() const
{
  return wireframe_linewidth;
}

inline int
EW_View3_Surface::get_render_mode() const
{
  return render_mode;
}

#endif
